SystemOrganization addCategory: #SMark!
SystemOrganization addCategory: #'SMark-Tests'!
SystemOrganization addCategory: #'SMark-Benchmarks-Classic'!
SystemOrganization addCategory: #'SMark-Benchmarks-DeltaBlue'!
SystemOrganization addCategory: #'SMark-Benchmarks-Richards'!
SystemOrganization addCategory: #'SMark-Benchmarks-SOM'!
SystemOrganization addCategory: #'SMark-Benchmarks-GraphSearch'!
SystemOrganization addCategory: #'SMark-Benchmarks-Json'!
SystemOrganization addCategory: #'SMark-Benchmarks-RSqueak'!
SystemOrganization addCategory: #'SMark-Benchmarks-SqueakJS'!
SystemOrganization addCategory: #'SMark-Benchmarks-NBody'!
SystemOrganization addCategory: #'SMark-GUI'!

!Time class methodsFor: '*SMark-squeak protocol' stamp: 'sm 5/25/2011 14:26'!
milliseconds: currentTime since: lastTime
	"Answer the elapsed time since last recorded in milliseconds.
	Compensate for rollover."

	| delta |
	delta := currentTime - lastTime.
	^ delta < 0
		ifTrue: [SmallInteger maxVal + delta]
		ifFalse: [delta]
! !

OrderedCollection subclass: #DBPlan
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-DeltaBlue'!

!DBPlan commentStamp: '<historical>' prior: 0!
A Plan is an ordered list of constraints to be executed in sequence to resatisfy all currently satisfiable constraints in the face of one or more changing inputs.!

!DBPlan methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:49'!
execute
	"Execute my constraints in order."

	self do: [: c | c execute].! !

Class subclass: #SMarkCompilerTargetClass
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Classic'!

!SMarkCompilerTargetClass methodsFor: 'compiling' stamp: ''!
acceptsLoggingOfCompilation
	"Answer whether the receiver's method submisions and class defintions should be logged to the changes file and to the current change set."

	^ false! !

!SMarkCompilerTargetClass methodsFor: 'accessing method dictionary' stamp: 'StefanMarr 2/13/2013 00:43'!
addAndClassifySelector: selector withMethod: compiledMethod inProtocol: aCategory notifying: requestor
	| priorMethodOrNil oldProtocol newProtocol |
	priorMethodOrNil := self compiledMethodAt: selector ifAbsent: [nil].
	self addSelectorSilently: selector withMethod: compiledMethod.
	oldProtocol := self organization categoryOfElement: selector.

	self classify: selector under: aCategory.

	newProtocol := self organization categoryOfElement: selector.! !

!SMarkCompilerTargetClass methodsFor: 'compatibility Pharo 1.4 vs. 2.0' stamp: 'StefanMarr 2/13/2013 00:42'!
classify: selector under: aCategory
	"Use image specific approach Pharo 1.4 vs. Pharo 2.0"
	| classify |
	classify := [self organization classify: selector under: aCategory].

	"Pharo 1.4"
	(Smalltalk at: #SystemChangeNotifier ifPresent: [:cls |
		^ cls uniqueInstance doSilently: classify ]).
	
	"Pharo 2.0"
	^ SystemAnnouncer uniqueInstance suspendAllWhile: classify.! !

!SMarkCompilerTargetClass methodsFor: 'initialization' stamp: 'sm 1/17/2011 09:38'!
initialize
	super initialize.
	self methodDictionary: MethodDictionary new.! !

TestCase subclass: #SMarkHarnessArgumentParserTest
	instanceVariableNames: 'parser'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkHarnessArgumentParserTest methodsFor: 'running' stamp: ''!
setUp
	parser := SMarkHarnessArgumentParser new.
	parser harness: SMarkHarness.! !

!SMarkHarnessArgumentParserTest methodsFor: 'tests' stamp: ''!
testParseArgumentsOnlyBench
	| arguments runnerAndReporter runner reporter runOnly |
	arguments := {#ignored. 'SMarkTestSuite.benchA' }.
	runnerAndReporter := parser parse: arguments.
	
	runner   := runnerAndReporter first.
	reporter := runnerAndReporter second.
	
	self assert: runner notNil.
	self assert: reporter notNil.
	
	self assert: (runner suite isKindOf: SMarkTestSuite).
	runOnly := runner suite selectedBenchmarks.
	self assert: 1 equals: runOnly size.
	self assert: (runOnly includes: #benchA).
! !

!SMarkHarnessArgumentParserTest methodsFor: 'tests' stamp: ''!
testParseArgumentsOnlySuite
	| arguments runnerAndReporter runner reporter |
	arguments := {#ignored. 'SMarkTestSuite' }.
	runnerAndReporter := parser parse: arguments.
	
	runner   := runnerAndReporter first.
	reporter := runnerAndReporter second.
	
	self assert: runner notNil.
	self assert: reporter notNil.
	
	self assert: (runner suite isKindOf: SMarkTestSuite).
	self assert: runner suite selectedBenchmarks isNil.
! !

!SMarkHarnessArgumentParserTest methodsFor: 'tests' stamp: ''!
testParseArgumentsReporterSuiteIterations
	| arguments runnerAndReporter runner reporter |
	arguments := {#ignored. 'SMarkSimpleStatisticsReporter'. 'SMarkTestSuite'. '13'. }.
	runnerAndReporter := parser parse: arguments.
	
	runner   := runnerAndReporter first.
	reporter := runnerAndReporter second.
	
	self assert: runner notNil.
	self assert: reporter notNil.
	
	self assert: (reporter isKindOf: SMarkSimpleStatisticsReporter).
	self assert: (runner suite isKindOf: SMarkTestSuite).
	self assert: runner suite selectedBenchmarks isNil.
	
	self assert: 13 equals: runner iterations.	! !

!SMarkHarnessArgumentParserTest methodsFor: 'tests' stamp: ''!
testParseArgumentsRunnerReporterSuiteIterations
	| arguments runnerAndReporter runner reporter |
	arguments := {#ignored. 'SMarkTestRunner'. 'SMarkSimpleStatisticsReporter'. 'SMarkTestSuite'. '13'. }.
	runnerAndReporter := parser parse: arguments.
	
	runner   := runnerAndReporter first.
	reporter := runnerAndReporter second.
	
	self assert: runner notNil.
	self assert: reporter notNil.
	
	self assert: (runner isKindOf: SMarkTestRunner).
	self assert: (reporter isKindOf: SMarkSimpleStatisticsReporter).
	self assert: (runner suite isKindOf: SMarkTestSuite).
	self assert: runner suite selectedBenchmarks isNil.
	
	self assert: 13 equals: runner iterations.! !

!SMarkHarnessArgumentParserTest methodsFor: 'tests' stamp: ''!
testParseArgumentsRunnerSuiteIterations
	| arguments runnerAndReporter runner reporter |
	arguments := {#ignored. 'SMarkTestRunner'. 'SMarkTestSuite'. '13'. }.
	runnerAndReporter := parser parse: arguments.
	
	runner   := runnerAndReporter first.
	reporter := runnerAndReporter second.
	
	self assert: runner notNil.
	self assert: reporter notNil.
	
	self assert: (runner isKindOf: SMarkTestRunner).
	self assert: (runner suite isKindOf: SMarkTestSuite).
	self assert: runner suite selectedBenchmarks isNil.
	
	self assert: 13 equals: runner iterations.! !

!SMarkHarnessArgumentParserTest methodsFor: 'tests' stamp: ''!
testParseArgumentsSuiteAndIterations
	| arguments runnerAndReporter runner reporter |
	arguments := {#ignored. 'SMarkTestSuite'. '13' }.
	runnerAndReporter := parser parse: arguments.
	
	runner   := runnerAndReporter first.
	reporter := runnerAndReporter second.
	
	self assert: runner notNil.
	self assert: reporter notNil.
	
	self assert: (runner suite isKindOf: SMarkTestSuite).
	self assert: runner suite selectedBenchmarks isNil.
	
	self assert: 13 equals: runner iterations.! !

!SMarkHarnessArgumentParserTest methodsFor: 'tests' stamp: ''!
testParseArgumentsSuiteIterationsProcesses
	| arguments runnerAndReporter runner reporter |
	arguments := {#ignored. 'SMarkTestSuite'. '13'. '91' }.
	runnerAndReporter := parser parse: arguments.
	
	runner   := runnerAndReporter first.
	reporter := runnerAndReporter second.
	
	self assert: runner notNil.
	self assert: reporter notNil.
	
	self assert: (runner suite isKindOf: SMarkTestSuite).
	self assert: runner suite selectedBenchmarks isNil.
	
	self assert: 13 equals: runner iterations.
	self assert: 91 equals: runner processes.
! !

!SMarkHarnessArgumentParserTest methodsFor: 'tests' stamp: ''!
testParseArgumentsSuiteIterationsProcessesProblemSize
	| arguments runnerAndReporter runner reporter |
	arguments := {#ignored. 'SMarkTestSuite'. '13'. '91'. 'SSS' }.
	runnerAndReporter := parser parse: arguments.
	
	runner   := runnerAndReporter first.
	reporter := runnerAndReporter second.
	
	self assert: runner notNil.
	self assert: reporter notNil.
	
	self assert: (runner suite isKindOf: SMarkTestSuite).
	self assert: runner suite selectedBenchmarks isNil.
	
	self assert: 13 equals: runner iterations.
	self assert: 91 equals: runner processes.
	self assert: 'SSS' equals: runner problemSize.
! !

!SMarkHarnessArgumentParserTest methodsFor: 'tests' stamp: ''!
testParseBenchmarkOrSuite
	| example1 example2 example3 example4 example5 |
	example1 := 'String>>findTokens:'.
	example2 := 'SMarkTestSuite'.
	example3 := { SMarkTestSuite. #benchA }.
	example4 := SMarkTestSuite.
	example5 := 'String.findTokens:'. ". (dot) notation is required to avoid quoting in bash"
	
	self assert: { String. #findTokens: }    equals: (parser parseBenchmarkOrSuite: example1).
	self assert: SMarkTestSuite              equals: (parser parseBenchmarkOrSuite: example2).
	self assert: { SMarkTestSuite. #benchA } equals: (parser parseBenchmarkOrSuite: example3).
	self assert: SMarkTestSuite              equals: (parser parseBenchmarkOrSuite: example4).
	self assert: { String. #findTokens: }    equals: (parser parseBenchmarkOrSuite: example5).
	
	"What to do when the input is not a symbol, array, or class?"
	
	"What to do when the class does not exist"
	self should: [parser parseBenchmarkOrSuite: 'FooBarClassThatDoesNotExist'] raise: Error.! !

TestCase subclass: #SMarkHarnessClassTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkHarnessClassTest methodsFor: 'tests' stamp: ''!
testShouldShowUsage
	| example1 example2 example3 example4 |
	example1 := { #ignored. '--help' }.
	example2 := { #ignored. }.
	example3 := { #ignored. '-?'}.
	example4 := { #ignored. 1. 2. }.
	
	self assert: (SMarkHarness shouldShowUsage: example1).
	self assert: (SMarkHarness shouldShowUsage: example2).
	self assert: (SMarkHarness shouldShowUsage: example3).
	self deny:   (SMarkHarness shouldShowUsage: example4).
! !

TestCase subclass: #SMarkReporterTest
	instanceVariableNames: 'runner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkReporterTest methodsFor: 'running' stamp: ''!
setUp
	runner := SMarkTestRunnerSuiteForAutosizing run.! !

!SMarkReporterTest methodsFor: 'compatibility Pharo 1.4 vs. 2.0' stamp: 'StefanMarr 2/13/2013 00:58'!
string: str includes: subStr
	"To be compatible with Pharo 1.4 and 2.0"
	(str respondsTo: #includesSubstring:)
		ifTrue:  [ ^ str includesSubstring: subStr ] "2.0"
		ifFalse: [ ^ str includesSubString: subStr ] "1.4"! !

!SMarkReporterTest methodsFor: 'tests' stamp: 'StefanMarr 2/13/2013 00:56'!
testContentOfStream
	"Hm, how to test that robustly?"
	| reporter stream out |
	stream := TextStream on: String new.
	reporter := SMarkReporter reportFor: runner on: stream.
	
	out := stream contents.
	
	self assert: (self string: out includes: 'Small').
	self assert: (self string: out includes: 'Big').! !

!SMarkReporterTest methodsFor: 'tests' stamp: 'StefanMarr 2/13/2013 00:55'!
testConvenienceReportOnRunner
	| stream out |
	stream := TextStream on: String new.
	
	"That should just work, cannot really check whether it is displayed anywhere."
	SMarkTestRunnerSuiteForAutosizing run report.
	
	"Check whether it reported as expected"
	SMarkTestRunnerSuiteForAutosizing run reportOn: stream.
	
	out := stream contents.
	
	self assert: (self string: out includes: 'Small').
	self assert: (self string: out includes: 'Big').! !

TestCase subclass: #SMarkResultTest
	instanceVariableNames: 'result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkResultTest commentStamp: 'StefanMarr 3/18/2011 23:29' prior: 0!
This test specifies the data stored in a result object.
The result object does not actually provide behavior on its own.!

!SMarkResultTest methodsFor: 'running' stamp: ''!
setUp
	| runner |
	runner := SMarkTestRunner execute: SMarkTestRunnerSuite new.
	result := (runner results at: #DoNothing) at: 1.! !

!SMarkResultTest methodsFor: 'tests' stamp: ''!
testBenchmarkName
	self assert: (result benchmarkName isKindOf: String)! !

!SMarkResultTest methodsFor: 'tests' stamp: ''!
testCriteria
	self assert: (result criteria respondsTo: #at:).
	self assert: (result criteria size > 0).! !

!SMarkResultTest methodsFor: 'tests' stamp: ''!
testSuite
	self assert: result suite notNil! !

!SMarkResultTest methodsFor: 'tests' stamp: ''!
testTotal
	self assert: (result total isKindOf: Number)! !

TestCase subclass: #SMarkRunnerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

SMarkRunnerTest subclass: #SMarkAutosizeRunnerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkAutosizeRunnerTest class methodsFor: 'as yet unclassified' stamp: ''!
shouldInheritSelectors
	^ true! !

!SMarkAutosizeRunnerTest methodsFor: 'helper' stamp: ''!
runnerClass
	^ SMarkTestAutosizeRunner! !

!SMarkAutosizeRunnerTest methodsFor: 'tests' stamp: ''!
testAutosizing
	"Ensure that the runtime is automatically sized up to a predefined value"
	| runner results big small runTime benchmarkSum |
	
	runner := self runnerClass new.
	
	"sanity check for target time"
	"Not checked since we reduced the targetTime for testing:
	 self assert: self runnerClass defaultTargetTime equals: runner targetTime."
	self assert: (self runnerClass defaultTargetTime > 250).
	
	runner iterations: 3.
	  
	runner suite: SMarkTestRunnerSuiteForAutosizing new.
	
	"Execute the benchmarks and take the overall time it takes"
	runTime := Time millisecondClockValue.
	results := runner execute.
	runTime := Time millisecondClockValue - runTime.
	
	"There should be three benchmarks"
	self assert: 3 equals: results size.
	self assert: (results includesKey: #baseBenchmark).
	big := results at: #Big.
	small := results at: #Small.
	
	self assert: (big respondsTo: #at:).
	self assert: 3 equals: big size.
	
	self assert: (runner targetTime <= (big at: 1) total).
	self assert: (runner targetTime <= (small at: 1) total).
	
	"Calculate the sum of all measurements"
	benchmarkSum := big inject: 0 into: [:sum :result| sum + result total].
	benchmarkSum := small inject: benchmarkSum into: [:sum :result| sum + result total].
	
	"And now make sure that we not try to find the problem size for every but only the first run"
	self assert: (benchmarkSum * 2) > runTime.
	
! !

!SMarkAutosizeRunnerTest methodsFor: 'tests' stamp: ''!
testAutosizingBaseBenchmark
	"Ensure that baseBenchmarks are not delivering any data since I do not see how that could be done nicely"
	| runner results |
	runner := self runnerClass new.
	runner iterations: 3.
	runner suite: SMarkTestRunnerSuiteForAutosizing new.
	
	"Execute the benchmarks and take the overall time it takes"
	results := runner execute.
	
	"There should be two benchmarks"
	self assert: 3 equals: results size.
	self assert: 0 equals: ((results at: #baseBenchmark) at: 1) total.
	
! !

!SMarkAutosizeRunnerTest methodsFor: 'tests' stamp: ''!
testBaseBenchmark
	"Different from base test: Ensure the base benchmark is NOT executed when available"
	
	| runner |
	runner := self runnerClass execute: SMarkTestSuiteWithBaseBenchmark new.

	self deny: runner baseBenchmarkExecuted  ! !

!SMarkAutosizeRunnerTest methodsFor: 'tests' stamp: ''!
testExecute
	"This test is slightly different from the one in the base class"
	| runner results performedBenchmarks |
	runner := self runnerClass new.
	runner suite: SMarkTestRunnerSuiteForPerfromBenchmark new.
	results := runner execute.
	
	performedBenchmarks := runner performedBenchmarks.
		
	self assert: 4 equals: performedBenchmarks size.				"Different from base test"
	self deny: (performedBenchmarks includes: #baseBenchmark).	"Different from base test"
	
	self assert: (results respondsTo: #at:).
	
	"But still we get a result" "Different from base test"
	self assert: (((results at: #baseBenchmark) at: 1) isKindOf: SMarkResult).! !

!SMarkAutosizeRunnerTest methodsFor: 'tests' stamp: ''!
testIterations
	| runner |
	"This test is different then the one in the base class, it is not direct, but just looks at the results"
	
	" simple scenario should default to 1 run for now "
	runner := self runnerClass execute: SMarkTestRunnerSuite new.
	
	self assert: 1 equals: (runner results at: #NotifyRunner) size.
	
	" but we can also be more fancy and set the number of iterations "
	runner := self runnerClass new.
	runner suite: SMarkTestRunnerSuite new.
	
	"first check again that the default is as expected"
	self assert: 1 equals: runner iterations.  
	
	runner iterations: 10.
	self assert: runner iterations equals: 10.
	
	runner execute.
	
	self assert: 10 equals: (runner results at: #NotifyRunner) size.! !

!SMarkRunnerTest methodsFor: 'helper' stamp: ''!
runnerClass
	^ SMarkTestRunner! !

!SMarkRunnerTest methodsFor: 'tests' stamp: ''!
testBaseBenchmark
	"Ensure the base benchmark is executed when available"
	
	| runner |
	runner := self runnerClass execute: SMarkTestSuiteWithBaseBenchmark new.

	self assert: runner baseBenchmarkExecuted  ! !

!SMarkRunnerTest methodsFor: 'tests' stamp: ''!
testBaseBenchmarkNone
	"Ensure that not having a base benchmark does not pose a problem"

	| runner |
	runner := self runnerClass execute: SMarkTestSuite new.

	self deny: runner baseBenchmarkExecuted  ! !

!SMarkRunnerTest methodsFor: 'tests' stamp: ''!
testBenchmarkWithMultipleCriteraTimers
	"Check whether the benchmark benchWithMultipleCriteria, really results in different unrelated timings"
	| runner result |
	runner := self runnerClass execute: SMarkTestSuite new.
	
	result := (runner results at: #WithMultipleCriteria) at: 1.
	self assert: (result isKindOf: SMarkResult).
	
	self assert: result benchmarkName equals: #WithMultipleCriteria.
	self assert: (result total >= 50).
	
	self assert: (result criteria notNil).
	self assert: (result criteria respondsTo: #at:).
	
	self assert: ((result criteria at: 't1') totalTime >= 10).
	self assert: ((result criteria at: 't2') totalTime >= 25).
	self assert: ((result criteria at: 't3') totalTime >= 50).
	! !

!SMarkRunnerTest methodsFor: 'tests' stamp: ''!
testClassExecute
	| runner |
	runner := self runnerClass execute: SMarkTestSuiteWithBaseBenchmark new.
	
	self assert: (runner isKindOf: self runnerClass).! !

!SMarkRunnerTest methodsFor: 'tests' stamp: ''!
testExecute
	"Ensure all benchmarks are run, including the baseBenchmark and the results are returned"
	| runner results performedBenchmarks |
	runner := self runnerClass new.
	runner suite: SMarkTestRunnerSuiteForPerfromBenchmark new.
	results := runner execute.
	
	performedBenchmarks := runner performedBenchmarks.
		
	self assert: 5 equals: performedBenchmarks size.
	self assert: (performedBenchmarks includes: #baseBenchmark).
	
	self assert: (results respondsTo: #at:).
	self assert: (((results at: #baseBenchmark) at: 1) isKindOf: SMarkResult).! !

!SMarkRunnerTest methodsFor: 'tests' stamp: ''!
testIterations
	| runner |
	" simple scenario should default to 1 run for now "
	runner := self runnerClass execute: SMarkTestRunnerSuite new.
	
	self assert: runner countWasExecuted equals: 1.
	
	" but we can also be more fancy and set the number of iterations "
	runner := self runnerClass new.
	runner suite: SMarkTestRunnerSuite new.
	
	"first check again that the default is as expected"
	self assert: runner iterations equals: 1.  
	
	runner iterations: 10.
	self assert: 10 equals: runner iterations.
	
	runner execute.
	
	self assert: 10 equals: runner countWasExecuted.! !

!SMarkRunnerTest methodsFor: 'tests' stamp: ''!
testPerformBenchmark
	"Ensure that performBenchmark executes all the defined #bench methods, but for instance not #Bench"
	
	| runner performedBenchmarks |
	runner := self runnerClass execute: SMarkTestRunnerSuiteForPerfromBenchmark new.
	performedBenchmarks := runner performedBenchmarks.

	self assert: (performedBenchmarks includes: #benchAfsdfsdfsdf).
	self assert: (performedBenchmarks includes: #benchBsdfasereSDSfsdfsdfs).
	self assert: (performedBenchmarks includes: #benchC).
	self assert: (performedBenchmarks includes: #benchZZ).

	self deny: (performedBenchmarks includes: #BenchShouldNotBeExecuted).! !

!SMarkRunnerTest methodsFor: 'tests' stamp: ''!
testResultStructure
	| runner results resultA resultB suite |
	suite := SMarkTestSuite new.
	runner := self runnerClass execute: suite.
	results := runner results.
	
	"Don't care whether it is a dictionary, but at least it should present us something that maps benchmark names to result objects"
	self assert: (results respondsTo: #at:).

	resultA := (results at: #A) at: 1.
	resultB := (results at: #B) at: 1.


	self assert: (resultA isKindOf: SMarkResult).
	self assert: (resultB isKindOf: SMarkResult).
	self assert: (((results at: #C) at: 1) isKindOf: SMarkResult).
	
	"Make sure some properties of the result objects is ensured"
	self assert: resultA total > 0.
	self assert: #A equals: resultA benchmarkName.
	self assert: suite equals: resultA suite.
	self assert: 1 equals: resultA criteria size.
	
	self assert: resultB total > 0.
	self assert: #B equals: resultB benchmarkName.
	self assert: suite equals: resultB suite.
	self assert: 1 equals: resultB criteria size.
	"Testing for multi-criteria results somewhere else"! !

!SMarkRunnerTest methodsFor: 'tests' stamp: ''!
testTimer
	"Simple test whether a timer is created and it is from the expected class.
	 This test is very basic, other tests like testBenchmarkWithMultipleCirteria* need to cover the rest"
	| runner timer |
	runner := self runnerClass new.
	timer := runner createTimer: #foo.

	self assert: timer name equals: #foo.
	self assert: (timer isKindOf: self runnerClass defaultTimer).  ! !

SMarkRunnerTest subclass: #SMarkWeakScalingRunnerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkWeakScalingRunnerTest class methodsFor: 'as yet unclassified' stamp: ''!
shouldInheritSelectors
	^ true! !

!SMarkWeakScalingRunnerTest methodsFor: 'helper' stamp: ''!
runnerClass
	^ SMarkTestWeakScalingRunner! !

!SMarkWeakScalingRunnerTest methodsFor: 'tests' stamp: ''!
testInnerIterations
	"Test the semantics of the inner loop parameter"
	| runner results |
	
	"Setup"
	runner := self runnerClass new.
	runner iterations: 1.
	runner innerIterations: 1.  
	runner processes: 10.
	runner suite: SMarkTestRunnerSuite new.
	results := runner execute.
	
	self assert: 10 equals: runner countWasExecuted.
	self assert: 1  equals: (results at: #NotifyRunner) size.
	
	runner := self runnerClass new.
	runner iterations: 2.
	runner innerIterations: 1.  
	runner processes: 10.
	runner suite: SMarkTestRunnerSuite new.
	results := runner execute.
	
	self assert: 20 equals: runner countWasExecuted.
	self assert: 2  equals: (results at: #NotifyRunner) size.
	
	
	runner := self runnerClass new.
	runner iterations: 2.
	runner innerIterations: 4.  
	runner processes: 10.
	runner suite: SMarkTestRunnerSuite new.
	results := runner execute.
	
	self assert: (20 * 4) equals: runner countWasExecuted.
	self assert: 2  equals: (results at: #NotifyRunner) size.
	
	runner := self runnerClass new.
	runner iterations: 4.
	runner innerIterations: 7.  
	runner processes: 10.
	runner suite: SMarkTestRunnerSuite new.
	results := runner execute.
	
	self assert: (40 * 7) equals: runner countWasExecuted.
	self assert: 4           equals: (results at: #NotifyRunner) size.! !

!SMarkWeakScalingRunnerTest methodsFor: 'tests' stamp: ''!
testWeakScaling3Processes
	"test with small number of processes and varying number of iterations"
	| runner results |
	
	"Setup for 2 iteration, and 3 process"
	runner := self runnerClass new.
	runner iterations: 2.
	runner processes: 3.
	runner suite: SMarkTestRunnerSuite new.
	results := runner execute.
	
	self assert: 6 equals: runner countWasExecuted.
	self assert: 2 equals: (results at: #NotifyRunner) size.! !

!SMarkWeakScalingRunnerTest methodsFor: 'tests' stamp: ''!
testWeakScalingFewProcesses
	"test with small number of processes and varying number of iterations"
	| runner results |
	
	"Setup for 1 iteration, and 1 process"
	runner := self runnerClass new.
	runner iterations: 1.
	runner processes: 1.
	runner suite: SMarkTestRunnerSuite new.
	results := runner execute.
	
	self assert: 1 equals: runner countWasExecuted.
	! !

!SMarkWeakScalingRunnerTest methodsFor: 'tests' stamp: ''!
testWeakScalingManyProcesses
	"test with large number of processes and varying number of iterations"
	| runner results |
	
	"Setup"
	runner := self runnerClass new.
	runner iterations: 1.
	runner processes: 10.
	runner suite: SMarkTestRunnerSuite new.
	results := runner execute.
	
	self assert: 10 equals: runner countWasExecuted.
	self assert: 1  equals: (results at: #NotifyRunner) size.
	
	runner := self runnerClass new.
	runner iterations: 2.
	runner processes: 10.
	runner suite: SMarkTestRunnerSuite new.
	results := runner execute.
	
	self assert: 20 equals: runner countWasExecuted.
	self assert: 2  equals: (results at: #NotifyRunner) size.
	
	runner := self runnerClass new.
	runner iterations: 4.
	runner processes: 10.
	runner suite: SMarkTestRunnerSuite new.
	results := runner execute.
	
	self assert: 40 equals: runner countWasExecuted.
	self assert: 4  equals: (results at: #NotifyRunner) size.! !

TestCase subclass: #SMarkSuiteTest
	instanceVariableNames: 'didSetUpBenchB didTearDownBenchC'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkSuiteTest methodsFor: 'tests' stamp: ''!
testBenchmarkNameForSelector
	| suite |
	suite := SMarkSuite new.
	
	self assert: (suite benchmarkNameForSelector: #benchSomething) equals: #Something.
	self assert: (suite benchmarkNameForSelector: #benchsomeThingElse) equals: #someThingElse.
	
	self assert: (suite benchmarkNameForSelector: #nobenchSomeThing) equals: #nobenchSomeThing.
	self assert: (suite benchmarkNameForSelector: #BenchSomeThing) equals: #BenchSomeThing.! !

!SMarkSuiteTest methodsFor: 'tests' stamp: ''!
testClassRun
	| runner |
	
	runner := SMarkTestSuite run.
	self assert: (runner isKindOf: SMarkTestSuite defaultRunner).
	self deny: runner results isNil    ! !

!SMarkSuiteTest methodsFor: 'tests' stamp: ''!
testCleanUpInstanceVariables
	| suite |
	suite := SMarkTestSuite new.
	suite runner: #dummy.
	
	self assert: suite runner	equals: #dummy.
	self assert: suite a			equals: 1.
	self assert: suite c			equals: #three.
				
	suite cleanUpInstanceVariables.
	
	self assert: suite runner	equals: #dummy.
	self assert: suite a			equals: nil.
	self assert: suite b			equals: nil.
	self assert: suite c			equals: nil.
	self assert: suite d			equals: nil.
! !

!SMarkSuiteTest methodsFor: 'tests' stamp: ''!
testCustomSetUp
	| suite runner |
	
	suite := SMarkTestSuite new.
	runner := SMarkTestRunner execute: suite.  
	
	self assert: runner hasSetUpBenchB! !

!SMarkSuiteTest methodsFor: 'tests' stamp: ''!
testCustomTearDown
	| suite runner |
	suite := SMarkTestSuite new.
	runner := SMarkTestRunner execute: suite.  
	
	self assert: runner hasTearDownBenchC.! !

!SMarkSuiteTest methodsFor: 'tests' stamp: ''!
testSetUp
	| suite runner |
	suite := SMarkTestSuite new.
	
	self assert: suite a			equals: 1.
	self assert: suite b			equals: 2.
	self assert: suite c			equals: #three.
	
	runner := SMarkTestRunner execute: suite.
	
	"Make sure they are reset after each run,
	 well we test just whether they are reset after the final run"
	self assert: suite a			equals: nil.
	self assert: suite b			equals: nil.
	self assert: suite c			equals: nil.
	self assert: suite d			equals: nil.

	"Make sure setUp was called 4 times."
	
	self assert: 4 equals: runner setUpCount.! !

!SMarkSuiteTest methodsFor: 'tests' stamp: ''!
testSetupAndTeaddownCalledForEveryBenchForEveryIteration
	| suite runner |

	"Setup needs to be executed for every benchmark on every iteration.
	 Test with iterations: 1"
	suite := SMarkTestSuite new.
	runner := SMarkTestRunner execute: suite with: 1.

	self assert: 4 equals: runner setUpCount.
	self assert: 4 equals: runner tearDownCount.
	
	"Setup needs to be executed for every benchmark on every iteration.
	 Test with iterations: 4"
	suite := SMarkTestSuite new.
	runner := SMarkTestRunner execute: suite with: 4.

	self assert: (4 * 4) equals: runner setUpCount.
	self assert: (4 * 4) equals: runner tearDownCount.
	
	"Test with iterations: 10"
	suite := SMarkTestSuite new.
	runner := SMarkTestRunner execute: suite with: 10.
	self assert: (4 * 10) equals: runner setUpCount.
	self assert: (4 * 10) equals: runner tearDownCount.! !

!SMarkSuiteTest methodsFor: 'tests' stamp: ''!
testShouldRunSelector
	| suite |
	suite := SMarkSuite new.
	
	self assert: (suite shouldRunSelector: #benchSomething).
	self assert: (suite shouldRunSelector: #benchsomeThingElse).
	
	self deny: (suite shouldRunSelector: #nobenchSomeThing).
	self deny: (suite shouldRunSelector: #BenchSomeThing).! !

!SMarkSuiteTest methodsFor: 'tests' stamp: ''!
testTearDown
	| suite runner |
	suite := SMarkTestSuite new.
	runner := SMarkTestRunner execute: suite.  
	
	self assert: 4 equals: runner tearDownCount.! !

TestCase subclass: #SMarkTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkTest commentStamp: 'StefanMarr 3/12/2011 21:57' prior: 0!
A general test to ensure the overall framework works as expected.!

!SMarkTest methodsFor: 'running' stamp: ''!
setUp
	"Necessary for Squeak 4.2 compatibility, since method anotations are not supported in the base image."
	(self respondsTo:  #timeout:) ifTrue: [
		self timeout: 300.
	].! !

!SMarkTest methodsFor: 'tests' stamp: 'StefanMarr 7/5/2013 11:16'!
testAllBenchmarksForReturningThemSelf
	"This is a regression test to ensure that the old behavior worked in the first place"
	| allSuites |
	allSuites := { SMarkCompiler. SMarkLoops }.
	
	allSuites do: [:suite |
		suite isAbstractClass ifFalse: [
			| runner results |
			Transcript show: (suite class name asString); cr.
			runner := suite run.
			results := runner results.
			self assert: (runner isKindOf: suite defaultRunner).
			self assert: 1 equals: runner iterations.
			self assert: (results size > 0).
		].
	].! !

!SMarkTest methodsFor: 'tests' stamp: 'StefanMarr 1/15/2012 20:40'!
testBasicRun
	| runner results benchResult result |
	
	runner := SMarkLoops run.
	results := runner results.
	result := (results at: #IntLoop).
	
	self assert: (result isKindOf: Collection).
	self assert: result size equals: 1.
	
	benchResult := result at: 1.
	self assert: (benchResult isKindOf: SMarkResult).
	self assert: (benchResult total notNil).
	! !

TestCase subclass: #SMarkTimerTest
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkTimerTest methodsFor: 'tests' stamp: ''!
testName
	| timer |
	timer := SMarkTimer new.
	
	self assert: timer name equals: nil.
	
	timer name: #foo.
	
	self assert: timer name equals: #foo.! !

!SMarkTimerTest methodsFor: 'tests' stamp: ''!
testReset
	| timer |
	timer := SMarkTimer new.
	
	timer start.
	(Delay forMilliseconds: 10) wait.
	timer stop.
	
	self assert: (timer totalTime >= 10).
	
	timer reset.
	
	self assert: timer totalTime equals: 0.! !

!SMarkTimerTest methodsFor: 'tests' stamp: ''!
testStartStop
	| timer |
	timer := SMarkTimer new.
	
	self assert: timer totalTime equals: 0.
	
	timer start.
	
	(Delay forMilliseconds: 10) wait.
	
	timer stop.
	
	self assert: (timer totalTime >= 10).! !

!SMarkTimerTest methodsFor: 'tests' stamp: ''!
testTotalTimeWithResume
	| timer |
	timer := SMarkTimer new.
	self assert: timer totalTime equals: 0.
	
	timer start.
	(Delay forMilliseconds: 10) wait.
	timer stop.
	
	self assert: (timer totalTime >= 10).
	
	timer start.
	(Delay forMilliseconds: 10) wait.
	timer stop.
	
	self assert: (timer totalTime >= 20).
	
	timer start.
	(Delay forMilliseconds: 10) wait.
	timer stop.
	
	self assert: (timer totalTime >= 30).! !

Object subclass: #BenchmarkRunner
	instanceVariableNames: 'categories categoriesSelected classes classIndex classesSelected resultList resultSelected failedList failedSelected errorList errorSelected lastUpdate categoryPattern classPattern numIterations doWarmup'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-GUI'!

!BenchmarkRunner commentStamp: '<historical>' prior: 0!
<lint: #ignore rule: #classNotReferenced rational: 'this view is only accessed from menus'>

!

!BenchmarkRunner class methodsFor: 'instance-creation' stamp: 'tfel 5/11/2015 17:45'!
build
	^ ToolBuilder build: self new.! !

!BenchmarkRunner class methodsFor: 'initialization' stamp: 'tfel 5/11/2015 17:45'!
initialize
	self registerInWorldMenu; registerInToolsFlap.! !

!BenchmarkRunner class methodsFor: 'instance-creation' stamp: 'tfel 5/11/2015 17:45'!
open
	^ ToolBuilder open: self new.! !

!BenchmarkRunner class methodsFor: 'initialization' stamp: 'tfel 5/11/2015 17:46'!
registerInToolsFlap
	self environment at: #Flaps ifPresent: [ :class |
		class
			registerQuad: #( TestRunner build 'SMark Runner' 'A benchmark-runner.' ) forFlapNamed: 'Tools';
			replaceToolsFlap ].! !

!BenchmarkRunner class methodsFor: 'initialization' stamp: 'tfel 5/11/2015 17:46'!
registerInWorldMenu
	self environment at: #TheWorldMenu ifPresent: [ :class |
		class registerOpenCommand: (Array 
			with: 'Benchmark Runner' 
			with: (Array
				with: self
				with: #open)) ].! !

!BenchmarkRunner class methodsFor: 'window color' stamp: 'tfel 5/11/2015 17:46'!
windowColorSpecification

	^ WindowColorSpec 
		classSymbol: self name 
		wording: 'Benchmark Runner' 
		brightColor:  Color orange
		pastelColor: (Color r: 0.65 g: 0.753 b: 0.976)
		helpMessage: 'The BenchmarkRunner tool for SMark'! !

!BenchmarkRunner methodsFor: 'accessing' stamp: 'tfel 5/11/2015 17:45'!
baseClass

	^ SMarkSuite! !

!BenchmarkRunner methodsFor: 'actions' stamp: 'tfel 5/11/2015 18:23'!
basicRunSuite: aTestSuite
	aTestSuite do: [:cls |
		cls benchmarkSelectors
			do: [:each | self runBenchmark: each inSuite: cls]
			displayingProgress: [ :test | cls name, '>>', test printString ]
			every: 0 "Update the label for all tests"
	].! !

!BenchmarkRunner methodsFor: 'accessing-classes' stamp: 'tfel 5/11/2015 17:45'!
browseClass
	(classes at: classIndex ifAbsent: [ ^ self ]) browse! !

!BenchmarkRunner methodsFor: 'private' stamp: 'tfel 5/11/2015 17:45'!
browserEnvironment
	^ Smalltalk classNamed: #BrowserEnvironment.! !

!BenchmarkRunner methodsFor: 'building' stamp: 'tfel 5/11/2015 17:45'!
buildButtonsWith: aBuilder
	^ aBuilder pluggablePanelSpec new
		model: self;
		layout: #horizontal;
		children: (self buttons collect: [ :each |
			aBuilder pluggableButtonSpec new
				model: self; 
				label: each first;
				action: each second;
				enabled: each third;
				yourself ]);
		yourself.! !

!BenchmarkRunner methodsFor: 'building' stamp: 'tfel 5/11/2015 17:45'!
buildCategoriesWith: aBuilder
	^ aBuilder pluggableMultiSelectionListSpec new
		model: self;
		list: #categoryList;
		menu: #categoryMenu:;
		getIndex: #categorySelected;
		setIndex: #categorySelected:;
		getSelectionList: #categoryAt:;
		setSelectionList: #categoryAt:put:;
		yourself.! !

!BenchmarkRunner methodsFor: 'building' stamp: 'tfel 5/11/2015 17:45'!
buildClassesWith: aBuilder
	^ aBuilder pluggableMultiSelectionListSpec new
		model: self;
		list: #classList;
		menu: #classMenu:;
		getIndex: #classSelected;
		setIndex: #classSelected:;
		getSelectionList: #classAt:;
		setSelectionList: #classAt:put:;
		yourself.! !

!BenchmarkRunner methodsFor: 'building' stamp: 'tfel 5/11/2015 17:45'!
buildErrorListWith: aBuilder
	^ aBuilder pluggableListSpec new
		model: self;
		name: 'Error List';
		list: #errorList; 
		menu: #errorMenu:;
		getIndex: #errorSelected; 
		setIndex: #errorSelected:;
		yourself.! !

!BenchmarkRunner methodsFor: 'building' stamp: 'tfel 5/11/2015 17:45'!
buildFailureListWith: aBuilder
	^ aBuilder pluggableListSpec new
		model: self;
		name: 'Failure List';
		list: #failedList; 
		menu: #failureMenu:;
		getIndex: #failedSelected; 
		setIndex: #failedSelected:;
		yourself.! !

!BenchmarkRunner methodsFor: 'building' stamp: 'tfel 5/11/2015 17:52'!
buildResultListWith: aBuilder
	^ aBuilder pluggableListSpec new
		model: self;
		name: 'Result List';
		list: #resultList; 
		menu: #resultMenu:;
		getIndex: #resultSelected; 
		setIndex: #resultSelected:;
		yourself.! !

!BenchmarkRunner methodsFor: 'building' stamp: 'tfel 5/11/2015 17:45'!
buildStatusWith: aBuilder
	^ aBuilder pluggableInputFieldSpec new
		model: self;
		menu: #statusMenu:;
		color: #statusColor;
		getText: #statusText;
		yourself.! !

!BenchmarkRunner methodsFor: 'building' stamp: 'tfel 5/11/2015 17:51'!
buildWith: aBuilder
	| window |
	window := aBuilder pluggableWindowSpec new
		model: self; label: self label; extent: self extent;
		children: (OrderedCollection new 
			add: ((self buildCategoriesWith: aBuilder)
				frame: self categoriesFrame;
				yourself);
			add: ((self buildClassesWith: aBuilder)
				frame: self classesFrame;
				yourself);
			add: ((self buildStatusWith: aBuilder)
				frame: self statusFrame;
				yourself);
			add: ((self buildResultListWith: aBuilder)
				frame: self resultListFrame;
				yourself);
			add: ((self buildFailureListWith: aBuilder)
				frame: self failureListFrame;
				yourself);
			add: ((self buildErrorListWith: aBuilder)
				frame: self errorListFrame;
				yourself);
			add: ((self buildButtonsWith: aBuilder)
				frame: self buttonsFrame;
				yourself);
			yourself);
		yourself.
	^ aBuilder build: window.! !

!BenchmarkRunner methodsFor: 'building' stamp: 'tfel 5/11/2015 17:45'!
buttonHeight
	^Preferences standardButtonFont height + 25! !

!BenchmarkRunner methodsFor: 'accessing-ui' stamp: 'tfel 5/11/2015 18:05'!
buttons
	^ #(( 'Run Selected' #runAll #hasRunnable )
		( 'Run Profiled' #runProfiled #hasRunnable )
		( 'Run Failures' #runFailures #hasFailures )
		( 'Run Errors' #runErrors #hasErrors ))! !

!BenchmarkRunner methodsFor: 'building' stamp: 'tfel 5/11/2015 17:45'!
buttonsFrame
	^LayoutFrame new
		leftFraction: 0 offset: 0;
		topFraction: 1 offset: self buttonHeight negated;
		rightFraction: 1 offset: 0;
		bottomFraction: 1 offset: 0! !

!BenchmarkRunner methodsFor: 'building' stamp: 'tfel 5/11/2015 17:45'!
categoriesFrame
	^LayoutFrame new
		leftFraction: 0 offset: 0;
		topFraction: 0 offset: 0;
		rightFraction: 0.25 offset: 0;
		bottomFraction: 1 offset: self buttonHeight negated + 4! !

!BenchmarkRunner methodsFor: 'accessing-categories' stamp: 'tfel 5/11/2015 17:45'!
categoryAt: anIndex
	^ categoriesSelected includes: (categories at: anIndex ifAbsent: [ ^ false ]).! !

!BenchmarkRunner methodsFor: 'accessing-categories' stamp: 'tfel 5/11/2015 17:45'!
categoryAt: anInteger put: aBoolean
	categoriesSelected := categoriesSelected
		perform: (aBoolean ifTrue: [ #copyWith: ] ifFalse: [ #copyWithout: ])
		with: (categories at: anInteger ifAbsent: [ ^ self ]).
	self changed: #categorySelected; updateClasses.! !

!BenchmarkRunner methodsFor: 'accessing-categories' stamp: 'tfel 5/11/2015 17:45'!
categoryList
	^ categories! !

!BenchmarkRunner methodsFor: 'accessing-categories' stamp: 'tfel 5/11/2015 17:45'!
categoryMenu: aMenu
	^ aMenu
		title: 'Categories';
		add: 'Select all' action: #selectAllCategories;
		add: 'Select inversion' action: #selectInverseCategories;
		add: 'Select none' action: #selectNoCategories;
		addLine;
		add: 'Filter...' action: #filterCategories;
		addLine;
		add: 'Refresh' action: #updateCategories;
		yourself.! !

!BenchmarkRunner methodsFor: 'accessing-categories' stamp: 'tfel 5/11/2015 17:45'!
categorySelected
	^ 0! !

!BenchmarkRunner methodsFor: 'accessing-categories' stamp: 'tfel 5/11/2015 17:45'!
categorySelected: anInteger
	self changed: #categorySelected.! !

!BenchmarkRunner methodsFor: 'accessing-classes' stamp: 'tfel 5/11/2015 17:45'!
classAt: anInteger
	^ classesSelected includes: (classes at: anInteger ifAbsent: [ ^ false ]).! !

!BenchmarkRunner methodsFor: 'accessing-classes' stamp: 'tfel 5/11/2015 17:45'!
classAt: anInteger put: aBoolean
	classesSelected := classesSelected
		perform: (aBoolean ifTrue: [ #copyWith: ] ifFalse: [ #copyWithout: ])
		with: (classes at: anInteger ifAbsent: [ ^ self ]).
	self changed: #classSelected; changed: #hasRunnable.! !

!BenchmarkRunner methodsFor: 'accessing-classes' stamp: 'tfel 5/11/2015 18:04'!
classList
	| offset |
	classes isEmpty ifTrue: [ ^ classes ].
	offset := classes first allSuperclasses size.
	^ classes collect: [ :each | | ident |
		ident := String 
			new: 2 * (0 max: each allSuperclasses size - offset) 
			withAll: $ .
		(each isAbstract or: [each benchmarkSelectors size = 0])
			ifFalse: [ ident , each name ]
			ifTrue: [ 
				ident asText , each name asText 
					addAttribute: TextEmphasis italic;
					yourself ] ].! !

!BenchmarkRunner methodsFor: 'accessing-classes' stamp: 'tfel 5/11/2015 18:42'!
classMenu: aMenu
	
	aMenu
		title: 'Classes';
		add: 'Browse' action: #browseClass;
		addLine;
		add: 'Select all' action: #selectAllClasses;
		add: 'Select subclasses' action: #selectSubclasses;
		add: 'Select inversion' action: #selectInverseClasses;
		add: 'Select none' action: #selectNoClasses;
		addLine;
		add: 'Filter...' action: #filterClasses;
		addLine;
		add: 'Refresh' action: #updateClasses.
	self classSelected ~= 0 ifTrue: [
		aMenu addLine.
		(classes at: self classSelected) benchmarkSelectors do: [:ea |
			aMenu add: 'Run ', ea asString selector: #runSingleTest: argument: ea]].
	^ aMenu! !

!BenchmarkRunner methodsFor: 'accessing-classes' stamp: 'tfel 5/11/2015 17:45'!
classSelected
	^ classIndex! !

!BenchmarkRunner methodsFor: 'accessing-classes' stamp: 'tfel 5/11/2015 17:45'!
classSelected: anInteger
	classIndex := anInteger.
	self changed: #classSelected! !

!BenchmarkRunner methodsFor: 'building' stamp: 'tfel 5/11/2015 17:45'!
classesFrame
	^LayoutFrame new
		leftFraction: 0.25 offset: 0;
		topFraction: 0 offset: 0;
		rightFraction: 0.5 offset: 0;
		bottomFraction: 1 offset: self buttonHeight negated + 4! !

!BenchmarkRunner methodsFor: 'accessing' stamp: 'tfel 5/11/2015 17:45'!
classesSelected
	^ classesSelected! !

!BenchmarkRunner methodsFor: 'actions' stamp: 'tfel 5/11/2015 20:17'!
compareWithPrevious: aReport

	| history |
	history := aReport suite class historyAt: aReport runner selector.
	history size > 1 ifFalse: [^ self].
	1 halt.! !

!BenchmarkRunner methodsFor: 'actions' stamp: 'tfel 5/15/2015 10:01'!
debug: anItem

	(anItem isKindOf: SMarkReporter)
		ifTrue: [^ Workspace new
					contents: (String streamContents: [:s | anItem reportOn: s]);
					openLabel: 'Results'].
	(anItem isKindOf: ContextPart)
		ifTrue: [^ Debugger openContext: anItem label: 'Debug' contents: nil].! !

!BenchmarkRunner methodsFor: 'private' stamp: 'tfel 5/11/2015 18:24'!
defaultBackgroundColor
	"<lint: #expect rule: #overridesSuper rational: 'we want a different color than the parent'>"

	^ Preferences testRunnerWindowColor muchLighter! !

!BenchmarkRunner methodsFor: 'accessing-testing' stamp: 'tfel 5/11/2015 17:45'!
errorList
	^ errorList collect: [ :each | each printString ].! !

!BenchmarkRunner methodsFor: 'building' stamp: 'tfel 5/11/2015 17:53'!
errorListFrame
	^LayoutFrame new
		leftFraction: 0.5 offset: 0;
		topFraction: 0.7 offset: 0;
		rightFraction: 1 offset: 0;
		bottomFraction: 1 offset: self buttonHeight negated + 4! !

!BenchmarkRunner methodsFor: 'accessing-menu' stamp: 'tfel 5/11/2015 17:45'!
errorMenu: aMenu
	^ self statusMenu: aMenu! !

!BenchmarkRunner methodsFor: 'accessing-testing' stamp: 'tfel 5/11/2015 17:45'!
errorSelected
	^ errorList indexOf: errorSelected.! !

!BenchmarkRunner methodsFor: 'accessing-testing' stamp: 'tfel 5/11/2015 17:45'!
errorSelected: anInteger
	errorSelected := errorList at: anInteger ifAbsent: nil.
	self changed: #errorSelected.
	errorSelected ifNotNil: [ self debug: errorSelected ].! !

!BenchmarkRunner methodsFor: 'accessing-ui' stamp: 'tfel 5/11/2015 17:45'!
extent
	^ 640 @ 480! !

!BenchmarkRunner methodsFor: 'accessing-testing' stamp: 'tfel 5/11/2015 17:45'!
failedList
	^ failedList collect: [ :each | each printString ].! !

!BenchmarkRunner methodsFor: 'accessing-testing' stamp: 'tfel 5/11/2015 17:45'!
failedSelected
	^ failedList indexOf: failedSelected.! !

!BenchmarkRunner methodsFor: 'accessing-testing' stamp: 'tfel 5/11/2015 17:45'!
failedSelected: anInteger
	failedSelected := failedList at: anInteger ifAbsent: nil.
	self changed: #failedSelected.
	failedSelected ifNotNil: [ self debug: failedSelected ].! !

!BenchmarkRunner methodsFor: 'building' stamp: 'tfel 5/11/2015 17:53'!
failureListFrame
	^LayoutFrame new
		leftFraction: 0.5 offset: 0;
		topFraction: 0.4 offset: 0;
		rightFraction: 1 offset: 0;
		bottomFraction: 0.7 offset: 0! !

!BenchmarkRunner methodsFor: 'accessing-menu' stamp: 'tfel 5/11/2015 17:45'!
failureMenu: aMenu
	^ aMenu! !

!BenchmarkRunner methodsFor: 'accessing-categories' stamp: 'tfel 5/11/2015 17:45'!
filterCategories
	| pattern |
	pattern := UIManager default 
					request: 'Pattern(s) to select categories:\    (separate patterns with '';'')' withCRs
					initialAnswer: (categoryPattern ifNil: ['*']).
	(pattern isNil or: [pattern isEmpty]) ifTrue:
		[^self].
	categoriesSelected := ((categoryPattern := pattern) subStrings: ';')
								inject: Set new
								into: [:matches :subPattern|
									matches
										addAll: (categories select: [ :each | subPattern match: each]);
										yourself].
	self changed: #allSelections; changed: #categorySelected; updateClasses! !

!BenchmarkRunner methodsFor: 'accessing-classes' stamp: 'tfel 5/11/2015 17:45'!
filterClasses
	| pattern |
	pattern := UIManager default 
					request: 'Pattern(s) to select tests:\  (separate patterns with '';'')' withCRs
					initialAnswer: (classPattern ifNil: '*').
	(pattern isNil or: [pattern isEmpty]) ifTrue:
		[^self].
	classesSelected := ((classPattern := pattern) subStrings: ';')
							inject: Set new
							into: [:matches :subPattern|
								matches
									addAll: (classes select: [ :each | subPattern match: each name]);
									yourself].
	self
		changed: #allSelections;
		changed: #classSelected;
		changed: #hasRunnable! !

!BenchmarkRunner methodsFor: 'utilities' stamp: 'tfel 5/11/2015 17:45'!
findCategories
	| visible |
	visible := Set new.
	self baseClass withAllSubclassesDo: [ :each |
		each category ifNotNil: [ :category |
			visible add: category ] ].
	^ Array streamContents: [ :stream |
		Smalltalk organization categories do: [ :each |
			(visible includes: each)
				ifTrue: [ stream nextPut: each ] ] ].! !

!BenchmarkRunner methodsFor: 'utilities' stamp: 'tfel 5/11/2015 17:45'!
findClassesForCategories: aCollection

	| items |
	aCollection isEmpty 
		ifTrue: [ ^ self baseClass withAllSubclasses asSet ].
	items := aCollection gather: [ :category |
		((Smalltalk organization listAtCategoryNamed: category)
			collect: [ :each | Smalltalk at: each ])
			select: [ :each | each includesBehavior: self baseClass ] ].
	^ items asSet.! !

!BenchmarkRunner methodsFor: 'testing' stamp: 'tfel 5/11/2015 18:24'!
hasErrors
	^ errorList notEmpty! !

!BenchmarkRunner methodsFor: 'testing' stamp: 'tfel 5/11/2015 18:24'!
hasFailures
	^ failedList notEmpty! !

!BenchmarkRunner methodsFor: 'testing' stamp: 'tfel 5/11/2015 18:24'!
hasRunnable
	^ classesSelected notEmpty! !

!BenchmarkRunner methodsFor: 'initialization' stamp: 'tfel 5/11/2015 18:41'!
initialize
	super initialize.
	self resetLists.
	SystemChangeNotifier uniqueInstance 
		notify: self ofSystemChangesOfItem: #class change: #Added using: #update;
		notify: self ofSystemChangesOfItem: #category change: #Added using: #update;
		notify: self ofSystemChangesOfItem: #class change: #Removed using: #update;
		notify: self ofSystemChangesOfItem: #category change: #Removed using: #update;
		notify: self ofSystemChangesOfItem: #class change: #Renamed using: #update;
		notify: self ofSystemChangesOfItem: #category change: #Renamed using: #update;
		notify: self ofSystemChangesOfItem: #class change: #Recategorized using: #update;
		notify: self ofSystemChangesOfItem: #category change: #Recategorized using: #update.
	self update; reset! !

!BenchmarkRunner methodsFor: 'accessing-ui' stamp: 'tfel 5/11/2015 18:05'!
label
	^ 'Benchmark Runner' ! !

!BenchmarkRunner methodsFor: 'accessing' stamp: 'tfel 5/11/2015 17:45'!
methodList
	self classesSelected ifEmpty: [ ^ #() ].
	^ self classesSelected gather: [:cls |
		cls benchmarkSelectors]! !

!BenchmarkRunner methodsFor: 'private' stamp: 'tfel 5/11/2015 17:45'!
perform: selector orSendTo: otherTarget
	"<lint: #expect rule: #badMessage rational: 'this is a common morphic pattern'>"
	
	^ (self respondsTo: selector)
		ifTrue: [ self perform: selector ]
		ifFalse: [ super perform: selector orSendTo: otherTarget ].! !

!BenchmarkRunner methodsFor: 'accessing-ui' stamp: 'tfel 5/11/2015 17:45'!
postAcceptBrowseFor: aModel
	"Nothing to do."! !

!BenchmarkRunner methodsFor: 'accessing-ui' stamp: 'tfel 5/11/2015 17:45'!
representsSameBrowseeAs: anotherModel 
	^ self class = anotherModel class
	and: [ classesSelected = anotherModel classesSelected ]! !

!BenchmarkRunner methodsFor: 'actions' stamp: 'tfel 5/11/2015 18:41'!
reset
	
	self
		resetLists;
		updateResults.! !

!BenchmarkRunner methodsFor: 'actions' stamp: 'tfel 5/11/2015 18:45'!
resetLists

	resultList := OrderedCollection new.
	failedList := OrderedCollection new.
	errorList := OrderedCollection new.! !

!BenchmarkRunner methodsFor: 'accessing-testing' stamp: 'tfel 5/11/2015 18:54'!
resultList
	^ resultList collect: [ :each | String streamContents: [:s | each reportOn: s]]! !

!BenchmarkRunner methodsFor: 'building' stamp: 'tfel 5/11/2015 17:54'!
resultListFrame
	^LayoutFrame new
		leftFraction: 0.5 offset: 0;
		topFraction: 0 offset: self statusHeight;
		rightFraction: 1 offset: 0;
		bottomFraction: 0.4 offset: 0! !

!BenchmarkRunner methodsFor: 'accessing-menu' stamp: 'tfel 5/11/2015 20:10'!
resultMenu: aMenu

	resultSelected ifNil: [
		^ aMenu addTitle: 'Select one of the results'].
	^ aMenu
		add: 'Compare with previous runs' selector: #compareWithPrevious: argument: resultSelected;
		yourself! !

!BenchmarkRunner methodsFor: 'accessing-testing' stamp: 'tfel 5/11/2015 17:55'!
resultSelected
	^ resultList indexOf: resultSelected.! !

!BenchmarkRunner methodsFor: 'accessing-testing' stamp: 'tfel 5/11/2015 17:55'!
resultSelected: anInteger
	resultSelected := resultList at: anInteger ifAbsent: nil.
	self changed: #resultSelected.
	resultSelected ifNotNil: [ self debug: resultSelected ].! !

!BenchmarkRunner methodsFor: 'actions' stamp: 'tfel 5/11/2015 20:28'!
runAll
	numIterations := (UIManager default chooseFrom: #(1 10 20) values: #(1 10 20) title: 'How many iterations?').
	doWarmup := (UIManager default chooseFrom: #(Yes No) values: #(true false) title: 'Should we do a warmup run?').
	self reset; runSuite: self classesSelected.
	self saveResultInHistory! !

!BenchmarkRunner methodsFor: 'actions' stamp: 'tfel 5/15/2015 09:32'!
runBenchmark: aBenchmark inSuite: suite
	| reporter |
	(doWarmup ifTrue: [2] ifFalse: [1]) timesRepeat: [
	[reporter := SMarkHarness
		run: suite
		specificBenchmark: aBenchmark
		iterations: numIterations
		processes: 1
		problemSize: 5
		runner: SMarkAutosizeRunner
		reporter: SMarkSimpleStatisticsReporter] on: Error do: [:ex | errorList add: thisContext copyStack]].
	reporter ifNotNil: [resultList add: reporter].
	self updateStatus: true.
! !

!BenchmarkRunner methodsFor: 'actions' stamp: 'tfel 5/11/2015 17:45'!
runErrors
	self result instVarNamed: 'errors' put: OrderedCollection new.
	self runSuite: self suiteErrors.! !

!BenchmarkRunner methodsFor: 'actions' stamp: 'tfel 5/11/2015 17:45'!
runFailures
	self result instVarNamed: 'failures' put: Set new.
	self runSuite: self suiteFailures.! !

!BenchmarkRunner methodsFor: 'actions' stamp: 'tfel 5/11/2015 17:45'!
runProfiled
	MessageTally spyOn: [ self runAll ].! !

!BenchmarkRunner methodsFor: 'actions' stamp: 'tfel 5/11/2015 18:46'!
runSingleTest: aSelector

	self reset.
	self runBenchmark: aSelector inSuite: (classes at: self classSelected).! !

!BenchmarkRunner methodsFor: 'actions' stamp: 'tfel 5/11/2015 18:22'!
runSuite: aTestSuite

	self basicRunSuite: aTestSuite.
	self updateResults
! !

!BenchmarkRunner methodsFor: 'history saving' stamp: 'tfel 5/11/2015 19:49'!
saveResultInHistory

	resultList do: [:r |
		r suite class historyAdd: r at: r runner selector].
! !

!BenchmarkRunner methodsFor: 'accessing-categories' stamp: 'tfel 5/11/2015 17:45'!
selectAllCategories
	categoriesSelected := categories asSet.
	self changed: #allSelections; changed: #categorySelected; updateClasses! !

!BenchmarkRunner methodsFor: 'accessing-classes' stamp: 'tfel 5/11/2015 17:45'!
selectAllClasses
	"Fixed to update all selections now that the
	selection invalidation has been optimised."
	
	classesSelected := classes asSet.
	self
		changed: #allSelections;
		changed: #classSelected;
		changed: #hasRunnable! !

!BenchmarkRunner methodsFor: 'accessing-categories' stamp: 'tfel 5/11/2015 17:45'!
selectInverseCategories
	categoriesSelected := categories asSet 
		removeAll: categoriesSelected;
		yourself.
	self changed: #allSelections; changed: #categorySelected; updateClasses! !

!BenchmarkRunner methodsFor: 'accessing-classes' stamp: 'tfel 5/11/2015 17:45'!
selectInverseClasses
	"Fixed to update all selections now that the
	selection invalidation has been optimised."
	
	classesSelected := classes asSet 
		removeAll: classesSelected;
		yourself.
	self
		changed: #allSelections;
		changed: #classSelected;
		changed: #hasRunnable! !

!BenchmarkRunner methodsFor: 'accessing-categories' stamp: 'tfel 5/11/2015 17:45'!
selectNoCategories
	categoriesSelected := Set new.
	self changed: #allSelections; changed: #categorySelected; updateClasses! !

!BenchmarkRunner methodsFor: 'accessing-classes' stamp: 'tfel 5/11/2015 17:45'!
selectNoClasses
	"Fixed to update all selections now that the
	selection invalidation has been optimised."
	
	classesSelected := Set new.
	self
		changed: #allSelections;
		changed: #classSelected;
		changed: #hasRunnable! !

!BenchmarkRunner methodsFor: 'accessing-classes' stamp: 'tfel 5/11/2015 17:45'!
selectSubclasses
	"Fixed to update all selections now that the
	selection invalidation has been optimised."
	
	| classesForPackages |
	classesForPackages := self findClassesForCategories: categoriesSelected.	
	classesSelected := (classesSelected gather: [ :class |
		class withAllSubclasses select: [ :each |
			classesForPackages includes: each ] ])
		asSet.
	self
		changed: #allSelections;
		changed: #classSelected;
		changed: #hasRunnable! !

!BenchmarkRunner methodsFor: 'history saving' stamp: 'tfel 5/11/2015 17:45'!
showDiffWith: aTestResult
	| string diff |

	diff := result diff: aTestResult.
	string := String streamContents: [:str|
		str nextPutAll: '----------------'; cr.
		str nextPutAll: 'Diff between current result with: ', aTestResult asString; cr.
		str nextPutAll: 'New passed: '.
		diff first do: [:s| str nextPutAll: s printString, ' '].
		str cr.
		str nextPutAll: 'New failures: '.
		diff second do: [:s| str nextPutAll: s printString, ' '].
		str cr.
		
		str nextPutAll: 'New errors: '.
		diff third do: [:s| str nextPutAll: s printString, ' '].
		str cr].
	
	Workspace new contents: string; openLabel: 'SUnit Progress'
	! !

!BenchmarkRunner methodsFor: 'utilities' stamp: 'tfel 5/11/2015 17:45'!
sortClass: aFirstClass before: aSecondClass
	| first second |
	first := aFirstClass withAllSuperclasses reversed.
	second := aSecondClass withAllSuperclasses reversed.
	1 to: (first size min: second size) do: [ :index | 
		(first at: index) == (second at: index)
			ifFalse: [ ^ (first at: index) name <= (second at: index) name ] ].
	^ second includes: aFirstClass.! !

!BenchmarkRunner methodsFor: 'accessing-testing' stamp: 'tfel 5/11/2015 18:34'!
statusColor
	self hasErrors 
		ifTrue: [ ^ Color red ].
	self hasFailures 
		ifTrue:[ ^ Color yellow ].
	^ Color green! !

!BenchmarkRunner methodsFor: 'building' stamp: 'tfel 5/11/2015 17:45'!
statusFrame
	^LayoutFrame new
		leftFraction: 0.5 offset: 0;
		topFraction: 0 offset: 0;
		rightFraction: 1 offset: 0;
		bottomFraction: 0 offset: self statusHeight! !

!BenchmarkRunner methodsFor: 'building' stamp: 'tfel 5/11/2015 17:45'!
statusHeight
	^Preferences standardCodeFont height * 2 + 12! !

!BenchmarkRunner methodsFor: 'accessing-menu' stamp: 'tfel 5/11/2015 20:08'!
statusMenu: aMenu

	^ aMenu! !

!BenchmarkRunner methodsFor: 'accessing-testing' stamp: 'tfel 5/15/2015 09:45'!
statusText
	^ resultList size asString, ' run, ', failedList size asString, ' slowed down, ', errorList size asString, ' errored'! !

!BenchmarkRunner methodsFor: 'updating' stamp: 'tfel 5/11/2015 17:45'!
update
	self updateCategories; updateClasses! !

!BenchmarkRunner methodsFor: 'updating' stamp: 'tfel 5/11/2015 17:45'!
updateCategories
	categories := self findCategories.
	categoriesSelected := categoriesSelected isNil
		ifTrue: [ Set new ]
		ifFalse: [
			categoriesSelected
				select: [ :each | categories includes: each ] ].
	self changed: #categoryList; changed: #categorySelected.! !

!BenchmarkRunner methodsFor: 'updating' stamp: 'tfel 5/11/2015 17:45'!
updateClasses
	| classesForCategories |
	classesForCategories := self findClassesForCategories: categoriesSelected.
	classes := classesForCategories asArray
		sort: [ :a :b | self sortClass: a before: b ].
	classIndex := 0.
	classesSelected := classesSelected isNil
		ifTrue: [ classesForCategories ]
		ifFalse: [ 
			classesSelected
				select: [ :each | classesForCategories includes: each ] ].
	self changed: #classList; changed: #classSelected; changed: #hasRunnable.! !

!BenchmarkRunner methodsFor: 'updating' stamp: 'tfel 5/11/2015 18:57'!
updateResults

	self updateStatus: false.
	resultSelected := nil.
	self 
		changed: #resultList; 
		changed: #resultSelected.
	failedSelected := nil.
	self 
		changed: #failedList; 
		changed: #failedSelected;
		changed: #hasFailures.
	errorSelected := nil.
	self 
		changed: #errorList; 
		changed: #errorSelected;
		changed: #hasErrors.! !

!BenchmarkRunner methodsFor: 'updating' stamp: 'tfel 5/11/2015 17:45'!
updateStatus: aBoolean
	"Update the status display, at most once a second if aBoolean is true."

	(aBoolean and: [ lastUpdate = Time totalSeconds ])
		ifTrue: [ ^ self ].
	self changed: #statusText; changed: #statusColor.
	lastUpdate := Time totalSeconds.! !

!BenchmarkRunner methodsFor: 'private' stamp: 'tfel 5/11/2015 17:45'!
windowIsClosing
	SystemChangeNotifier uniqueInstance noMoreNotificationsFor: self! !

Object subclass: #DBAbstractConstraint
	instanceVariableNames: 'strength'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-DeltaBlue'!

!DBAbstractConstraint commentStamp: '<historical>' prior: 0!
I am an abstract class representing a system-maintainable relationship (or "constraint") between a set of variables. I supply a strength instance variable; concrete subclasses provide a means of storing the constrained variables and other information required to represent a constraint.

Instance variables:
	strength			the strength of this constraint <Strength>!

!DBAbstractConstraint methodsFor: 'adding' stamp: 'ClementBera 9/23/2013 10:54'!
addConstraint
	"Activate this constraint and attempt to satisfy it."

	self addToGraph.
	DBPlanner current incrementalAdd: self.! !

!DBAbstractConstraint methodsFor: 'adding' stamp: 'ClementBera 9/23/2013 10:54'!
addToGraph
	"Add myself to the constraint graph."

	self subclassResponsibility! !

!DBAbstractConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:55'!
chooseMethod: mark
	"Decide if I can be satisfied and record that decision. The output of
	 the choosen method must not have the given mark and must have a
	 walkabout strength less than that of this constraint."

	self subclassResponsibility! !

!DBAbstractConstraint methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 14:54'!
destroyConstraint
	"Deactivate this constraint, remove it from the constraint graph,
	 possibly causing other constraints to be satisfied, and destroy it."

	(self isSatisfied) ifTrue: [DBPlanner current incrementalRemove: self].
	self removeFromGraph.
	"self release."! !

!DBAbstractConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:55'!
execute
	"Enforce this constraint. Assume that it is satisfied."

	self subclassResponsibility! !

!DBAbstractConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:55'!
inputsDo: aBlock
	"Assume that I am satisfied. Evaluate the given block on all my current
	 input variables."

	self subclassResponsibility! !

!DBAbstractConstraint methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 14:55'!
inputsKnown: mark
	"Assume that I am satisfied. Answer true if all my current inputs are
	 known. A variable is known if either a) it is 'stay' (i.e. it is a
	 constant at plan execution time), b) it has the given mark (indicating
	 that it has been computed by a constraint appearing earlier in the
	 plan), or c) it is not determined by any constraint."

	self inputsDo:
		[ :v |
		 (v mark = mark or: [(v stay) or: [v determinedBy == nil]]) ifFalse:
			[^false]].
	^true! !

!DBAbstractConstraint methodsFor: 'testing' stamp: 'ClementBera 9/23/2013 10:53'!
isInput
	"Normal constraints are not input constraints. An input constraint is
	 one that depends on external state, such as the mouse, the keyboard,
	 a clock, or some arbitrary piece of imperative code."

	^false! !

!DBAbstractConstraint methodsFor: 'testing' stamp: 'ClementBera 9/23/2013 10:54'!
isSatisfied
	"Answer true if this constraint is satisfied in the current solution."

	self subclassResponsibility! !

!DBAbstractConstraint methodsFor: 'printing' stamp: 'ClementBera 9/23/2013 11:02'!
longPrintOn: aStream

	aStream nextPut: $(.
	self shortPrintOn: aStream.
	aStream space; nextPutAll: self strength printString.
	(self isSatisfied)
		ifTrue:
			[aStream cr; space; space; space.
			 self inputsDo:
				[ :in | aStream nextPutAll: 'v', in printString, ' '].
			aStream nextPutAll: '-> '.
			aStream nextPutAll: 'v', self output printString]
		ifFalse:
			[aStream space; nextPutAll: 'UNSATISFIED'].
	aStream nextPut: $); cr.! !

!DBAbstractConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:55'!
markUnsatisfied
	"Record the fact that I am unsatisfied."

	self subclassResponsibility! !

!DBAbstractConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:55'!
output
	"Answer my current output variable. Raise an error if I am not
	 currently satisfied."

	self subclassResponsibility! !

!DBAbstractConstraint methodsFor: 'printing' stamp: 'ClementBera 9/23/2013 10:57'!
printOn: aStream

	self shortPrintOn: aStream! !

!DBAbstractConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:56'!
recalculate
	"Calculate the walkabout strength, the stay flag, and, if it is 'stay',
	 the value for the current output of this constraint. Assume this
	 constraint is satisfied."

	self subclassResponsibility! !

!DBAbstractConstraint methodsFor: 'adding' stamp: 'ClementBera 9/23/2013 10:54'!
removeFromGraph
	"Remove myself from the constraint graph."

	self subclassResponsibility! !

!DBAbstractConstraint methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 14:56'!
satisfy: mark
	"Attempt to find a way to enforce this (still unsatisfied) constraint.
	 If successful, record the solution, perhaps modifying the current
	 dataflow graph.  Answer the constraint that this constraint overrides,
	 if there is one, or nil, if there isn't."

	| overridden out |
	self chooseMethod: mark.
	(self isSatisfied)
		ifTrue:			"constraint can be satisfied"
			["mark inputs to allow cycle detection in addPropagate"
			 self inputsDo: [ :in | in mark: mark].
			 out := self output.
			 overridden := out determinedBy.
			 (overridden == nil) ifFalse: [overridden markUnsatisfied].
			 out determinedBy: self.
			 (DBPlanner current addPropagate: self mark: mark) ifFalse:
				[self notify:
					('Cycle encountered adding:\   ',
					 self printString,
					 '\Constraint removed.') withCRs.
				 ^nil].
			 out mark: mark]
		ifFalse:			"constraint cannot be satisfied"
			[overridden := nil.
			 (strength sameAs: (DBStrength required)) ifTrue:
				[self notify: 'Failed to satisfy a required constraint']].
	^ overridden! !

!DBAbstractConstraint methodsFor: 'printing' stamp: 'ClementBera 9/23/2013 11:02'!
shortPrintOn: aStream

	aStream nextPutAll: self class name, '(', self printString, ')'.! !

!DBAbstractConstraint methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:53'!
strength
	^ strength! !

!DBAbstractConstraint methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 14:52'!
strength: strengthSymbol
	strength := DBStrength of: strengthSymbol! !

DBAbstractConstraint subclass: #DBBinaryConstraint
	instanceVariableNames: 'direction v1 v2'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-DeltaBlue'!

!DBBinaryConstraint commentStamp: '<historical>' prior: 0!
I am an abstract superclass for constraints having two possible output variables.

Instance variables:
	v1, v2		possible output variables <Variable>
	direction		one of:
					#forward (v2 is output)
					#backward (	v1 is output)
					nil (not satisfied)!

!DBBinaryConstraint methodsFor: 'adding' stamp: 'ClementBera 9/23/2013 10:21'!
addToGraph
	"Add myself to the constraint graph."

	v1 addConstraint: self.
	v2 addConstraint: self.
	direction := nil.! !

!DBBinaryConstraint methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 15:01'!
chooseMethod: mark
	"Decide if I can be satisfied and which way I should flow based on the relative strength of the variables I relate, and record that decision."

	(v1 mark == mark) ifTrue:		"forward or nothing"
		[((v2 mark ~= mark) and: [strength stronger: v2 walkStrength]) 
			ifTrue: [ ^ direction := #forward ]
			ifFalse: [ ^ direction := nil ] ].

	(v2 mark == mark) ifTrue:		"backward or nothing"
		[((v1 mark ~= mark) and: [strength stronger: v1 walkStrength])
			ifTrue: [ ^ direction := #backward ]
			ifFalse: [ ^ direction := nil ] ].

	"if we get here, neither variable is marked, so we have choice"
	(v1 walkStrength weaker: v2 walkStrength)
		ifTrue:
			[(strength stronger: v1 walkStrength)
				ifTrue: [ ^ direction := #backward ]
				ifFalse: [ ^ direction := nil ]]
		ifFalse:
			[(strength stronger: v2 walkStrength)
				ifTrue: [ ^ direction := #forward ]
				ifFalse: [ ^ direction := nil ]]. ! !

!DBBinaryConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:25'!
execute
	"Enforce this constraint. Assume that it is satisfied."

	self subclassResponsibility! !

!DBBinaryConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:25'!
inputsDo: aBlock
	"Evaluate the given block on my current input variable."

	direction == #forward
		ifTrue: [ aBlock value: v1 ]
		ifFalse: [ aBlock value: v2 ].! !

!DBBinaryConstraint methodsFor: 'testing' stamp: 'ClementBera 9/23/2013 10:21'!
isSatisfied
	"Answer true if this constraint is satisfied in the current solution."

	^ direction notNil! !

!DBBinaryConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:25'!
markUnsatisfied
	"Record the fact that I am unsatisfied."

	direction := nil.! !

!DBBinaryConstraint methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 15:01'!
output
	"Answer my current output variable."

	direction == #forward
		ifTrue: [ ^ v2 ]
		ifFalse: [ ^ v1 ]! !

!DBBinaryConstraint methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 15:02'!
recalculate
	"Calculate the walkabout strength, the stay flag, and, if it is 'stay',
         the value for the current output of this constraint. Assume this
	 constraint is satisfied."

	| in out |
	(direction == #forward)
		ifTrue: [in := v1. out := v2]
		ifFalse: [in := v2. out := v1].
	out walkStrength: (strength weakest: in walkStrength).
	out stay: (in stay).
	(out stay) ifTrue: [ self execute ].		"stay optimization"! !

!DBBinaryConstraint methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 14:58'!
removeFromGraph
	"Remove myself from the constraint graph."

	(v1 == nil) ifFalse: [v1 removeConstraint: self].
	(v2 == nil) ifFalse: [v2 removeConstraint: self].
	direction := nil.! !

!DBBinaryConstraint methodsFor: 'initialize' stamp: 'ClementBera 9/23/2013 10:21'!
var: variable1 var: variable2 strength: strengthSymbol
	"Initialize myself with the given variables and strength."

	strength := DBStrength of: strengthSymbol.
	v1 := variable1.
	v2 := variable2.
	direction := nil.
	self addConstraint.! !

DBBinaryConstraint subclass: #DBEqualityConstraint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-DeltaBlue'!

!DBEqualityConstraint commentStamp: '<historical>' prior: 0!
I constrain two variables to have the same value: "v1 = v2".!

!DBEqualityConstraint class methodsFor: 'instance creation' stamp: 'StefanMarr 4/2/2015 15:55'!
var: variable1 var: variable2 strength: strengthSymbol
	"Install a constraint with the given strength equating the given
	 variables."

	^(self new) var: variable1 var: variable2 strength: strengthSymbol! !

!DBEqualityConstraint methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 15:18'!
execute
	"Enforce this constraint. Assume that it is satisfied."
 
	(direction == #forward)
		ifTrue: [v2 value: v1 value]
		ifFalse: [v1 value: v2 value].! !

DBBinaryConstraint subclass: #DBScaleConstraint
	instanceVariableNames: 'offset scale'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-DeltaBlue'!

!DBScaleConstraint commentStamp: '<historical>' prior: 0!
I relate two variables by the linear scaling relationship:
"v2 = (v1 * scale) + offset". Either v1 or v2 may be changed to maintain this
relationship but the scale factor and offset are considered read-only.

Instance variables:
	scale		scale factor input variable <Variable>
	offset		offset input variable <Variable>!

!DBScaleConstraint class methodsFor: 'instance creation' stamp: 'ClementBera 9/23/2013 10:32'!
var: src var: scale var: offset var: dst strength: strengthSymbol
	"Install a scale constraint with the given strength on the given variables."

	^(self new) src: src scale: scale offset: offset dst: dst strength: strengthSymbol! !

!DBScaleConstraint methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 16:23'!
addToGraph
	"Add myself to the constraint graph."

	v1 addConstraint: self.
   v2 addConstraint: self.
   scale addConstraint: self.
   offset addConstraint: self.
   direction := nil.
! !

!DBScaleConstraint methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 16:30'!
execute
	"Enforce this constraint. Assume that it is satisfied."

	(direction == #forward)
		ifTrue: [v2 value: (v1 value * scale value) + offset value]
		ifFalse: [v1 value: (v2 value - offset value) // scale value].! !

!DBScaleConstraint methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 16:32'!
inputsDo: aBlock
	"Evaluate the given block on my current input variable."

	(direction == #forward)
		ifTrue: [ aBlock value: v1.
					 aBlock value: scale.
					 aBlock value: offset]
		ifFalse: [ aBlock value: v2.
					  aBlock 	value: scale. 
					  aBlock 	value: offset].! !

!DBScaleConstraint methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 16:32'!
recalculate
	"Calculate the walkabout strength, the stay flag, and, if it is 'stay', the value for the current output of this constraint. Assume this constraint is satisfied."

	| in out |
	(direction == #forward)
		ifTrue: [in := v1. out := v2]
		ifFalse: [out := v1. in := v2].
	out walkStrength: (strength weakest: in walkStrength).
	out stay: ((in stay) and: [(scale stay) and: [offset stay]]).
	out stay ifTrue: [self execute].		"stay optimization"! !

!DBScaleConstraint methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 16:24'!
removeFromGraph
	"Remove myself from the constraint graph."

	(v1 == nil) ifFalse: [v1 removeConstraint: self].
	(v2 == nil) ifFalse: [v2 removeConstraint: self].
	(scale == nil) ifFalse: [scale removeConstraint: self].
	(offset == nil) ifFalse: [offset removeConstraint: self].
	direction := nil.
! !

!DBScaleConstraint methodsFor: 'initialize' stamp: 'ClementBera 9/23/2013 10:27'!
src: srcVar scale: scaleVar offset: offsetVar dst: dstVar strength: strengthSymbol
	"Initialize myself with the given variables and strength."

	strength := DBStrength of: strengthSymbol.
	v1 := srcVar.
	v2 := dstVar.
	scale := scaleVar.
	offset := offsetVar.
	direction := nil.
	self addConstraint.! !

DBAbstractConstraint subclass: #DBUnaryConstraint
	instanceVariableNames: 'output satisfied'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-DeltaBlue'!

!DBUnaryConstraint commentStamp: '<historical>' prior: 0!
I am an abstract superclass for constraints having a single possible output variable.

Instance variables:
	output		possible output variable <Variable>
	satisfied		true if I am currently satisfied <Boolean>!

DBUnaryConstraint subclass: #DBEditConstraint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-DeltaBlue'!

!DBEditConstraint commentStamp: '<historical>' prior: 0!
I mark variables that should, with some level of preference, stay the same. I have one method with zero inputs and one output, which does nothing. Planners may exploit the fact that, if I am satisfied, my output will not change during plan execution. This is called "stay optimization."!

!DBEditConstraint class methodsFor: 'instance creation' stamp: 'StefanMarr 4/2/2015 15:16'!
var: aVariable strength: strengthSymbol
	"Install an edit constraint with the given strength on the given
	 variable."

	^(self new) var: aVariable strength: strengthSymbol! !

!DBEditConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:19'!
execute
	"Edit constraints do nothing."! !

!DBEditConstraint methodsFor: 'testing' stamp: 'ClementBera 9/23/2013 10:18'!
isInput
	"I indicate that a variable is to be changed by imperative code."

	^true! !

DBUnaryConstraint subclass: #DBStayConstraint
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-DeltaBlue'!

!DBStayConstraint commentStamp: '<historical>' prior: 0!
I mark variables that should, with some level of preference, stay the same.
I have one method with zero inputs and one output, which does nothing. Planners
may exploit the fact that, if I am satisfied, my output will not change during
plan execution. This is called "stay optimization."!

!DBStayConstraint class methodsFor: 'instance creation' stamp: 'StefanMarr 4/2/2015 15:17'!
var: aVariable strength: strengthSymbol
	"Install an edit constraint with the given strength on the given
	 variable."

	^(self new) var: aVariable strength: strengthSymbol! !

!DBStayConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:20'!
execute
	"Stay constraints do nothing."! !

!DBUnaryConstraint methodsFor: 'adding' stamp: 'ClementBera 9/23/2013 10:16'!
addToGraph
	"Add myself to the constraint graph."

	output addConstraint: self.
	satisfied := false.! !

!DBUnaryConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:19'!
chooseMethod: mark
	"Decide if I can be satisfied and record that decision."

	satisfied :=
		output mark ~= mark and:
		[strength stronger: output walkStrength].! !

!DBUnaryConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:17'!
execute
	"Enforce this constraint. Assume that it is satisfied."

	self subclassResponsibility! !

!DBUnaryConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:17'!
inputsDo: aBlock
	"I have no input variables."! !

!DBUnaryConstraint methodsFor: 'testing' stamp: 'ClementBera 9/23/2013 10:15'!
isSatisfied
	"Answer true if this constraint is satisfied in the current solution."

	^satisfied! !

!DBUnaryConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:17'!
markUnsatisfied
	"Record the fact that I am unsatisfied."

	satisfied := false.! !

!DBUnaryConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:18'!
output
	"Answer my current output variable."

	^ output! !

!DBUnaryConstraint methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:18'!
recalculate
	"Calculate the walkabout strength, the stay flag, and, if it is 'stay',
	 the value for the current output of this constraint. Assume this
	 constraint is satisfied."

	output walkStrength: strength.
	output stay: self isInput not.
	output stay ifTrue: [self execute].	"stay optimization"! !

!DBUnaryConstraint methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 16:41'!
removeFromGraph
	"Remove myself from the constraint graph."

	(output == nil) ifFalse: [output removeConstraint: self].
	satisfied := false.! !

!DBUnaryConstraint methodsFor: 'initialize' stamp: 'ClementBera 9/23/2013 10:15'!
var: aVariable strength: strengthSymbol
	"Initialize myself with the given variable and strength."

	strength := DBStrength of: strengthSymbol.
	output := aVariable.
	satisfied := false.
	self addConstraint.! !

Object subclass: #DBPlanner
	instanceVariableNames: 'currentMark'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-DeltaBlue'!

DBPlanner class
	instanceVariableNames: 'currentPlanner'!

!DBPlanner commentStamp: '<historical>' prior: 0!
This benchmark is an implementation of the DeltaBlue Constraint Solver described in `The DeltaBlue Algorithm: An Incremental Constraint Hierarchy Solver'', by Bjorn N. Freeman-Benson and John Maloney, Communications of the ACM, January 1990 (also as University of Washington TR 89-08-06).
!

DBPlanner class
	instanceVariableNames: 'currentPlanner'!

!DBPlanner class methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 21:11'!
chainTest: n
        "Do chain-of-equality-constraints performance tests."
        | vars editConstraint plan planner |
        planner := DBPlanner new.
        vars := Array new: n+1.
        1 to: n+1 do: [:i | vars at: i put: DBVariable new ].

        "thread a chain of equality constraints through the variables"
        1 to: n do:
            [ :i | | v1 v2 |
             v1 := vars at: i.
             v2 := vars at: i + 1.
             DBEqualityConstraint var: v1 var: v2 strength: #required].

        DBStayConstraint var: vars last strength: #strongDefault.
        editConstraint := DBEditConstraint var: (vars first) strength: #preferred.
        plan := planner extractPlanFromConstraints: (Array with: editConstraint).
        1 to: 100 do: [ :v | 
            vars first value: v.
            plan execute.
            vars last value ~= v ifTrue: [self error: 'Chain test failed!!!!']].
        editConstraint destroyConstraint
  ! !

!DBPlanner class methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 11:16'!
current
	^ currentPlanner! !

!DBPlanner class methodsFor: 'instance creation' stamp: 'ClementBera 9/23/2013 10:39'!
new
	^ currentPlanner := super new! !

!DBPlanner class methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 16:22'!
projectionTest: n
        "This test constructs a two sets of variables related to each other by
         a simple linear transformation (scale and offset)."

        | scale offset src dst planner dests |
        planner := DBPlanner new.
        dests := OrderedCollection new.
        scale := DBVariable value: 10.
        offset := DBVariable value: 1000.
        1 to: n do:
            [ :i |
            src := DBVariable value: i.
            dst := DBVariable value: i.
            dests add: dst.
            DBStayConstraint var: src strength: #default.
            DBScaleConstraint var: src var: scale var: offset var: dst strength: #required].

        planner changeVar: src newValue: 17.
        dst value ~= 1170 ifTrue: [self error: 'Projection test 1 failed!!!!'].

        planner changeVar: dst newValue: 1050.
        src value ~= 5 ifTrue: [self error: 'Projection test 2 failed!!!!'].

        planner changeVar: scale newValue: 5.
        1 to: n - 1 do: [ :i |
            (dests at: i) value ~= (i*5 + 1000)
                ifTrue: [self error: 'Projection test 3 failed!!!!']].

        planner changeVar: offset newValue: 2000.
        1 to: n - 1 do: [ :i |
            (dests at: i) value ~= (i*5 + 2000)
                ifTrue: [self error: 'Projection test 4 failed!!!!']]! !

!DBPlanner methodsFor: 'private' stamp: 'ClementBera 9/23/2013 10:36'!
addConstraintsConsuming: v to: aCollection

	| determiningC |
	determiningC := v determinedBy.
	v constraints do:
		[ :c |
		 (c == determiningC or: [c isSatisfied not]) ifFalse:
			[aCollection add: c]].! !

!DBPlanner methodsFor: 'private' stamp: 'ClementBera 9/23/2013 10:36'!
addPropagate: c mark: mark
	"Recompute the walkabout strengths and stay flags of all variables
	 downstream of the given constraint and recompute the actual values
	 of all variables whose stay flag is true. If a cycle is detected,
	 remove the given constraint and answer false. Otherwise, answer true.

	 Details: Cycles are detected when a marked variable is encountered
	 downstream of the given constraint. The sender is assumed to have
	 marked the inputs of the given constraint with the given mark. Thus,
	 encountering a marked node downstream of the output constraint means
	 that there is a path from the constraint's output to one of its
	 inputs."

	| todo d |
	todo := OrderedCollection with: c.
	[todo isEmpty] whileFalse:
		[d := todo removeFirst.
		 (d output mark = mark) ifTrue:
			[self incrementalRemove: c.
			 ^ false].
		 d recalculate.
		 self addConstraintsConsuming: d output to: todo].
	^ true! !

!DBPlanner methodsFor: 'private' stamp: 'ClementBera 9/23/2013 10:37'!
changeVar: aVariable newValue: newValue

	| editConstraint plan |
	editConstraint := DBEditConstraint var: aVariable strength: #preferred.
	plan := self extractPlanFromConstraints: (Array with: editConstraint).
	10 timesRepeat: [
		aVariable value: newValue.
		plan execute].
	editConstraint destroyConstraint.! !

!DBPlanner methodsFor: 'private' stamp: 'StefanMarr 4/2/2015 16:03'!
constraintsConsuming: v do: aBlock

	| determiningC |
	determiningC := v determinedBy.
	v constraints do:
		[ :c |
		 ((c == determiningC) or: [c isSatisfied not]) ifFalse:
			[aBlock value: c]].! !

!DBPlanner methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 15:59'!
extractPlanFromConstraints: constraints
	"Extract a plan for resatisfaction starting from the outputs of the
	 given constraints, usually a set of input constraints."

	| sources |
	sources := OrderedCollection new.
	constraints do:
		[: c | ((c isInput) and: [c isSatisfied]) ifTrue: [sources add: c]].
	^self makePlan: sources! !

!DBPlanner methodsFor: 'planning' stamp: 'StefanMarr 4/2/2015 16:00'!
extractPlanFromVariables: variables
	"Extract a plan from the dataflow graph having the given variables. It
	 is assumed that the given set of variables is complete, or at least
	 that it contains all the input variables."

	| sources |
	sources := OrderedCollection new.
	variables do:
		[: v |
		 (v constraints) do:
			[: c | ((c isInput) and: [c isSatisfied]) ifTrue: [sources add: c]]].
	^self makePlan: sources! !

!DBPlanner methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 15:58'!
incrementalAdd: c
	"Attempt to satisfy the given constraint and, if successful,
	 incrementally update the dataflow graph.

	 Details: If satifying the constraint is successful, it may override a
	 weaker constraint on its output. The algorithm attempts to resatisfy
	 that constraint using some other method. This process is repeated
	 until either a) it reaches a variable that was not previously
	 determined by any constraint or b) it reaches a constraint that
	 is too weak to be satisfied using any of its methods. The variables
	 of constraints that have been processed are marked with a unique mark
	 value so that we know where we've been. This allows the algorithm to
	 avoid getting into an infinite loop even if the constraint graph has
	 an inadvertent cycle."

	| mark overridden |
	mark := self newMark.
	overridden := c satisfy: mark.
	[overridden == nil] whileFalse:
		[overridden := overridden satisfy: mark].! !

!DBPlanner methodsFor: 'adding' stamp: 'ClementBera 9/23/2013 10:34'!
incrementalRemove: c
	"Entry point for retracting a constraint. Remove the given constraint,
	 which should be satisfied, and incrementally update the dataflow
	 graph.

	 Details: Retracting the given constraint may allow some currently
	 unsatisfiable downstream constraint be satisfied. We thus collect a
	 list of unsatisfied downstream constraints and attempt to satisfy
	 each one in turn. This list is sorted by constraint strength,
	 strongest first, as a heuristic for avoiding unnecessarily adding
	 and then overriding weak constraints."

	| out unsatisfied |
	out := c output.
	c markUnsatisfied.
	c removeFromGraph.
	unsatisfied := self removePropagateFrom: out.
	unsatisfied do: [: u | self incrementalAdd: u].! !

!DBPlanner methodsFor: 'initialize' stamp: 'StefanMarr 4/2/2015 15:57'!
initialize

	"super initialize."

	currentMark := 1.! !

!DBPlanner methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:35'!
makePlan: sources
	"Extract a plan for resatisfaction starting from the given satisfied
	 source constraints, usually a set of input constraints. This method
	 assumes that stay optimization is desired; the plan will contain only
	 constraints whose output variables are not stay. Constraints that do
	 no computation, such as stay and edit constraints, are not included
	 in the plan.

	 Details: The outputs of a constraint are marked when it is added to
	 the plan under construction. A constraint may be appended to the plan
	 when all its input variables are known. A variable is known if either
	 a) the variable is marked (indicating that has been computed by a
	 constraint appearing earlier in the plan), b) the variable is 'stay'
	 (i.e. it is a constant at plan execution time), or c) the variable
	 is not determined by any constraint. The last provision is for past
	 states of history variables, which are not stay but which are also
	 not computed by any constraint."

	| mark plan todo c |
	mark := self newMark.
	plan := DBPlan new.
	todo := sources.
	[todo isEmpty] whileFalse:
		[c := todo removeFirst.
		 ((c output mark ~= mark) and:		"not in plan already and..."
		  [c inputsKnown: mark]) ifTrue:	"eligible for inclusion"
			[plan addLast: c.
			 c output mark: mark.
			 self addConstraintsConsuming: c output to: todo]].
	^ plan! !

!DBPlanner methodsFor: 'private' stamp: 'ClementBera 9/23/2013 10:37'!
newMark
	"Select a previously unused mark value.

	 Details: We just keep incrementing. If necessary, the counter will
	 turn into a LargePositiveInteger. In that case, it will be a bit
	 slower to compute the next mark but the algorithms will all behave
	 correctly. We reserve the value '0' to mean 'unmarked'. Thus, this
	 generator starts at '1' and will never produce '0' as a mark value."

	^currentMark := currentMark + 1! !

!DBPlanner methodsFor: 'planning' stamp: 'ClementBera 9/23/2013 10:36'!
propagateFrom: v
	"The given variable has changed. Propagate new values downstream."

	| todo c |
	todo := OrderedCollection new.
	self addConstraintsConsuming: v to: todo.
	[todo isEmpty] whileFalse:
		[c := todo removeFirst.
		 c execute.
		 self addConstraintsConsuming: c output to: todo].! !

!DBPlanner methodsFor: 'private' stamp: 'StefanMarr 4/2/2015 16:05'!
removePropagateFrom: out
	"Update the walkabout strengths and stay flags of all variables
	 downstream of the given constraint. Answer a collection of unsatisfied
	 constraints sorted in order of decreasing strength."

	| unsatisfied todo v |
	unsatisfied := OrderedCollection new.
	
	out determinedBy: nil.
	out walkStrength: DBStrength absoluteWeakest.
	out stay: true.
	todo := OrderedCollection with: out.
	[todo isEmpty] whileFalse:
		[v := todo removeFirst.
		 v constraints do:
		 	[ :c | (c isSatisfied) ifFalse: [unsatisfied add: c]].
		 self constraintsConsuming: v do:
			[ :c |
			 c recalculate.
			 todo add: c output]].
	
	unsatisfied sort: [ :c1 :c2 | c1 strength stronger: c2 strength].
	^ unsatisfied! !

Object subclass: #DBStrength
	instanceVariableNames: 'symbolicValue arithmeticValue'
	classVariableNames: 'AbsoluteStrongest AbsoluteWeakest Required StrengthConstants StrengthTable'
	poolDictionaries: ''
	category: 'SMark-Benchmarks-DeltaBlue'!

!DBStrength commentStamp: '<historical>' prior: 0!
Strengths are used to measure the relative importance of constraints. The hierarchy of available strengths is determined by the class variable StrengthTable (see my class initialization method). Because Strengths are invariant, references to Strength instances are shared (i.e. all references to
"Strength of: #required" point to a single, shared instance). New strengths may be inserted in the strength hierarchy without disrupting current constraints.

Instance variables:
	symbolicValue		symbolic strength name (e.g. #required) <Symbol>
	arithmeticValue		index of the constraint in the hierarchy, used for comparisons <Number>!

!DBStrength class methodsFor: 'constants' stamp: 'ClementBera 9/23/2013 10:53'!
absoluteStrongest

	^AbsoluteStrongest! !

!DBStrength class methodsFor: 'constants' stamp: 'ClementBera 9/23/2013 10:53'!
absoluteWeakest

	^AbsoluteWeakest! !

!DBStrength class methodsFor: 'initialize' stamp: 'ClementBera 9/23/2013 10:52'!
initialize
	"Initialize the symbolic strength table. Fix the internally caches
	 values of all existing instances."
	"Strength initialize"

	StrengthTable := Dictionary new.
	StrengthTable at: #absoluteStrongest put: -10000.
	StrengthTable at: #required put: -800.
	StrengthTable at: #strongPreferred put: -600.
	StrengthTable at: #preferred put: -400.
	StrengthTable at: #strongDefault put: -200.
	StrengthTable at: #default put: 0.
	StrengthTable at: #weakDefault put: 500.
	StrengthTable at: #absoluteWeakest put: 10000.

	StrengthConstants := Dictionary new.
	StrengthTable keys do:
		[: strengthSymbol |
			StrengthConstants
				at: strengthSymbol
				put: ((super new) initializeWith: strengthSymbol)].

	AbsoluteStrongest := DBStrength of: #absoluteStrongest.
	AbsoluteWeakest := DBStrength of: #absoluteWeakest.
	Required := DBStrength of: #required.! !

!DBStrength class methodsFor: 'instance creation' stamp: 'ClementBera 9/23/2013 10:52'!
of: aSymbol
	"Answer an instance with the specified strength."

	^ StrengthConstants at: aSymbol! !

!DBStrength class methodsFor: 'constants' stamp: 'ClementBera 9/23/2013 10:53'!
required

	^Required! !

!DBStrength class methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 16:39'!
strengthTable
	^ StrengthTable! !

!DBStrength methodsFor: 'private' stamp: 'ClementBera 9/23/2013 10:50'!
arithmeticValue
	"Answer my arithmetic value. Used for comparisons. Note that
	 STRONGER constraints have SMALLER arithmetic values."

	^arithmeticValue! !

!DBStrength methodsFor: 'private' stamp: 'StefanMarr 4/2/2015 16:38'!
initializeWith: symVal
	"Record my symbolic value and reset my arithmetic value."

	symbolicValue := symVal.
	arithmeticValue := DBStrength strengthTable at: symbolicValue.! !

!DBStrength methodsFor: 'printing' stamp: 'ClementBera 9/23/2013 10:50'!
printOn: aStream
	"Append a string which represents my strength onto aStream."

	aStream nextPutAll: '%', symbolicValue, '%'.! !

!DBStrength methodsFor: 'comparing' stamp: 'ClementBera 9/23/2013 10:49'!
sameAs: aStrength
	"Answer true if I am the same strength as the given Strength."

	^ arithmeticValue = aStrength arithmeticValue! !

!DBStrength methodsFor: 'comparing' stamp: 'ClementBera 9/23/2013 10:49'!
stronger: aStrength
	"Answer true if I am stronger than the given Strength."

	^ arithmeticValue < aStrength arithmeticValue! !

!DBStrength methodsFor: 'max / min' stamp: 'StefanMarr 4/2/2015 16:37'!
strongest: aStrength
	"Answer the stronger of myself and aStrength."

	(aStrength stronger: self)
		ifTrue: [^aStrength]
		ifFalse: [^self]! !

!DBStrength methodsFor: 'comparing' stamp: 'ClementBera 9/23/2013 10:49'!
weaker: aStrength
	"Answer true if I am weaker than the given Strength."

	^ arithmeticValue > aStrength arithmeticValue! !

!DBStrength methodsFor: 'max / min' stamp: 'StefanMarr 4/2/2015 16:37'!
weakest: aStrength
	"Answer the weaker of myself and aStrength."

	(aStrength weaker: self)
		ifTrue: [^aStrength]
		ifFalse: [^self].! !

Object subclass: #DBVariable
	instanceVariableNames: 'constraints determinedBy mark stay value walkStrength'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-DeltaBlue'!

!DBVariable commentStamp: '<historical>' prior: 0!
I represent a constrained variable. In addition to my value, I maintain the structure of the constraint graph, the current dataflow graph, and various parameters of interest to the DeltaBlue incremental constraint solver.

Instance variables:
	value			my value; changed by constraints, read by client <Object>
	constraints		normal constraints that reference me <Array of Constraint>
	determinedBy	the constraint that currently determines
					my value (or nil if there isn''t one) <Constraint>
	walkStrength		my walkabout strength <Strength>
	stay			true if I am a planning-time constant <Boolean>
	mark			used by the planner to mark constraints <Number>!

!DBVariable class methodsFor: 'instance creation' stamp: 'ClementBera 9/23/2013 11:01'!
value: aValue

	^self new value: aValue! !

!DBVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:58'!
addConstraint: aConstraint
	"Add the given constraint to the set of all constraints that refer
	 to me."

	constraints add: aConstraint.! !

!DBVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:58'!
constraints
	^ constraints! !

!DBVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:58'!
determinedBy
	^ determinedBy! !

!DBVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:58'!
determinedBy: anObject
	determinedBy := anObject! !

!DBVariable methodsFor: 'initialize' stamp: 'StefanMarr 4/2/2015 16:43'!
initialize
	value := 0.
	constraints := OrderedCollection new: 2.
	determinedBy := nil.
	walkStrength := DBStrength absoluteWeakest.
	stay := true.
	mark := 0.! !

!DBVariable methodsFor: 'printing' stamp: 'StefanMarr 4/2/2015 21:00'!
longPrintOn: aStream
	^ super longPrintOn: aStream
"	self shortPrintOn: aStream.
	aStream nextPutAll: '   Constraints: '.
	(constraints isEmpty)
		ifTrue: [aStream cr; tab; nextPutAll: 'none']
		ifFalse:
			[constraints do:
				[: c | aStream cr; tab. c shortPrintOn: aStream]].
	(determinedBy isNil) ifFalse:
		[aStream cr; nextPutAll: '   Determined by: '.
		 aStream cr; tab. determinedBy shortPrintOn: aStream].
	aStream cr."! !

!DBVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:58'!
mark
	^ mark! !

!DBVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:58'!
mark: anObject
	mark := anObject! !

!DBVariable methodsFor: 'printing' stamp: 'StefanMarr 4/2/2015 20:59'!
printOn: aStream
	^ super printOn: aStream
	"self shortPrintOn: aStream"! !

!DBVariable methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 16:44'!
removeConstraint: c
	"Remove all traces of c from this variable."

	constraints remove: c ifAbsent: [].
	(determinedBy == c) ifTrue: [determinedBy := nil].! !

!DBVariable methodsFor: 'update' stamp: 'ClementBera 9/23/2013 10:59'!
setValue: aValue
	"Attempt to assign the given value to me using a strength of
	 #preferred."

	self setValue: aValue strength: #preferred.! !

!DBVariable methodsFor: 'update' stamp: 'ClementBera 9/23/2013 11:00'!
setValue: aValue strength: strengthSymbol
	"Attempt to assign the given value to me using the given strength."

	| editConstraint |
	editConstraint := DBEditConstraint var: self strength: strengthSymbol.
	(editConstraint isSatisfied) ifTrue:
		[self value: aValue.
		 DBPlanner propagateFrom: self].
	editConstraint destroyConstraint.! !

!DBVariable methodsFor: 'printing' stamp: 'StefanMarr 4/2/2015 21:00'!
shortPrintOn: aStream
	^ super shortPrintOn: aStream.
"
	aStream nextPutAll: 'V(', self printString, ', '.
	aStream nextPutAll: walkStrength printString, ', '.
	(stay isNil) ifFalse:
		[aStream nextPutAll: (stay ifTrue: ['stay, '] ifFalse: ['changing, '])].
	aStream nextPutAll: value printString.
	aStream nextPutAll: ')'.
	aStream cr."! !

!DBVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:59'!
stay
	^ stay! !

!DBVariable methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 16:44'!
stay: aBoolean
	stay := aBoolean! !

!DBVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:59'!
value
	^ value! !

!DBVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:59'!
value: anObject
	value := anObject! !

!DBVariable methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 10:59'!
walkStrength
	^ walkStrength! !

!DBVariable methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 16:44'!
walkStrength: aStrength
	walkStrength := aStrength! !

Object subclass: #GSEdge
	instanceVariableNames: 'dest weight'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-GraphSearch'!

!GSEdge class methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 15:25'!
newWith: dest and: weight
	^ self basicNew initializeWith: dest and: weight! !

!GSEdge methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 15:24'!
dest
	^ dest! !

!GSEdge methodsFor: 'initialization' stamp: 'StefanMarr 4/3/2015 15:25'!
initializeWith: destination and: w
	dest   := destination.
	weight := w.! !

!GSEdge methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 15:25'!
weight
	^ weight! !

Object subclass: #GSNode
	instanceVariableNames: 'starting noOfEdges'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-GraphSearch'!

!GSNode class methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 15:24'!
newWith: starting and: noOfEdges
	^ self basicNew initializeWith: starting and: noOfEdges! !

!GSNode methodsFor: 'initialization' stamp: 'StefanMarr 4/3/2015 15:23'!
initializeWith: start and: edges
	starting := start.
	noOfEdges:= edges! !

!GSNode methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 15:23'!
noOfEdges
	^ noOfEdges! !

!GSNode methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 15:23'!
starting
	^ starting! !

Object subclass: #JSHashIndexTable
	instanceVariableNames: 'hashTable'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Json'!

!JSHashIndexTable methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 22:22'!
at: name
    | slot |
    slot := self hashSlotFor: name.

    " subtract 1, 0 stands for empty "
    ^ ((hashTable at: slot) & 255) - 1! !

!JSHashIndexTable methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 22:16'!
at: name put: index
    | slot |
    slot := self hashSlotFor: name.
    
    index < 255
      ifTrue:  [ hashTable at: slot put: index + 1 ]
      ifFalse: [ hashTable at: slot put: 0 ]! !

!JSHashIndexTable methodsFor: 'hash' stamp: 'StefanMarr 4/3/2015 13:06'!
hashSlotFor: element
    ^ (element hash & 31) + 1! !

!JSHashIndexTable methodsFor: 'initialization' stamp: 'StefanMarr 4/2/2015 22:16'!
initialize
	hashTable := IntegerArray new: 32 withAll: 0! !

!JSHashIndexTable methodsFor: 'removing' stamp: 'StefanMarr 4/2/2015 22:21'!
remove: index
    1 to: hashTable size do: [:i |
      index + 1 = (hashTable at: i)
        ifTrue:  [ hashTable at: i put: 0 ]
        ifFalse: [ 
          (hashTable at: i) > index + 1 ifTrue: [
            hashTable at: i put: (hashTable at: i) - 1]]]! !

Object subclass: #JSJsonParser
	instanceVariableNames: 'input index line column current captureBuffer captureStart exceptionBlock'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Json'!

!JSJsonParser class methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 11:38'!
with: aJsonString
    ^ self basicNew initializeWith: aJsonString! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 13:03'!
endCapture
    | captured |
    '' = captureBuffer
      ifTrue:  [ captured := input copyFrom: captureStart to: index - 1 ]
      ifFalse: [
        self pauseCapture.
        captured := captureBuffer.
        captureBuffer := '' ].
    captureStart := -1.
    
    ^ captured! !

!JSJsonParser methodsFor: 'error handling' stamp: 'StefanMarr 4/3/2015 11:37'!
error: message
    'error:' print.
    message print.
    ':' print.
    line print.
    column println.
    exceptionBlock value: (JSParseException with: message at: index
                                          line: line    column: column )
  ! !

!JSJsonParser methodsFor: 'error handling' stamp: 'StefanMarr 4/3/2015 11:45'!
expected: expected
    self isEndOfText ifTrue: [
      self error: 'Unexpected end of input, expected ', expected asString
    ].
    self error: 'Expected ', expected! !

!JSJsonParser methodsFor: 'initialization' stamp: 'StefanMarr 4/3/2015 11:31'!
initializeWith: string
    input := string.
    index := 0.
    line  := 1.
    column := 0.
    current := nil.
    captureBuffer := ''.! !

!JSJsonParser methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 11:38'!
isDigit
    current = '0' ifTrue: [^ true].
    current = '1' ifTrue: [^ true].
    current = '2' ifTrue: [^ true].
    current = '3' ifTrue: [^ true].
    current = '4' ifTrue: [^ true].
    current = '5' ifTrue: [^ true].
    current = '6' ifTrue: [^ true].
    current = '7' ifTrue: [^ true].
    current = '8' ifTrue: [^ true].
    current = '9' ifTrue: [^ true].
    ^ false! !

!JSJsonParser methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 11:38'!
isEndOfText
    ^ current isNil! !

!JSJsonParser methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 11:38'!
isWhiteSpace
    current = ' '  ifTrue: [^ true].
    current = '\t' ifTrue: [^ true].
    current = '\n' ifTrue: [^ true].
    current = '\r' ifTrue: [^ true].
    ^ false! !

!JSJsonParser methodsFor: 'parsing' stamp: 'StefanMarr 4/3/2015 11:31'!
parse
    | result |
    exceptionBlock := [:ex | ^ ex ].
    self read.
    self skipWhiteSpace.
    result := self readValue.
    self skipWhiteSpace.
    self isEndOfText ifFalse: [ self error: 'Unexpected character'].
    ^ result! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:37'!
pauseCapture
    captureBuffer := captureBuffer concatenate: (
      input substringFrom: captureStart to: index - 1).
    captureStart := -1! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 13:02'!
read
	"TODO: this is probably broken"
    current = '\n' ifTrue: [
      line := line + 1.
      column := 0.
    ].
    
    index := index + 1.
    column := column + 1.

    input ifNil: [ self error:'input nil'].
    index <= input size
      ifTrue:  [ current := input copyFrom: index to: index ]
      ifFalse: [ current := nil ]! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:32'!
readArray
    | array |
    self read.
    array := JSJsonArray new.
    
    "Array might be empty"
    self skipWhiteSpace.
    (self readChar: ']') ifTrue: [
      ^ array
    ].
    
    self readArrayElement: array.
    [self readChar: ','] whileTrue: [
      self readArrayElement: array.
    ].

    (self readChar: ']') ifFalse: [
      self expected: '"," or "]"'
    ].
    ^ array! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:32'!
readArrayElement: array
    self skipWhiteSpace.
    array add: self readValue.
    self skipWhiteSpace.! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:36'!
readChar: ch
    current = ch ifFalse: [ ^ false ].
    self read.
    ^ true! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:36'!
readDigit
    self isDigit ifFalse: [ ^ false ].
    self read.
    ^ true! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:35'!
readEscape
    self read.    
    captureBuffer := captureBuffer concatenate: self readEscapeChar.
    self read! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:35'!
readEscapeChar
    current = '"'  ifTrue: [ ^ '"'  ].
    current = '/'  ifTrue: [ ^ '/'  ].
    current = '\\' ifTrue: [ ^ '\\' ].

    current = 'b'  ifTrue: [ ^ '\b'  ].
    current = 'f'  ifTrue: [ ^ '\f'  ].
    current = 'n'  ifTrue: [ ^ '\n'  ].
    current = 'r'  ifTrue: [ ^ '\r'  ].
    current = 't'  ifTrue: [ ^ '\t'  ].

    "TODO: SOM doesn't have a way to create unicode characters."
    self expected: 'valid escape sequence. note, some are not supported'
! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:35'!
readExponent
    ((self readChar: 'e') not and: [
      (self readChar: 'E') not]) ifTrue: [ ^ false ].
    
    
    (self readChar: '+') ifFalse: [ self readChar: '-' ].
    
    self readDigit ifFalse: [ self expected: 'digit' ].

    [self readDigit] whileTrue: [].

    ^ true! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:33'!
readFalse
    self read.
    self readRequiredChar: 'a'.
    self readRequiredChar: 'l'.
    self readRequiredChar: 's'.
    self readRequiredChar: 'e'.
    ^ JSJsonLiteral FALSE! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:35'!
readFraction
    (self readChar: '.') ifFalse: [ ^ false ].
    
    self readDigit ifFalse: [ self expected: 'digit' ].
    
    [self readDigit] whileTrue: [].

    ^ true! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:33'!
readName
    current = '"' ifFalse: [ self expected: 'name' ].
    ^ self readStringInternal! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:33'!
readNull
    self read.
    self readRequiredChar: 'u'.
    self readRequiredChar: 'l'.
    self readRequiredChar: 'l'.
    ^ JSJsonLiteral NULL! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 13:05'!
readNumber
    | firstDigit |
    self startCapture.
    self readChar: '-'.
    firstDigit := current.
    
    self readDigit ifFalse: [ self expected: 'digit' ].
    firstDigit ~= '0' ifTrue: [ [self readDigit] whileTrue: []].
        
    self readFraction.
    self readExponent.
    ^ JSJsonNumber new: self endCapture! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:32'!
readObject
    | object |
    self read.
    object := JSJsonObject new.
    self skipWhiteSpace.

    (self readChar: '}') ifTrue: [
      ^ object
    ].
    
    self readObjectKeyValuePair: object.
    [self readChar: ','] whileTrue: [
      self readObjectKeyValuePair: object.
    ].
    
    (self readChar: '}') ifFalse: [
      self expected: '"," or "}"'
    ].
    
    ^ object! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:32'!
readObjectKeyValuePair: object
    | name |
    self skipWhiteSpace.
    name := self readName.
    self skipWhiteSpace.
  
    (self readChar: ':') ifFalse: [ self expected: ':' ].
  
    self skipWhiteSpace.
  
    object add: name with: self readValue.
  
    self skipWhiteSpace! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:34'!
readRequiredChar: ch
    (self readChar: ch) ifFalse: [
      self expected: 'character: ' + ch
    ]! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:34'!
readString
    ^ JSJsonString new: self readStringInternal! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:34'!
readStringInternal
    | string |
    self read.
    self startCapture.
    
    [current = '"'] whileFalse: [
      current = '\\' ifTrue: [
        self pauseCapture.
        self readEscape.
        self startCapture.
      ] ifFalse: [
        "if (current < 0x20) { throw expected('valid string character'); }"
        "we currently don't have a way to get the ordinal value for a character"
        "} else {"
        self read.
      ]
    ].
    string := self endCapture.
    self read.
    ^ string! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:33'!
readTrue
    self read.
    self readRequiredChar: 'r'.
    self readRequiredChar: 'u'.
    self readRequiredChar: 'e'.
    ^ JSJsonLiteral TRUE! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:31'!
readValue
    current = 'n' ifTrue: [ ^ self readNull   ].
    current = 't' ifTrue: [ ^ self readTrue   ].
    current = 'f' ifTrue: [ ^ self readFalse  ].
    current = '"' ifTrue: [ ^ self readString ].
    current = '[' ifTrue: [ ^ self readArray  ].
    current = '{' ifTrue: [ ^ self readObject ].
    
    "Is this really the best way to write this?, or better #or:?,
     but with all the nesting, it's just ugly."
    current = '-' ifTrue: [ ^ self readNumber ].
    current = '0' ifTrue: [ ^ self readNumber ].
    current = '1' ifTrue: [ ^ self readNumber ].
    current = '2' ifTrue: [ ^ self readNumber ].
    current = '3' ifTrue: [ ^ self readNumber ].
    current = '4' ifTrue: [ ^ self readNumber ].
    current = '5' ifTrue: [ ^ self readNumber ].
    current = '6' ifTrue: [ ^ self readNumber ].
    current = '7' ifTrue: [ ^ self readNumber ].
    current = '8' ifTrue: [ ^ self readNumber ].
    current = '9' ifTrue: [ ^ self readNumber ].
    
    "else"
    self expected: 'value'! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:36'!
skipWhiteSpace
    [ self isWhiteSpace ]
      whileTrue:
        [ self read ].! !

!JSJsonParser methodsFor: 'parsing-private' stamp: 'StefanMarr 4/3/2015 11:36'!
startCapture
    captureStart := index! !

Object subclass: #JSJsonValue
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Json'!

JSJsonValue subclass: #JSJsonArray
	instanceVariableNames: 'values'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Json'!

!JSJsonArray class methodsFor: 'instance creation' stamp: 'StefanMarr 4/2/2015 22:52'!
readFrom: string
    ^ (JSJsonValue readFrom: string) asArray! !

!JSJsonArray methodsFor: 'comparing' stamp: 'StefanMarr 4/2/2015 22:51'!
= other
    self == other ifTrue: [ ^ true ].
    other == nil ifTrue: [ ^ false ].
    self class == other class ifFalse: [ ^ false ].

    ^ values = other values! !

!JSJsonArray methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 22:49'!
add: value
    value ifNil: [ self error: 'value is null' ].
    values add: value! !

!JSJsonArray methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 22:48'!
addBoolean: value
    values add: (JSJsonValue boolean: value)! !

!JSJsonArray methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 22:48'!
addDouble: value
	values add: (JSJsonValue double: value)! !

!JSJsonArray methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 22:48'!
addInteger: value
    values add: (JSJsonValue integer: value)! !

!JSJsonArray methodsFor: 'adding' stamp: 'StefanMarr 4/2/2015 22:49'!
addString: value
    values add: (JSJsonValue string: value)! !

!JSJsonArray methodsFor: 'converting' stamp: 'StefanMarr 4/2/2015 22:51'!
asArray
	^ self! !

!JSJsonArray methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 22:50'!
at: index
	^ values at: index! !

!JSJsonArray methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 22:50'!
at: index put: value
    value ifNil: [ self error: 'value is null' ].
    values at: index put: value! !

!JSJsonArray methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 22:49'!
at: index putBoolean: value
    values at: index put: (JSJsonValue boolean: value)! !

!JSJsonArray methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 22:49'!
at: index putDouble: value
    values at: index put: (JSJsonValue double: value)! !

!JSJsonArray methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 22:49'!
at: index putInteger: value
    values at: index put: (JSJsonValue integer: value)! !

!JSJsonArray methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 22:50'!
at: index putString: value
    values at: index put: (JSJsonValue string: value)! !

!JSJsonArray methodsFor: 'hash' stamp: 'StefanMarr 4/2/2015 22:51'!
hashcode
	^ values hash! !

!JSJsonArray methodsFor: 'initialization' stamp: 'StefanMarr 4/2/2015 22:47'!
initialize
	values := OrderedCollection new.! !

!JSJsonArray methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:51'!
isArray
	^ true! !

!JSJsonArray methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:50'!
isEmpty
    ^ values isEmpty! !

!JSJsonArray methodsFor: 'removing' stamp: 'StefanMarr 4/2/2015 22:50'!
removeAt: index
    values removeAt: index! !

!JSJsonArray methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 22:50'!
size
    ^ values size! !

!JSJsonArray methodsFor: 'accessing' stamp: 'StefanMarr 4/2/2015 22:51'!
values
	^ values! !

JSJsonValue subclass: #JSJsonLiteral
	instanceVariableNames: 'value isNull isTrue isFalse'
	classVariableNames: 'FALSE NULL TRUE'
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Json'!

!JSJsonLiteral class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 10:51'!
FALSE
	^ FALSE! !

!JSJsonLiteral class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 10:51'!
NULL
	^ NULL! !

!JSJsonLiteral class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 10:51'!
TRUE
	^ TRUE! !

!JSJsonLiteral class methodsFor: 'class initialization' stamp: 'StefanMarr 4/3/2015 13:13'!
initialize
   "self initialize"
	NULL := self basicNew initializeWith: 'null'.
	TRUE := self basicNew initializeWith: 'true'.
	FALSE:= self basicNew initializeWith: 'false'.! !

!JSJsonLiteral methodsFor: 'comparing' stamp: 'StefanMarr 4/3/2015 10:50'!
= other
    self == other ifTrue: [ ^ true ].  
    other == nil ifTrue: [ ^ false ].
    self class == other class ifFalse: [ ^ false ].

    ^ value = other asString
! !

!JSJsonLiteral methodsFor: 'converting' stamp: 'StefanMarr 4/3/2015 10:49'!
asBoolean
	isNull ifTrue: [ ^ super asBoolean ]
	       ifFalse:[ ^ isTrue ]! !

!JSJsonLiteral methodsFor: 'converting' stamp: 'StefanMarr 4/3/2015 10:44'!
asString
	^ value! !

!JSJsonLiteral methodsFor: 'hash' stamp: 'StefanMarr 4/3/2015 10:44'!
hashcode
	^ value hash! !

!JSJsonLiteral methodsFor: 'initialization' stamp: 'StefanMarr 4/3/2015 10:44'!
initializeWith: val
    value   := val.
    isNull  := 'null'  = val.
    isTrue  := 'true'  = val.
    isFalse := 'false' = val.! !

!JSJsonLiteral methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 10:49'!
isBoolean
	^ isTrue | isFalse! !

!JSJsonLiteral methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 10:48'!
isFalse
	^ isFalse! !

!JSJsonLiteral methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 10:48'!
isNull
	^ isNull! !

!JSJsonLiteral methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 10:48'!
isTrue
	^ isTrue! !

JSJsonValue subclass: #JSJsonNumber
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Json'!

!JSJsonNumber class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 11:05'!
new: string
    string ifNil: [ self error: 'string is null' ].
    ^ self new initializeWith: string
! !

!JSJsonNumber methodsFor: 'comparing' stamp: 'StefanMarr 4/3/2015 10:58'!
= other
    self == other ifTrue: [ ^ true ].
    other == nil ifTrue: [ ^ false ].
    
    self class == other class ifFalse: [ ^ false ].
    ^ string = other asString
! !

!JSJsonNumber methodsFor: 'converting' stamp: 'StefanMarr 4/3/2015 10:57'!
asDouble
	^ Float readFromString: string! !

!JSJsonNumber methodsFor: 'converting' stamp: 'StefanMarr 4/3/2015 10:53'!
asInteger
	^ string asInteger! !

!JSJsonNumber methodsFor: 'converting' stamp: 'StefanMarr 4/3/2015 10:52'!
asString
	^ string! !

!JSJsonNumber methodsFor: 'hash' stamp: 'StefanMarr 4/3/2015 10:58'!
hashcode
	^ string hash! !

!JSJsonNumber methodsFor: 'initialization' stamp: 'StefanMarr 4/3/2015 10:52'!
initializeWith: str
	string := str! !

!JSJsonNumber methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 10:52'!
isNumber
	^ true! !

JSJsonValue subclass: #JSJsonObject
	instanceVariableNames: 'names values table'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Json'!

!JSJsonObject class methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 11:26'!
readFrom: string
	^ (JSJsonValue readFrom: string) asObject! !

!JSJsonObject methodsFor: 'comparing' stamp: 'StefanMarr 4/3/2015 11:16'!
= other
    self == other ifTrue: [ ^ true  ].
    self == nil   ifTrue: [ ^ false ].
    self class == other class ifFalse: [ ^ false ].
    
    ^ names = other names & values = other values! !

!JSJsonObject methodsFor: 'adding' stamp: 'StefanMarr 4/3/2015 13:05'!
add: name with: aJsonValue
    name ifNil: [ self error: 'name is null'  ].
    aJsonValue ifNil: [ self error: 'aJsonValue is null' ].

    names  add: name.
    values add: aJsonValue.
    table at: name put: names size.! !

!JSJsonObject methodsFor: 'adding' stamp: 'StefanMarr 4/3/2015 11:09'!
add: name withBoolean: value
    self add: name with: (JSJsonValue boolean: value)! !

!JSJsonObject methodsFor: 'adding' stamp: 'StefanMarr 4/3/2015 11:09'!
add: name withNumber: value
    self add: name with: (JSJsonValue number: value)! !

!JSJsonObject methodsFor: 'adding' stamp: 'StefanMarr 4/3/2015 11:09'!
add: name withString: value
    self add: name with: (JSJsonValue string: value)! !

!JSJsonObject methodsFor: 'converting' stamp: 'StefanMarr 4/3/2015 11:15'!
asObject
	^ self! !

!JSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:14'!
at: name
    | idx |
    name ifNil: [ self error: 'name is null'  ].
    idx := self indexOf: name.
    idx = -1
      ifTrue:  [ ^ nil ]
      ifFalse: [ ^ values at: idx ]! !

!JSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:14'!
at: name asBooleanWith: default
    | value |
    value := self at: name.
    value
      ifNil:    [ ^ default ]
      ifNotNil: [ ^ value asBoolean ]! !

!JSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:14'!
at: name asDoubleWith: default
    | value |
    value := self at: name.
    value
      ifNil:    [ ^ default ]
      ifNotNil: [ ^ value asDouble ]! !

!JSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:14'!
at: name asIntegerWith: default
    | value |
    value := self at: name.
    value
      ifNil:    [ ^ default ]
      ifNotNil: [ ^ value asInteger ]! !

!JSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:14'!
at: name asStringWith: default
    | value |
    value := self at: name.
    value
      ifNil:    [ ^ default ]
      ifNotNil: [ ^ value asString ]! !

!JSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:12'!
at: name put: aJsonValue
    | idx |
    name ifNil: [ self error: 'name is null'  ].
    aJsonValue ifNil: [ self error: 'aJsonValue is null' ].
  
    idx := self indexOf: name.
    idx <> -1
      ifTrue:  [values at: idx put: aJsonValue]
      ifFalse: [
        names  append: name.
        values append: aJsonValue.
        table at: name put: names size.
      ].! !

!JSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:11'!
at: name putBoolean: value
    self at: name put: (JSJsonValue boolean: value)! !

!JSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:11'!
at: name putNumber: value
    self at: name put: (JSJsonValue number: value)! !

!JSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:11'!
at: name putString: value
    self at: name put: (JSJsonValue string: value)! !

!JSJsonObject methodsFor: 'hash' stamp: 'StefanMarr 4/3/2015 11:15'!
hashcode
	| result |
    result := 1.
    result := 31 * result + names hash.
    result := 31 * result + values hash.
    ^ result
! !

!JSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 13:15'!
indexOf: name
    | idx |
    idx := table at: name.    
    idx ~= -1 & name = (names at: idx) ifTrue: [ ^ idx ].
    ^ names lastIndexOf: name! !

!JSJsonObject methodsFor: 'initialization' stamp: 'StefanMarr 4/3/2015 11:08'!
initialize
    names  := OrderedCollection new.
    values := OrderedCollection new.
    table  := JSHashIndexTable new
! !

!JSJsonObject methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 11:15'!
isEmpty
	^ names isEmpty! !

!JSJsonObject methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 11:15'!
isObject
	^ true! !

!JSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:15'!
names
	^ names! !

!JSJsonObject methodsFor: 'removing' stamp: 'StefanMarr 4/3/2015 11:14'!
remove: name
    | idx |
    name ifNil: [ self error: 'name is null' ].
    idx := self indexOf: name.
    idx = -1 ifFalse: [
      table  remove: name.
      names  remove: idx.
      values remove: idx.
    ].! !

!JSJsonObject methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:15'!
size
	^ names size! !

!JSJsonObject methodsFor: 'hash' stamp: 'StefanMarr 4/3/2015 11:17'!
updateHashIndex
    | size |
    size := names size.
    1 to: size do: [: i |
      table add: (names at: i) put: i
    ]! !

JSJsonValue subclass: #JSJsonString
	instanceVariableNames: 'string'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Json'!

!JSJsonString class methodsFor: 'instance creation' stamp: 'StefanMarr 4/2/2015 22:33'!
new: str
	^ self basicNew initializeWith: str! !

!JSJsonString methodsFor: 'comparing' stamp: 'StefanMarr 4/2/2015 22:35'!
= other
	self == other ifTrue: [ ^ true ].
	other == nil ifTrue: [ ^ false ].

	self class == other class ifFalse: [ ^ false ].

	^ string =  other asString! !

!JSJsonString methodsFor: 'converting' stamp: 'StefanMarr 4/2/2015 22:34'!
asString
	^ string! !

!JSJsonString methodsFor: 'hash' stamp: 'StefanMarr 4/2/2015 22:34'!
hashcode
	^ string hash! !

!JSJsonString methodsFor: 'initialization' stamp: 'StefanMarr 4/2/2015 22:34'!
initializeWith: str
	string := str! !

!JSJsonString methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:34'!
isString
	^ true! !

!JSJsonValue class methodsFor: 'instance creation' stamp: 'StefanMarr 4/2/2015 22:31'!
boolean: aBoolean
	^ aBoolean
		ifTrue:  [ JSJsonLiteral TRUE ]
		ifFalse: [ JSJsonLiteral FALSE ]! !

!JSJsonValue class methodsFor: 'helper' stamp: 'StefanMarr 4/2/2015 22:32'!
cutOffPointZero: str
    (str endsWith: '.0') ifTrue: [ ^ str substringFrom: 1 to: str length - 2].
    ^ str! !

!JSJsonValue class methodsFor: 'instance creation' stamp: 'StefanMarr 4/2/2015 22:30'!
double: aDouble
	aDouble ifNil: [ self error: 'aDouble nil' ].
	^ JSJsonNumber new: (self cutOffPointZero: aDouble asString)! !

!JSJsonValue class methodsFor: 'instance creation' stamp: 'StefanMarr 4/2/2015 22:28'!
integer: anInteger
	^ JSJsonNumber new: anInteger asString! !

!JSJsonValue class methodsFor: 'instance creation' stamp: 'StefanMarr 4/2/2015 22:28'!
readFrom: str 
	^ (JSJsonParser with: str) parse! !

!JSJsonValue class methodsFor: 'instance creation' stamp: 'StefanMarr 4/2/2015 22:31'!
string: aString
	^ aString == nil
		ifTrue:  [ JSJsonLiteral NULL ]
		ifFalse: [ JSJsonString new: aString ]! !

!JSJsonValue methodsFor: 'converting' stamp: 'StefanMarr 4/2/2015 22:26'!
asArray
	self error: 'Unsupported operation, not an array ' + self asString! !

!JSJsonValue methodsFor: 'converting' stamp: 'StefanMarr 4/2/2015 22:27'!
asBoolean
	self error: 'Unsupported operation, not a boolean ' + self asString! !

!JSJsonValue methodsFor: 'converting' stamp: 'StefanMarr 4/2/2015 22:26'!
asDouble
	self error: 'Unsupported operation, not a double ' + self asString! !

!JSJsonValue methodsFor: 'converting' stamp: 'StefanMarr 4/2/2015 22:26'!
asInteger
	self error: 'Unsupported operation, not an integer ' + self asString! !

!JSJsonValue methodsFor: 'converting' stamp: 'StefanMarr 4/2/2015 22:26'!
asObject
	self error: 'Unsupported operation, not an object: ' + self asString! !

!JSJsonValue methodsFor: 'converting' stamp: 'StefanMarr 4/2/2015 22:27'!
asString
	self error: 'Unsupported operation, not a string ' + self asString! !

!JSJsonValue methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:25'!
isArray
	^ false! !

!JSJsonValue methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:26'!
isBoolean
	^ false! !

!JSJsonValue methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:26'!
isFalse
	^ false! !

!JSJsonValue methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:26'!
isNull
	^ false! !

!JSJsonValue methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:25'!
isNumber
	^ false! !

!JSJsonValue methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:25'!
isObject
	^ false! !

!JSJsonValue methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:25'!
isString
	^ false! !

!JSJsonValue methodsFor: 'testing' stamp: 'StefanMarr 4/2/2015 22:26'!
isTrue
	^ false! !

!JSJsonValue methodsFor: 'writing' stamp: 'StefanMarr 4/2/2015 22:27'!
writeOn: writer
	^ self subclassResponsibility! !

!JSJsonValue methodsFor: 'writing' stamp: 'StefanMarr 4/2/2015 22:27'!
writeTo: writer
	self writeTo: writer with: nil! !

Object subclass: #JSParseException
	instanceVariableNames: 'msg offset line column'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Json'!

!JSParseException methodsFor: 'converting' stamp: 'StefanMarr 4/3/2015 11:41'!
asString ^ msg, ':', line, ':', column! !

!JSParseException methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:41'!
column ^ column! !

!JSParseException methodsFor: 'initialization' stamp: 'StefanMarr 4/3/2015 11:40'!
initializeWith: message at: anOffset line: aLine column: aColumn
    msg    := message.
    offset := anOffset.
    line   := aLine.
    column := aColumn.! !

!JSParseException methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:41'!
line ^ line! !

!JSParseException methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:40'!
message ^ msg! !

!JSParseException methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 11:40'!
offset ^ offset! !

Object subclass: #NBBody
	instanceVariableNames: 'x y z vx vy vz mass'
	classVariableNames: 'SolarMass'
	poolDictionaries: ''
	category: 'SMark-Benchmarks-NBody'!

!NBBody class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:18'!
DaysPerYear
	^ 365.24! !

!NBBody class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:18'!
Pi
	^ 3.141592653589793! !

!NBBody class methodsFor: 'as yet unclassified' stamp: 'tfel 5/11/2015 15:47'!
SolarMass
	^ SolarMass! !

!NBBody class methodsFor: 'as yet unclassified' stamp: 'tfel 5/11/2015 15:47'!
initialize
	"self initialize"
	SolarMass := 4 * self Pi * self Pi.
	! !

!NBBody class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:19'!
jupiter
		| b |
       b := super new.
       b x:    4.8414314424647209.
       b y:   -1.16032004402742839.
       b z:   -0.103622044471123109.
       b vx:   0.00166007664274403694   * self DaysPerYear.
       b vy:   0.00769901118419740425   * self DaysPerYear.
       b vz:  -0.0000690460016972063023 * self DaysPerYear.
       b mass: 0.000954791938424326609  * self SolarMass.
       ^ b! !

!NBBody class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:20'!
neptune
		| b |
       b := super new.
       b x:   15.3796971148509165.
       b y:  -25.9193146099879641.
       b z:    0.179258772950371181.
       b vx:   0.00268067772490389322   * self DaysPerYear.
       b vy:   0.00162824170038242295   * self DaysPerYear.
       b vz:  -0.000095159225451971587  * self DaysPerYear.
       b mass: 0.0000515138902046611451 * self SolarMass.
       ^ b! !

!NBBody class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:19'!
new
	| b |
        b := super new.
        b x: 0.0.  b vx: 0.0.
        b y: 0.0.  b vy: 0.0.
        b z: 0.0.  b vz: 0.0.
        b mass: 0.0.
        ^ b! !

!NBBody class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:19'!
saturn 
		| b |
       b := super new.
       b x:    8.34336671824457987.
       b y:    4.12479856412430479.
       b z:   -0.403523417114321381.
       b vx:  -0.00276742510726862411   * self DaysPerYear.
       b vy:   0.00499852801234917238   * self DaysPerYear.
       b vz:   0.0000230417297573763929 * self DaysPerYear.
       b mass: 0.000285885980666130812  * self SolarMass.
       ^ b! !

!NBBody class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:20'!
sun 
		| b |
       b := self new.
       b mass: self SolarMass.
       ^ b! !

!NBBody class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:19'!
uranus
		| b |
       b := super new.
       b x:   12.894369562139131.
       b y:  -15.1111514016986312.
       b z:   -0.223307578892655734.
       b vx:   0.00296460137564761618   * self DaysPerYear.
       b vy:   0.0023784717395948095    * self DaysPerYear.
       b vz:  -0.0000296589568540237556 * self DaysPerYear.
       b mass: 0.0000436624404335156298 * self SolarMass.
       ^ b! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:16'!
mass ^ mass! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:17'!
mass: val
	mass := val! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:17'!
offsetMomentumX: px y: py z: pz
       vx := 0.0 - (px / NBBody SolarMass).
       vy := 0.0 - (py / NBBody SolarMass).
       vz := 0.0 - (pz / NBBody SolarMass).! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:15'!
vx ^ vx! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:16'!
vx: val
	vx := val! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:16'!
vy ^ vy! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:16'!
vy: val
	vy := val! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:16'!
vz ^ vz! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:17'!
vz: val
	vz := val! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:15'!
x ^ x! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:16'!
x: val
	x := val! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:15'!
y ^ y! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:16'!
y: val
	y := val! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:15'!
z ^ z! !

!NBBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:16'!
z: val
	z := val! !

Object subclass: #NBNBodySystem
	instanceVariableNames: 'bodies'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-NBody'!

!NBNBodySystem methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:21'!
advance: dt
        1 to: bodies size do: [:i |
            | iBody |
            iBody := bodies at: i.

            i + 1 to: bodies size do: [:j |
                | dx dy dz jBody dSquared distance mag |
                jBody := bodies at: j.
                dx := iBody x - jBody x.
                dy := iBody y - jBody y.
                dz := iBody z - jBody z.

                dSquared := (dx * dx) + (dy * dy) + (dz * dz).
                distance := dSquared sqrt.
                mag      := dt / (dSquared * distance).

                iBody vx: iBody vx - (dx * jBody mass * mag).
                iBody vy: iBody vy - (dy * jBody mass * mag).
                iBody vz: iBody vz - (dz * jBody mass * mag).

                jBody vx: jBody vx + (dx * iBody mass * mag).
                jBody vy: jBody vy + (dy * iBody mass * mag).
                jBody vz: jBody vz + (dz * iBody mass * mag).
            ].
       ].
       
       bodies do: [:body |
           body x: body x + (dt * body vx).
           body y: body y + (dt * body vy).
           body z: body z + (dt * body vz).
       ].! !

!NBNBodySystem methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:21'!
energy
        | dx dy dz distance e |
        e := 0.0.

        1 to: bodies size do: [:i |
            | iBody |
            iBody := bodies at: i.
            
            e := e + (0.5 * iBody mass *
                 ((iBody vx * iBody vx) +
                  (iBody vy * iBody vy) +
                  (iBody vz * iBody vz))).
            
            i + 1 to: bodies size do: [:j |
                | jBody |
                jBody := bodies at: j.
                
                dx := iBody x - jBody x.
                dy := iBody y - jBody y.
                dz := iBody z - jBody z.

                distance := ((dx*dx) + (dy*dy) + (dz*dz)) sqrt.
                e := e - ((iBody mass * jBody mass) / distance).
            ].
       ].
       ^ e! !

!NBNBodySystem methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:14'!
initialize
        | px py pz |

        bodies := Array new: 5.
        bodies at: 1 put: NBBody sun.
        bodies at: 2 put: NBBody jupiter.
        bodies at: 3 put: NBBody saturn.
        bodies at: 4 put: NBBody uranus.
        bodies at: 5 put: NBBody neptune.
        
        "bodies do: [:b | b print. '' println ]."

        px := 0.0.  py := 0.0.  pz := 0.0.

        bodies do: [:b |
            px := px + (b vx * b mass).
            py := py + (b vy * b mass).
            pz := pz + (b vz * b mass).
        ].
        
        (bodies at: 1) offsetMomentumX: px y: py z: pz.
        
        "bodies do: [:b | b print. '' println ]."! !

Object subclass: #RichObject
	instanceVariableNames: ''
	classVariableNames: 'DeviceA DeviceB DevicePacketKind HandlerA HandlerB Idler NoTask NoWork WorkPacketKind Worker'
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Richards'!

!RichObject commentStamp: '<historical>' prior: 0!
This class mostly adds some constants that are used in the Richards benchmarks.!

RichObject subclass: #RichDeviceTaskDataRecord
	instanceVariableNames: 'pending'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Richards'!

!RichDeviceTaskDataRecord commentStamp: '<historical>' prior: 0!
A task that suspends itself after each time it has been run to simulate waiting for data from an external device.
!

!RichDeviceTaskDataRecord class methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 13:40'!
create
	^ super basicNew create! !

!RichDeviceTaskDataRecord methodsFor: 'initialization' stamp: 'StefanMarr 4/3/2015 13:39'!
create
	pending := RichObject NoWork! !

!RichDeviceTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:10'!
pending
	^ pending! !

!RichDeviceTaskDataRecord methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 13:39'!
pending: packet
	pending := packet! !

RichObject subclass: #RichHandlerTaskDataRecord
	instanceVariableNames: 'workIn deviceIn'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Richards'!

!RichHandlerTaskDataRecord commentStamp: '<historical>' prior: 0!
A task that manipulates work packets and then suspends itself.!

!RichHandlerTaskDataRecord class methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 15:03'!
create
	^ super basicNew create! !

!RichHandlerTaskDataRecord methodsFor: 'converting' stamp: 'StefanMarr 4/3/2015 13:42'!
asString
	^ 'HandlerTaskDataRecord(', workIn asString, ', ', deviceIn asString, ')'! !

!RichHandlerTaskDataRecord methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 13:42'!
create
	workIn := deviceIn := RichObject NoWork! !

!RichHandlerTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:10'!
deviceIn
	^ deviceIn! !

!RichHandlerTaskDataRecord methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 13:41'!
deviceIn: aPacket
	deviceIn := aPacket! !

!RichHandlerTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:10'!
deviceInAdd: packet
    deviceIn := self append: packet head: deviceIn! !

!RichHandlerTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:10'!
workIn
	^ workIn! !

!RichHandlerTaskDataRecord methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 13:41'!
workIn: aWorkQueue
	workIn := aWorkQueue! !

!RichHandlerTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:11'!
workInAdd: packet
    workIn := self append: packet head: workIn! !

RichObject subclass: #RichIdleTaskDataRecord
	instanceVariableNames: 'control count'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Richards'!

!RichIdleTaskDataRecord commentStamp: '<historical>' prior: 0!
An idle task doesn't do any work itself but cycles control between the two device tasks.!

!RichIdleTaskDataRecord class methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 13:43'!
create
	^ super basicNew create! !

!RichIdleTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:18'!
control
	^ control! !

!RichIdleTaskDataRecord methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 13:43'!
control: aNumber
	control := aNumber! !

!RichIdleTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:18'!
count
	^ count! !

!RichIdleTaskDataRecord methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 13:43'!
count: aCount
	count := aCount! !

!RichIdleTaskDataRecord methodsFor: 'initialize' stamp: 'StefanMarr 4/3/2015 15:04'!
create
    control := 1.
    count := 10000! !

!RichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:38'!
DeviceA
	^ DeviceA! !

!RichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:38'!
DeviceB
	^ DeviceB! !

!RichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:38'!
DevicePacketKind
	^ DevicePacketKind! !

!RichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:38'!
HandlerA
	^ HandlerA! !

!RichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:38'!
HandlerB
	^ HandlerB! !

!RichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:37'!
Idler
	^ Idler! !

!RichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:37'!
NoTask
	^ NoTask! !

!RichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:37'!
NoWork
	^ NoWork! !

!RichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:38'!
WorkPacketKind
	^ WorkPacketKind! !

!RichObject class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 13:38'!
Worker
	^ Worker! !

!RichObject class methodsFor: 'initialize' stamp: 'StefanMarr 4/3/2015 13:37'!
initialize
    	DeviceA := 5.
    	DeviceB := 6.
    	DevicePacketKind := 1.
    	HandlerA := 3.
    	HandlerB := 4.
    	Idler := 1.
	   NoWork := nil.
	   NoTask := nil.
    	Worker := 2.
    	WorkPacketKind := 2! !

!RichObject methodsFor: 'utilities' stamp: 'StefanMarr 4/3/2015 13:35'!
append: packet head: queueHead
    | mouse link |
    packet link: RichObject NoWork.
    RichObject NoWork == queueHead ifTrue: [ ^ packet ].
    mouse := queueHead.
    [RichObject NoWork == (link := mouse link)]
        whileFalse: [ mouse := link ].
    mouse link: packet.
    ^ queueHead! !

RichObject subclass: #RichPacket
	instanceVariableNames: 'data datum identity kind link'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Richards'!

!RichPacket commentStamp: '<historical>' prior: 0!
A simple package of data that is manipulated by the tasks.  The exact layout of the payload data carried by a packet is not importaint, and neither is the nature of the work performed on packets by the tasks. Besides carrying data, packets form linked lists and are hence used both as data and worklists.!

!RichPacket class methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 13:44'!
create: link identity: identity kind: kind 
    ^ self basicNew
		link: link
		identity: identity
		kind: kind! !

!RichPacket methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 15:21'!
data
	^ data! !

!RichPacket methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 15:21'!
datum
	^ datum! !

!RichPacket methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 15:21'!
datum: anObject
	datum := anObject! !

!RichPacket methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 15:21'!
identity
	^ identity! !

!RichPacket methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 15:21'!
identity: anObject
	identity := anObject! !

!RichPacket methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 15:21'!
kind
	^ kind! !

!RichPacket methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 15:21'!
link
	^ link! !

!RichPacket methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 13:45'!
link: aWorkQueue
	link := aWorkQueue! !

!RichPacket methodsFor: 'initialize' stamp: 'ClementBera 9/27/2013 15:20'!
link: aLink identity: anIdentity kind: aKind 
    link := aLink.
    identity := anIdentity. 
    kind := aKind.
    datum := 1.
    data := ByteArray new: 4! !

RichObject subclass: #RichTaskState
	instanceVariableNames: 'packetPending taskHolding taskWaiting'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Richards'!

!RichTaskState commentStamp: '<historical>' prior: 0!
Abstract task that manipulates work packets.!

RichTaskState subclass: #RichTaskControlBlock
	instanceVariableNames: 'link identity priority input function handle'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Richards'!

!RichTaskControlBlock commentStamp: '<historical>' prior: 0!
A task control block manages a task and the queue of work packages associated with it.!

!RichTaskControlBlock class methodsFor: 'instance creation' stamp: 'ClementBera 9/27/2013 16:17'!
link: link create: identity priority: priority initialWorkQueue: initialWorkQueue initialState: initialState function: aBlock privateData: privateData 
    ^ self new
		link: link
		identity: identity
		priority: priority
		initialWorkQueue: initialWorkQueue
		initialState: initialState
		function: aBlock
		privateData: privateData! !

!RichTaskControlBlock methodsFor: 'scheduling' stamp: 'StefanMarr 4/3/2015 14:35'!
addInput: packet checkPriority: oldTask
    RichObject NoWork == input
	ifTrue: 
	    [input := packet.
	    self packetPending: true.
	    priority > oldTask priority ifTrue: [ ^ self ]]
	ifFalse: 
	    [ input := self append: packet head: input ].
    ^ oldTask! !

!RichTaskControlBlock methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:13'!
identity
	^ identity! !

!RichTaskControlBlock methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:13'!
link
	^ link! !

!RichTaskControlBlock methodsFor: 'initialize' stamp: 'StefanMarr 4/3/2015 15:16'!
link: aLink identity: anIdentity priority: aPriority initialWorkQueue: anInitialWorkQueue initialState: anInitialState function: aBlock privateData: aPrivateData 
    link := aLink.
    identity := anIdentity.
    priority := aPriority.
    input := anInitialWorkQueue.
    self packetPending: anInitialState isPacketPending.
    self taskWaiting: anInitialState isTaskWaiting.
    self taskHolding: anInitialState isTaskHolding.
    function := aBlock.
    handle := aPrivateData! !

!RichTaskControlBlock methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:13'!
priority
	^ priority! !

!RichTaskControlBlock methodsFor: 'scheduling' stamp: 'StefanMarr 4/3/2015 14:35'!
runTask
    | message |
    self isWaitingWithPacket
	ifTrue: 
	    [message := input.
	    input := message link.
	    RichObject NoWork == input 
		ifTrue: [self running]
		ifFalse: [self packetPending]]
	ifFalse: [message := RichObject NoWork].
    ^ function value: message value: handle! !

!RichTaskState class methodsFor: 'instance creation' stamp: 'ClementBera 9/27/2013 16:09'!
packetPending
    ^super new packetPending! !

!RichTaskState class methodsFor: 'instance creation' stamp: 'ClementBera 9/27/2013 16:09'!
running
    ^super new running! !

!RichTaskState class methodsFor: 'instance creation' stamp: 'ClementBera 9/27/2013 16:09'!
waiting
    ^super new waiting! !

!RichTaskState class methodsFor: 'instance creation' stamp: 'ClementBera 9/27/2013 16:09'!
waitingWithPacket
    ^super new waitingWithPacket! !

!RichTaskState methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 14:36'!
isPacketPending
    ^packetPending! !

!RichTaskState methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 14:38'!
isRunning
    ^ packetPending not and: [taskWaiting not and: [taskHolding not]]! !

!RichTaskState methodsFor: 'testing' stamp: 'ClementBera 9/27/2013 16:06'!
isTaskHolding
    ^taskHolding! !

!RichTaskState methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 14:38'!
isTaskHoldingOrWaiting
    ^taskHolding or: [packetPending not and: [taskWaiting]]! !

!RichTaskState methodsFor: 'testing' stamp: 'ClementBera 9/27/2013 16:06'!
isTaskWaiting
    ^taskWaiting! !

!RichTaskState methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 14:39'!
isWaiting
    ^packetPending not and: [taskWaiting and: [taskHolding not]]! !

!RichTaskState methodsFor: 'testing' stamp: 'StefanMarr 4/3/2015 14:39'!
isWaitingWithPacket
    ^packetPending and: [taskWaiting and: [taskHolding not]]! !

!RichTaskState methodsFor: 'initialize' stamp: 'StefanMarr 4/3/2015 14:37'!
packetPending
    packetPending := true.
    taskWaiting := false.
    taskHolding := false! !

!RichTaskState methodsFor: 'initialize' stamp: 'StefanMarr 4/3/2015 14:37'!
packetPending: aBoolean
	packetPending := aBoolean! !

!RichTaskState methodsFor: 'initialize' stamp: 'StefanMarr 4/3/2015 14:38'!
running
    packetPending := taskWaiting := taskHolding := false! !

!RichTaskState methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 14:37'!
taskHolding: aBoolean
	^ taskHolding := aBoolean! !

!RichTaskState methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 14:37'!
taskWaiting: aBoolean
	^ taskWaiting := aBoolean! !

!RichTaskState methodsFor: 'initialize' stamp: 'StefanMarr 4/3/2015 14:38'!
waiting
    packetPending := taskHolding := false.
    taskWaiting := true! !

!RichTaskState methodsFor: 'initialize' stamp: 'StefanMarr 4/3/2015 14:38'!
waitingWithPacket
    taskHolding := false.
    taskWaiting := packetPending := true! !

RichObject subclass: #RichWorkerTaskDataRecord
	instanceVariableNames: 'destination count'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Richards'!

!RichWorkerTaskDataRecord commentStamp: '<historical>' prior: 0!
A task that manipulates work packets.!

!RichWorkerTaskDataRecord class methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 14:40'!
create
	^ super basicNew create! !

!RichWorkerTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:11'!
count
	^ count! !

!RichWorkerTaskDataRecord methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 15:16'!
count: anInteger
	count := anInteger! !

!RichWorkerTaskDataRecord methodsFor: 'instance creation' stamp: 'StefanMarr 4/3/2015 14:41'!
create
    destination := RichObject HandlerA.
    count := 0! !

!RichWorkerTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:11'!
destination
	^ destination! !

!RichWorkerTaskDataRecord methodsFor: 'accessing' stamp: 'ClementBera 9/27/2013 16:11'!
destination: anObject
	destination := anObject! !

Object subclass: #RichRunner
	instanceVariableNames: 'taskList currentTask currentTaskIdentity taskTable tracing layout queuePacketCount holdCount'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Richards'!

!RichRunner commentStamp: '<historical>' prior: 0!
Richards simulates the task dispatcher of an operating system.!

!RichRunner class methodsFor: 'starting' stamp: 'ClementBera 9/27/2013 15:58'!
start
    "RichardsBenchmark start"

    ^self new start! !

!RichRunner methodsFor: 'creation' stamp: 'StefanMarr 4/3/2015 13:53'!
createDevice: identity priority: priority work: work state: state 
    | data |
    data := RichDeviceTaskDataRecord create.
    self
	createTask: identity
	priority: priority
	work: work
	state: state
	function: 
	    [:work1 :word | | data1 functionWork |
	    data1 := word.
	    functionWork := work1.
	    RichObject NoWork == functionWork 
		ifTrue:
		    [RichObject NoWork == (functionWork := data1 pending)
			ifTrue: [self wait]
			ifFalse: 
			    [data1 pending: RichObject NoWork.
			    self queuePacket: functionWork]]
		ifFalse: 
		    [data1 pending: functionWork.
			tracing ifTrue: [ self trace: functionWork datum ].
		    self holdSelf]]
	data: data! !

!RichRunner methodsFor: 'creation' stamp: 'StefanMarr 4/3/2015 15:15'!
createHandler: identity priority: priority work: work state: state 
    | data |
    data := RichHandlerTaskDataRecord create.
    self
	createTask: identity
	priority: priority
	work: work
	state: state
	function: 
	    [:work1 :word | | data1 workPacket |
	    data1 := word.
	    RichObject NoWork == work1 
		ifFalse: [RichObject WorkPacketKind == work1 kind 
		    ifTrue: [data1 workInAdd: work1]
		    ifFalse: [data1 deviceInAdd: work1]].
	    RichObject NoWork == (workPacket := data1 workIn) 
		ifTrue: [self wait]
		ifFalse: 
		    [ | count |
			count := workPacket datum.
		    count > 4
			ifTrue: 
			    [data1 workIn: workPacket link.
			    self queuePacket: workPacket]
			ifFalse:
			    [ | devicePacket |
				RichObject NoWork == (devicePacket := data1 deviceIn) 
				ifTrue: [self wait]
				ifFalse: 
				    [data1 deviceIn: devicePacket link.
				    devicePacket datum: (workPacket data at: count).
				    workPacket datum: count + 1.
				    self queuePacket: devicePacket]]]]
	data: data! !

!RichRunner methodsFor: 'creation' stamp: 'StefanMarr 4/3/2015 15:14'!
createIdler: identity priority: priority work: work state: state 
    | data |
    data := RichIdleTaskDataRecord create.
    self
	createTask: identity
	priority: priority
	work: work
	state: state
	function: 
	    [:work1 :word | | data1 |
	    data1 := word.
	    data1 count: data1 count - 1.
	    0 = data1 count
		ifTrue: [self holdSelf]
		ifFalse:
		    [0 = (data1 control bitAnd: 1)
			ifTrue: 
			    [data1 control: data1 control // 2.
			    self release: RichObject DeviceA]
			ifFalse: 
			    [data1 control: (data1 control // 2 bitXor: 53256).
			    self release: RichObject DeviceB]]]
	data: data! !

!RichRunner methodsFor: 'creation' stamp: 'ClementBera 9/27/2013 15:27'!
createPacket: link identity: identity kind: kind 
    ^ RichPacket
		create: link
		identity: identity
		kind: kind! !

!RichRunner methodsFor: 'creation' stamp: 'ClementBera 9/27/2013 15:28'!
createTask: identity priority: priority work: work state: state function: aBlock data: data 
    | t |
    t := RichTaskControlBlock
		link: taskList
		create: identity
		priority: priority
		initialWorkQueue: work
		initialState: state
		function: aBlock
		privateData: data.
    taskList := t.
    taskTable at: identity put: t! !

!RichRunner methodsFor: 'creation' stamp: 'StefanMarr 4/3/2015 15:15'!
createWorker: identity priority: priority work: work state: state 
    | data |
    data := RichWorkerTaskDataRecord create.
    self
	createTask: identity
	priority: priority
	work: work
	state: state
	function: 
	    [:work1 :word | | data1 |
	    data1 := word.
	    RichObject NoWork == work1
		ifTrue: [self wait]
		ifFalse: 
		    [data1 destination: (RichObject HandlerA = data1 destination
			    ifTrue:  [RichObject HandlerB]
			    ifFalse: [RichObject HandlerA]).
		    work1 identity: data1 destination.
		    work1 datum: 1.
		    1 to: 4 do: [ :i | 
			data1 count: data1 count + 1.
			data1 count > 26 ifTrue: [data1 count: 1].
			work1 data at: i put: $A asInteger + data1 count - 1].
		    self queuePacket: work1]]
	data: data! !

!RichRunner methodsFor: 'private' stamp: 'StefanMarr 4/3/2015 14:07'!
findTask: identity 
    | t |
    t := taskTable at: identity.
    RichObject NoTask == t ifTrue: [self error: 'findTask failed'].
    ^ t! !

!RichRunner methodsFor: 'private' stamp: 'ClementBera 9/27/2013 15:35'!
holdSelf
    holdCount := holdCount + 1.
    currentTask taskHolding: true.
    ^ currentTask link! !

!RichRunner methodsFor: 'private' stamp: 'StefanMarr 4/3/2015 14:08'!
initScheduler
    queuePacketCount := 0.
    holdCount := 0.
    taskTable := Array new: 6 withAll: RichObject NoTask.
    taskList := RichObject NoTask.! !

!RichRunner methodsFor: 'private' stamp: 'StefanMarr 4/3/2015 14:08'!
initTrace
	tracing := false.
	layout := 0! !

!RichRunner methodsFor: 'private' stamp: 'StefanMarr 4/3/2015 14:09'!
queuePacket: packet 
    | t |
    t := self findTask: packet identity.
    RichObject NoTask ==  t ifTrue: [ ^ RichObject NoTask ].
    queuePacketCount := queuePacketCount + 1.
    packet link: RichObject NoWork.
    packet identity: currentTaskIdentity.
    ^ t addInput: packet checkPriority: currentTask! !

!RichRunner methodsFor: 'private' stamp: 'StefanMarr 4/3/2015 14:10'!
release: identity 
    | t |
    t := self findTask: identity.
    RichObject NoTask == t ifTrue: [ ^ RichObject NoTask ].
    t taskHolding: false.
    t priority > currentTask priority
		ifTrue: [ ^ t ]
		ifFalse: [ ^ currentTask ]! !

!RichRunner methodsFor: 'scheduling' stamp: 'StefanMarr 4/3/2015 14:11'!
schedule
    currentTask := taskList. 
    [RichObject NoTask == currentTask]
		whileFalse: 
	    [currentTask isTaskHoldingOrWaiting
			ifTrue: [currentTask := currentTask link]
			ifFalse: 
		    [currentTaskIdentity := currentTask identity.
			 tracing ifTrue: [ self trace: currentTaskIdentity ].
		    currentTask := currentTask runTask]]! !

!RichRunner methodsFor: 'initialize' stamp: 'StefanMarr 4/3/2015 14:44'!
start
    | workQ |
	 self initTrace.
    self initScheduler.
    self 	createIdler: RichObject Idler
			priority: 0
			work: RichObject NoWork
			state: RichTaskState running.
    workQ := self
		createPacket: RichObject NoWork
		identity: RichObject Worker
		kind: RichObject WorkPacketKind.
    workQ := self
		createPacket: workQ
		identity: RichObject Worker
		kind: RichObject WorkPacketKind.
    self
		createWorker: RichObject Worker
		priority: 1000
		work: workQ
		state: RichTaskState waitingWithPacket.
    workQ := self
		createPacket: RichObject NoWork
		identity: RichObject DeviceA
		kind: RichObject DevicePacketKind.
    workQ := self
		createPacket: workQ
		identity: RichObject DeviceA
		kind: RichObject DevicePacketKind.
    workQ := self
		createPacket: workQ
		identity: RichObject DeviceA
		kind: RichObject DevicePacketKind.
    self
		createHandler: RichObject HandlerA
		priority: 2000
		work: workQ
		state: RichTaskState waitingWithPacket.
    workQ := self
		createPacket: RichObject NoWork
		identity: RichObject DeviceB
		kind: RichObject DevicePacketKind.
    workQ := self
		createPacket: workQ
		identity: RichObject DeviceB
		kind: RichObject DevicePacketKind.
    workQ := self
		createPacket: workQ
		identity: RichObject DeviceB
		kind: RichObject DevicePacketKind.
    self
		createHandler: RichObject HandlerB
		priority: 3000
		work: workQ
		state: RichTaskState waitingWithPacket.
    self
		createDevice: RichObject DeviceA
		priority: 4000
		work: RichObject NoWork
		state: RichTaskState waiting.
    self
		createDevice: RichObject DeviceB
		priority: 5000
		work: RichObject NoWork
		state: RichTaskState waiting.
    
	 self schedule.
    ^ (queuePacketCount = 23246) & (holdCount = 9297)
! !

!RichRunner methodsFor: 'private' stamp: 'ClementBera 9/27/2013 15:40'!
wait 
    currentTask taskWaiting: true.
    ^ currentTask! !

Object subclass: #SJenkinsRandom
	instanceVariableNames: ''
	classVariableNames: 'seed'
	poolDictionaries: ''
	category: 'SMark-Benchmarks-SOM'!

!SJenkinsRandom methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 16:19'!
random
	"Robert Jenkins' 32 bit integer hash function."
	 seed := ((seed       + 2127912214 "0x7ed55d16")       + (seed as32BitUnsignedValue  << 12) as32BitSignedValue) as32BitSignedValue.
    seed := ((seed bitXor: 3345072700 "0xc761c23c") bitXor: (seed as32BitUnsignedValue >>> 19)) as32BitSignedValue.
    seed := ((seed       +  374761393 "0x165667B1")       + (seed as32BitUnsignedValue  <<  5) as32BitSignedValue) as32BitSignedValue.
    seed := ((seed       + 3550635116 "0xd3a2646c") bitXor: (seed as32BitUnsignedValue  <<  9) as32BitSignedValue) as32BitSignedValue.
    seed := ((seed       + 4251993797 "0xfd7046c5")       + (seed as32BitUnsignedValue  <<  3) as32BitSignedValue) as32BitSignedValue.
    seed := ((seed bitXor: 3042594569 "0xb55a4f09") bitXor: (seed as32BitUnsignedValue >>> 16)) as32BitSignedValue.
    ^ seed! !

!SJenkinsRandom methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 16:37'!
seed: val
	seed := ThirtyTwoBitRegister basicNew load: val! !

Object subclass: #SMarkConsole
	instanceVariableNames: ''
	classVariableNames: 'IsRVM LastPrintedChar OutputToTranscript Position PrintTarget'
	poolDictionaries: ''
	category: 'SMark-GUI'!

!SMarkConsole class methodsFor: 'streaming' stamp: 'tfel 5/12/2015 10:06'!
<< aString
	self print: aString! !

!SMarkConsole class methodsFor: 'printing' stamp: 'tfel 5/12/2015 10:06'!
cr
	self println: ''.! !

!SMarkConsole class methodsFor: 'initialization' stamp: 'tfel 5/15/2015 09:25'!
detectAvailableMechanismForStdout
	"The prefered means is to use the stdout file handle.
	 Will try to fall back on the RoarVM primitive if necessary."
	| cls |
	((FileStream respondsTo: #stdout) and: [[FileStream stdout nextPut: Character space. true] on: Error do: [false]])
		ifTrue: [
			PrintTarget := FileStream stdout.
			IsRVM := false. "Not needed when stdout is used"
			^ self.
		].
		
	cls := Smalltalk at: #RVMOperations ifAbsent: [nil].
	IsRVM := (cls notNil and: [cls perform: #isRVM]).
	IsRVM 
		ifTrue: [ 
			PrintTarget := cls.
			^ self.
		]
		ifFalse: [
			"Squeak 4.2 provides a stream at FileStream>>stdout"
			
		].
	
	PrintTarget := Transcript.
	^ Transcript! !

!SMarkConsole class methodsFor: 'as yet unclassified' stamp: 'tfel 5/12/2015 10:06'!
flush
	PrintTarget ifNotNil: [PrintTarget flush].! !

!SMarkConsole class methodsFor: 'initialization' stamp: 'tfel 5/15/2015 09:25'!
initialize
	"self initialize"
	OutputToTranscript := false.
	self detectAvailableMechanismForStdout.
	Position := 0.! !

!SMarkConsole class methodsFor: 'streaming' stamp: 'tfel 5/12/2015 10:06'!
nextPut: somethingAsString
	self print: somethingAsString! !

!SMarkConsole class methodsFor: 'streaming' stamp: 'tfel 5/12/2015 10:06'!
nextPutAll: somethingAsString
	self print: somethingAsString! !

!SMarkConsole class methodsFor: 'printing' stamp: 'tfel 5/12/2015 10:06'!
outputToTranscript
	^ OutputToTranscript! !

!SMarkConsole class methodsFor: 'printing' stamp: 'tfel 5/12/2015 10:06'!
outputToTranscript: aBool
	OutputToTranscript := aBool! !

!SMarkConsole class methodsFor: 'read, write, position' stamp: 'tfel 5/12/2015 10:06'!
peekLast
	^ LastPrintedChar! !

!SMarkConsole class methodsFor: 'read, write, position' stamp: 'tfel 5/12/2015 10:06'!
position
	^ Position! !

!SMarkConsole class methodsFor: 'printing' stamp: 'tfel 5/12/2015 10:06'!
print: somethingAsString
	self print: somethingAsString withLineEnding: ''.! !

!SMarkConsole class methodsFor: 'printing' stamp: 'tfel 5/15/2015 10:03'!
print: somethingAsString withLineEnding: lineEnd
	"Will output the somethingAsString to stdout using one of the available mechansims and to the transcript if requested"
	| output |
	output := somethingAsString asString.
	Position := Position + output size.
	output size > 0 ifTrue: [
		LastPrintedChar := output last].
	
	PrintTarget ifNotNil: [
		IsRVM ifTrue: [ PrintTarget print: output. ]
			   ifFalse: [ PrintTarget nextPutAll: (output, lineEnd) ].
	].

	(OutputToTranscript or: [PrintTarget isNil])
		ifTrue: [
			Transcript show: output, lineEnd.
		].! !

!SMarkConsole class methodsFor: 'printing' stamp: 'tfel 5/12/2015 10:06'!
println: somethingAsString
	self print: somethingAsString withLineEnding: String crlf.! !

!SMarkConsole class methodsFor: 'printing' stamp: 'tfel 5/12/2015 10:06'!
space
	"Append a space character to the receiver."

	self nextPut: Character space! !

!SMarkConsole class methodsFor: 'printing' stamp: 'tfel 5/12/2015 10:06'!
tab
	"Append a tab character to the receiver."

	self nextPut: Character tab! !

Object subclass: #SMarkHarness
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

!SMarkHarness commentStamp: '<historical>' prior: 0!
A benchmark harness steers the execution and reporting of benchmarks.
For that purpose, it will use a designated benchmark runner to do the execution and a benchmark reporter to output the results.
The benchmark harness is also parameterized by the benchmark suites that are to be executed.

The simplest way to execute a benchmark suite is to use SMarkSuite >> #run.

However, directly using the harness classes gives more freedom on reporting and execution strategies.

A typical call of the harness from the commandline would result in the following invokation:
	SMarkHarness run: {'SMarkHarness'. 'SMarkLoops.benchIntLoop'. 1. 1. 5}!

SMarkHarness subclass: #ReBenchHarness
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

!ReBenchHarness commentStamp: 'StefanMarr 5/16/2011 09:10' prior: 0!
The ReBenchHarness is optimized for use from the command-line.
It is especially meant to be used together with ReBench, a tool to execute and document benchmarks reproducibly.

See: https://github.com/smarr/ReBench#readme!

!ReBenchHarness class methodsFor: 'helper' stamp: 'StefanMarr 12/30/2011 19:59'!
defaultArgumentParser
	^ ReBenchHarnessArgumentParser! !

!ReBenchHarness class methodsFor: 'defaults' stamp: ''!
defaultReporter
	^ ReBenchReporter! !

!ReBenchHarness class methodsFor: 'defaults' stamp: 'sm 5/25/2011 17:06'!
defaultRunner
	^ SMarkWeakScalingRunner! !

!ReBenchHarness class methodsFor: 'helper' stamp: 'sm 5/25/2011 17:20'!
usageBenchmarkParameters: usage
	^ usage,		' processes          optional, number of processes/threads used by the benchmarks', String crlf,
				' inner-iterations   optional, number of iterations done by a single process', String crlf,
				' problemSize        optional, depending on benchmark for instance size of used data set', String crlf.
				! !

!ReBenchHarness class methodsFor: 'helper' stamp: 'sm 5/25/2011 17:20'!
usageHeader
	| usage |
	usage := 'SMark Benchmark Framework, version: ', self version, String crlf.
	usage := usage, String crlf.
	usage := usage, 'Usage: <vm+image> ', self name,
				' <suiteOrBenchmark> [processes [inner-iterations [problemSize]]]', String crlf.
	usage := usage, String crlf.
	
	usage := usage, '  This harness is used for weak-scalling benchmarks.', String crlf.
	usage := usage, '  Use the SMarkHarness for more general settings, it offers more options.', String crlf.
	
	usage := usage, String crlf.
	^ usage! !

!ReBenchHarness class methodsFor: 'helper' stamp: ''!
usageReporter: usage
	"Will rely on default, which is good for ReBench, so do not advertise option."
	^ usage! !

!ReBenchHarness class methodsFor: 'helper' stamp: ''!
usageRunner: usage
	"Will rely on default, which is good for ReBench, so do not advertise option."
	^ usage! !

!SMarkHarness class methodsFor: 'helper' stamp: 'StefanMarr 12/30/2011 19:59'!
defaultArgumentParser
	^ SMarkHarnessArgumentParser! !

!SMarkHarness class methodsFor: 'defaults' stamp: 'tfel 5/12/2015 10:06'!
defaultOutputDestination
	^ Smalltalk at:       #SMarkConsole
	            ifAbsent: [SMarkReporter defaultOutputDestination]! !

!SMarkHarness class methodsFor: 'defaults' stamp: ''!
defaultReporter
	^ SMarkReporter defaultReporter! !

!SMarkHarness class methodsFor: 'defaults' stamp: ''!
defaultRunner
	^ SMarkSuite defaultRunner! !

!SMarkHarness class methodsFor: 'benchmarking' stamp: 'sm 5/25/2011 17:41'!
execute: runner andReport: reporter
	runner reportConfiguration: self defaultOutputDestination.
	runner execute.
	reporter runner: runner.
	reporter outputStream: self defaultOutputDestination.
	reporter report.! !

!SMarkHarness class methodsFor: 'benchmarking' stamp: ''!
execute: aBenchmarkOrSuite using: aRunnerClass andReport: withAReporterClass
	| parsedBenchmarkOrSuite runner reporter |
	
	parsedBenchmarkOrSuite := self parseBenchmarkOrSuite: aBenchmarkOrSuite.
	
	runner := aRunnerClass new.
	reporter := withAReporterClass new.
	self instructRunner: runner with: parsedBenchmarkOrSuite.
	self execute: runner andReport: reporter.   ! !

!SMarkHarness class methodsFor: 'helper' stamp: 'StefanMarr 12/30/2011 20:00'!
parseArguments: arguments
	| parser |
	parser := self defaultArgumentParser new.
	parser harness: self.
	^ parser parse: arguments.! !

!SMarkHarness class methodsFor: 'script entry' stamp: 'StephaneDucasse 6/16/2013 20:45'!
run: arguments
	"Executed from the command line using something similar to
	 ./vm my.image SMarkHarness SMarkRunner SMarkReporter SMarkLoops\>\>benchIntLoop 1 1 5
	 ./vm my.image SMarkHarness SMarkRunner SMarkReporter SMarkLoops.benchIntLoop 1 1 5"

	| runner reporter runnerAndReporter |
	
	(self shouldShowUsage: arguments)
		ifTrue: [
			self usage.
			^ self.
		].
	
	runnerAndReporter := self parseArguments: arguments.
	runner := runnerAndReporter first.
	reporter := runnerAndReporter second.  
	
	self execute: runner andReport: reporter. ! !

!SMarkHarness class methodsFor: 'script entry' stamp: 'tfel 5/11/2015 19:57'!
run: aSuiteClass specificBenchmark: aBenchmark iterations: i processes: p problemSize: s runner: runnerClass reporter: reporterClass

	| reporter runner suite |
	suite := aSuiteClass new.
	runner := (runnerClass ifNil: [self defaultRunner]) new
		selector: aBenchmark;
		suite: suite;
		iterations: i;
		processes: p;
		problemSize: s;
		yourself.
	reporter := (reporterClass ifNil: [self defaultReporter]) new.
	self execute: runner andReport: reporter.
	^ reporter
! !

!SMarkHarness class methodsFor: 'helper' stamp: ''!
shouldShowUsage: arguments
	
	arguments size < 2 ifTrue: [^ true ].
	
	^ arguments anySatisfy: [:elem | (elem = '--help') or: [elem = '-?'] ].  ! !

!SMarkHarness class methodsFor: 'helper' stamp: 'StephaneDucasse 6/16/2013 21:04'!
usage
	| usage |
	"Example usage: SMarkHarness SMarkRunner SMarkReporter SMarkLoops.benchIntLoop 1 1 5"
	
	usage := self usageHeader.
	
	usage := usage, 'Arguments:', String crlf.
	usage := self usageRunner:   usage.
	usage := self usageReporter: usage.
	usage := usage, ' suiteOrBenchmark   required, either a SMarkSuite with benchmarks,', String crlf.
	usage := usage, '                              or a benchmark denoted by Suite.benchName', String crlf.
	usage := self usageBenchmarkParameters: usage.
	
	self defaultOutputDestination print: usage.! !

!SMarkHarness class methodsFor: 'helper' stamp: ''!
usageBenchmarkParameters: usage
	^ usage,	' iterations         optional, number of times the benchmarks are repeated', String crlf,
				' processes          optional, number of processes/threads used by the benchmarks', String crlf,
				' problemSize        optional, depending on benchmark for instance number of', String crlf,
				'                              inner iterations or size of used data set', String crlf.
! !

!SMarkHarness class methodsFor: 'helper' stamp: ''!
usageHeader
	| usage |
	usage := 'SMark Benchmark Framework, version: ', self version, String crlf.
	usage := usage, String crlf.
	usage := usage, 'Usage: <vm+image> ', self name, ' [runner] [reporter] <suiteOrBenchmark>', String crlf.
	usage := usage, '                               [iterations [processes [problemSize]]]', String crlf.
	usage := usage, String crlf.
	^ usage! !

!SMarkHarness class methodsFor: 'helper' stamp: ''!
usageReporter: usage
	^ usage,	' reporter           optional, a SMarkReporter class that processes', String crlf,
				'                              and displays the results', String crlf.
	! !

!SMarkHarness class methodsFor: 'helper' stamp: ''!
usageRunner: usage
	^ usage, ' runner             optional, a SMarkRunner class that executes the benchmarks', String crlf.! !

!SMarkHarness class methodsFor: 'helper' stamp: ''!
version
	(Smalltalk classNamed: #ConfigurationOfBenchmarking)
		ifNotNilDo: [:cfg |
			^ cfg project currentVersion versionNumber asString.
		].
	  
	(Smalltalk classNamed: #MCPackage)
		ifNotNilDo: [:mcp |
			| package |
			package := mcp named: 'SMark'.
			package hasWorkingCopy ifTrue: [
				^ package workingCopy ancestors first name.
			].
		].
	
	^ ''.! !

Object subclass: #SMarkHarnessArgumentParser
	instanceVariableNames: 'runner reporter suiteOrBenchmark iterations processes problemSize i current numParams currentObj arguments suite specificBenchmark suiteClass harness'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

SMarkHarnessArgumentParser subclass: #ReBenchHarnessArgumentParser
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

!ReBenchHarnessArgumentParser methodsFor: 'argument parsing' stamp: 'sm 5/25/2011 17:17'!
determineBenchmarkParametersFromArguments
	i := i + 1.
	i <= numParams ifTrue: [
		processes := (arguments at: i) asInteger.
		i := i + 1.
		i <= numParams ifTrue: [
			iterations := (arguments at: i) asInteger.
			i := i + 1.
			i <= numParams ifTrue: [
				problemSize := arguments at: i.
			]
		]
	].! !

!ReBenchHarnessArgumentParser methodsFor: 'argument parsing' stamp: ''!
determineReporter
	reporter := harness defaultReporter new.! !

!ReBenchHarnessArgumentParser methodsFor: 'argument parsing' stamp: ''!
determineRunner
	runner := harness defaultRunner new.! !

!ReBenchHarnessArgumentParser methodsFor: 'helper' stamp: 'sm 5/25/2011 17:31'!
instructRunner
	super instructRunner.
	
	runner iterations: runner class defaultNumberOfIterations.
	runner innerIterations: iterations.! !

!SMarkHarnessArgumentParser methodsFor: 'argument parsing' stamp: 'sm 5/25/2011 17:38'!
determineBenchmarkParameters
	"Initialize with defaults, will be overwritten in case
	 it is specified."
	iterations := runner class defaultNumberOfIterations.
	processes  := runner class defaultNumberOfProcesses.
	problemSize:= suiteClass defaultProblemSize.
	
	self determineBenchmarkParametersFromArguments.! !

!SMarkHarnessArgumentParser methodsFor: 'argument parsing' stamp: ''!
determineBenchmarkParametersFromArguments
	i := i + 1.
	i <= numParams ifTrue: [
		iterations := (arguments at: i) asInteger.
		i := i + 1.
		i <= numParams ifTrue: [
			processes := (arguments at: i) asInteger.
			i := i + 1.
			i <= numParams ifTrue: [
				problemSize := arguments at: i.
			]
		]
	].! !

!SMarkHarnessArgumentParser methodsFor: 'argument parsing' stamp: ''!
determineReporter
	(currentObj isKindOf: SMarkReporter)
		ifFalse: [ reporter := harness defaultReporter new. ]
		ifTrue:  [ reporter := currentObj.
			i := i + 1.
			i <= numParams ifTrue: [
				current := arguments at: i.
			]
		].! !

!SMarkHarnessArgumentParser methodsFor: 'argument parsing' stamp: ''!
determineRunner
	(currentObj isKindOf: SMarkRunner)
		ifFalse: [ runner := harness defaultRunner new. ]
		ifTrue:  [ runner := currentObj.
			i := i + 1.
			i <= numParams ifTrue: [
				current := arguments at: i.
				currentObj := (Smalltalk classNamed: current) ifNotNilDo: [:cls | cls new].
			]
		].! !

!SMarkHarnessArgumentParser methodsFor: 'argument parsing' stamp: 'sm 5/25/2011 17:26'!
determineSuiteOrBenchmark
	self parseBenchmarkOrSuite: current.! !

!SMarkHarnessArgumentParser methodsFor: 'accessing' stamp: ''!
harness: aHarness
	harness := aHarness! !

!SMarkHarnessArgumentParser methodsFor: 'helper' stamp: 'sm 5/25/2011 17:29'!
instructRunner
	suite := suiteClass new.
	specificBenchmark ifNotNil: [
		suite runOnly: specificBenchmark.
	].
	
	runner suite: suite.
 	runner iterations: iterations.
	runner processes: processes.
	runner problemSize: problemSize.! !

!SMarkHarnessArgumentParser methodsFor: 'parsing' stamp: 'sm 5/25/2011 17:27'!
parse: argumentsArray
	arguments := argumentsArray.
	numParams := arguments size.
	
	i := 2.
	current := arguments at: i.
	currentObj := (Smalltalk classNamed: current) ifNotNilDo: [:cls | cls new].
		
	self determineRunner.
	self determineReporter.
	
	self determineSuiteOrBenchmark.
	
	self determineBenchmarkParameters.
	
	self instructRunner.
	
	^ {runner. reporter}! !

!SMarkHarnessArgumentParser methodsFor: 'argument parsing' stamp: ''!
parseBenchmarkOrSuite: aBenchmarkOrSuite
	"Identify the benchmark suite or suite and benchmark method
	 that should be executed. The string should be of the format 'Class>>benchName' or 'Class.benchName' for shell/bash compatibility.
	 Accepts a string, class, or array.
	 Returns, a class, or an array of a class and a symbol."
	| parsed |
	(aBenchmarkOrSuite isKindOf: Class)
		ifTrue: [
			suiteClass := aBenchmarkOrSuite.
			^ suiteClass
		].
	  
	(aBenchmarkOrSuite isKindOf: Array)
		ifTrue:  [ parsed := aBenchmarkOrSuite. ]
		ifFalse: [ parsed := aBenchmarkOrSuite findTokens: '>.'. ].
	
	((parsed size > 2) or: [parsed size < 1])
				ifTrue: [ Error signal: 'The passed argument has to represent two elements. A class/classname and a method symbol' ].
	
	suiteClass := parsed first.
	
	(suiteClass isKindOf: Class) 
		ifFalse: [ suiteClass := Smalltalk at: (suiteClass asSymbol) ifAbsent: [Error signal: 'Class that was supposed to represent a benchmark suite was not found: ', suiteClass asString ]].
	
	parsed size = 1
		ifTrue: [^suiteClass].

	specificBenchmark := parsed second asSymbol.
	
	^ { suiteClass. specificBenchmark }
! !

Object subclass: #SMarkReporter
	instanceVariableNames: 'runner stream'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

!SMarkReporter commentStamp: '<historical>' prior: 0!
SMarkReporter is a simple formatter of benchmark results. 

Subclass such as SMarkSimpleStatisticsReporter might implement more advanced reporting functionality, e.g., including a statistical evaluation of the results.

Example:

	| f |
	f := TextStream on: String new.
	SMarkSimpleStatisticsReporter reportFor: (SMarkTestRunnerSuiteForAutosizing run: 10) on: f.
	f contents!

!SMarkReporter class methodsFor: 'defaults' stamp: 'tfel 5/12/2015 10:06'!
defaultOutputDestination
	^ SMarkConsole! !

!SMarkReporter class methodsFor: 'defaults' stamp: ''!
defaultReporter
	^ SMarkSimpleStatisticsReporter! !

!SMarkReporter class methodsFor: 'reporting' stamp: 'DamienPollet 1/15/2015 18:22'!
reportFor: aRunner
	^ self
		reportFor: aRunner
		on: self defaultOutputDestination.! !

!SMarkReporter class methodsFor: 'reporting' stamp: 'DamienPollet 1/15/2015 18:52'!
reportFor: aRunner on: aStream
	| reporter |
	reporter := self new
		runner: aRunner;
		outputStream: aStream;
		report; yourself.
	^ reporter.! !

!SMarkReporter methodsFor: 'reporting' stamp: ''!
benchmarkFooter: aName
	stream cr.! !

!SMarkReporter methodsFor: 'reporting' stamp: ''!
benchmarkHeader: aName
	stream << 'Benchmark ' << (aName asString); cr.! !

!SMarkReporter methodsFor: 'reporting' stamp: ''!
footer
	"No output at the moment"
	^ self! !

!SMarkReporter methodsFor: 'reporting' stamp: ''!
header
	| suiteName |
	suiteName := runner suite class name asString.
	stream << 'Report for: ' << suiteName; cr.! !

!SMarkReporter methodsFor: 'initialization' stamp: 'StephaneDucasse 6/16/2013 21:10'!
initialize
	super initialize.
	stream := self class defaultOutputDestination.! !

!SMarkReporter methodsFor: 'accessing' stamp: ''!
outputStream: aStream
	stream := aStream  ! !

!SMarkReporter methodsFor: 'accessing' stamp: ''!
report
	self header.
	
	runner results keysAndValuesDo: [:key :value |
		self benchmarkHeader: key.
		self reportAllRuns: value of: key.
		self benchmarkFooter: key.
	].

	self footer.
	^ self! !

!SMarkReporter methodsFor: 'reporting' stamp: 'CamilloBruni 9/5/2011 13:13'!
reportAllRuns: aListOfResults of: benchmark
	aListOfResults do: [:result |
		result criteria keysAndValuesDo: [:benchName :timer |
			stream << benchName << ': ' << (timer totalTime asString, 'ms'); cr.]]! !

!SMarkReporter methodsFor: 'accessing' stamp: 'tfel 5/11/2015 18:52'!
reportOn: aStream
	| oldStream |
	oldStream := stream.
	[stream := aStream.
	self header.
	runner results keysAndValuesDo: [:key :value |
		self benchmarkHeader: key.
		self reportAllRuns: value of: key.
		self benchmarkFooter: key].
	self footer] ensure: [stream := oldStream].! !

!SMarkReporter methodsFor: 'accessing' stamp: 'tfel 5/11/2015 19:35'!
runner

	^ runner! !

!SMarkReporter methodsFor: 'accessing' stamp: ''!
runner: aRunner
	runner := aRunner.! !

!SMarkReporter methodsFor: 'accessing' stamp: 'tfel 5/11/2015 19:36'!
suite

	^ runner suite! !

SMarkReporter subclass: #SMarkSimpleStatisticsReporter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

SMarkSimpleStatisticsReporter subclass: #ReBenchReporter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

!ReBenchReporter commentStamp: '<historical>' prior: 0!
A ReBenchReporter is reporter for the ReBench framework (cf. https://github.com/smarr/ReBench).
It is used be the ReBenchHarness, which is the main class of interest for users that just want to execute benchmarks.!

!ReBenchReporter methodsFor: 'as yet unclassified' stamp: 'StefanMarr 5/16/2011 15:23'!
benchmarkHeader: aName
	^ self! !

!SMarkSimpleStatisticsReporter methodsFor: 'statistics' stamp: 'tfel 5/11/2015 20:20'!
confidenceVariance: times
	| numMeasurements |
	numMeasurements := times size.
	(numMeasurements >= 30) 
		ifTrue: [
			^ (self gaussianConfidenceFactor) * (self stdev: times) / (numMeasurements asFloat sqrt)].
		
	"use the students T distribution for small probe counts"
	^ (self studentsTConfidenceFactorFor: numMeasurements) * (self stdev: times) / (numMeasurements asFloat sqrt)! !

!SMarkSimpleStatisticsReporter methodsFor: 'statistics' stamp: 'StefanMarr 3/20/2011 11:49'!
gaussianConfidenceFactor
	"used for large probe counts >= 30"
	"1 ~ 68.27%"
	"1.644853626951 ~ 90%"
	"2 ~ 95.45%"
	^ 1.644853626951! !

!SMarkSimpleStatisticsReporter methodsFor: 'reporting' stamp: 'StefanMarr 5/16/2011 16:08'!
reportAllRuns: aListOfResults of: benchmark
	| criteria |

	criteria := aListOfResults first criteria.
	
	criteria keysDo: [:criterion |
		| times |
		times := self resultsFor: criterion from: aListOfResults.
		self reportResult: times for: criterion of: benchmark.
		stream cr.
	].! !

!SMarkSimpleStatisticsReporter methodsFor: 'reporting' stamp: 'CamilloBruni 9/6/2011 10:41'!
reportResult: aResultsArray for: aCriterion of: benchmark
	| convidenceVariance significantDigits |

	stream << benchmark <<  ' ' <<  aCriterion <<  ': iterations='.
	aResultsArray size printOn: stream .
	stream << ' runtime: '.
	
	aResultsArray size < 2 ifTrue: [
		aResultsArray average printOn: stream.
		stream << 'ms'.
		^ self.
	].
	
	convidenceVariance := self confidenceVariance: aResultsArray.  
	
	"only print significant "
	significantDigits := self significantDigits: convidenceVariance.

	aResultsArray average printOn: stream showingDecimalPlaces: significantDigits.
	stream << 'ms +/-'.
	convidenceVariance printOn: stream showingDecimalPlaces: significantDigits.! !

!SMarkSimpleStatisticsReporter methodsFor: 'helper' stamp: 'StefanMarr 5/16/2011 16:02'!
resultsFor: aCriterion from: aListOfResults
	^aListOfResults collect: [:result | (result criteria at: aCriterion) totalTime]
	! !

!SMarkSimpleStatisticsReporter methodsFor: 'statistics' stamp: 'StefanMarr 3/20/2011 11:46'!
significantDigits: confidenceVariance
	confidenceVariance = 0 
		ifTrue: [ ^ 2].
	
	confidenceVariance >= 10
		ifTrue: [ ^ 0].
	
	^ 1 - (confidenceVariance log floor)! !

!SMarkSimpleStatisticsReporter methodsFor: 'statistics' stamp: 'tfel 5/11/2015 20:25'!
stdev: aCollection
	| m |
	m := aCollection median.
	^ ((aCollection inject: 0 into: [:acc :each | acc + (each - m) squared]) / aCollection size) sqrt! !

!SMarkSimpleStatisticsReporter methodsFor: 'statistics' stamp: 'StefanMarr 3/20/2011 12:03'!
studentsTConfidenceFactorFor: aNumberOfMeasurements
	"used for small probe counts < 30"
	"the students T distribution sucks to calculate since the value depends on the probeCout"
	"these values are for a confidence interval of ~90%"
	| values |
	values := Array new: 30.
	values at: 1  put: 6.314.
	values at: 2  put: 2.920.
	values at: 3  put: 2.353.
	values at: 4  put: 2.132.
	values at: 5  put: 2.015.
	values at: 6  put: 1.943.
	values at: 7  put: 1.895.
	values at: 8  put: 1.860.
	values at: 9  put: 1.833.
	values at: 10 put: 1.812.
	values at: 11 put: 1.796.
	values at: 12 put: 1.782.
	values at: 13 put: 1.771.
	values at: 14 put: 1.761.
	values at: 15 put: 1.753.
	values at: 16 put: 1.746.
	values at: 17 put: 1.740.
	values at: 18 put: 1.734.
	values at: 19 put: 1.729.
	values at: 20 put: 1.725.
	values at: 21 put: 1.721.
	values at: 22 put: 1.717.
	values at: 23 put: 1.714.
	values at: 24 put: 1.711.
	values at: 25 put: 1.708.
	values at: 26 put: 1.706.
	values at: 27 put: 1.703.
	values at: 28 put: 1.701.
	values at: 29 put: 1.699.
	values at: 30 put: 1.697.
	^ values at: aNumberOfMeasurements.
	! !

!SMarkSimpleStatisticsReporter methodsFor: 'helper' stamp: 'DamienPollet 1/15/2015 18:30'!
totalResultsFor: aListOfResults
	^ aListOfResults collect: [:result | result total]
	! !

Object subclass: #SMarkResult
	instanceVariableNames: 'time benchName suite criteria'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

!SMarkResult commentStamp: 'StefanMarr 3/18/2011 23:45' prior: 0!
A benchmark result is characterized by:
 - the total execution time (#total is the least a benchmark results in)
 - the name of the benchmark that was executed
 - the suite object specifies the used input used for the benchmark
 - dictionary of additional the criteria and the related timings

A benchmark can produced multiple resuts for different criteria. The standard criterion is #total.!

!SMarkResult methodsFor: 'accessing' stamp: ''!
benchmarkName
	^ benchName! !

!SMarkResult methodsFor: 'accessing' stamp: ''!
benchmarkName: aString
	benchName := aString! !

!SMarkResult methodsFor: 'accessing' stamp: ''!
criteria
	^ criteria! !

!SMarkResult methodsFor: 'accessing' stamp: ''!
criteria: aCollectionOfTimers
	criteria := aCollectionOfTimers! !

!SMarkResult methodsFor: 'accessing' stamp: ''!
suite
	^ suite! !

!SMarkResult methodsFor: 'accessing' stamp: ''!
suite: aBenchmarkSuite
	suite := aBenchmarkSuite! !

!SMarkResult methodsFor: 'accessing' stamp: ''!
total
	^ time! !

!SMarkResult methodsFor: 'accessing' stamp: ''!
total: aTime
	time := aTime! !

Object subclass: #SMarkRunner
	instanceVariableNames: 'numIterations suite selector runner results currentBenchmark timers problemSize numProcesses'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

SMarkRunner subclass: #SMarkAutosizeRunner
	instanceVariableNames: 'targetTime innerLoopIterations'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

!SMarkAutosizeRunner class methodsFor: 'defaults' stamp: 'tfel 5/11/2015 19:32'!
defaultTargetTime
	"600 milliseconds seems to be a reasonable target time for most problems.
	 It is a compromise between the general measurment noise as well as timer accuracy
	 and the absolute runtime of benchmarks"
	^ 600! !

!SMarkAutosizeRunner methodsFor: 'initialization' stamp: 'StefanMarr 3/19/2011 19:59'!
initialize
	super initialize.
	targetTime := self class defaultTargetTime.! !

!SMarkAutosizeRunner methodsFor: 'benchmarking' stamp: 'StefanMarr 3/20/2011 10:27'!
performBenchmark: aSelector
	"First determine a useful number of inner loop iterations until the targetTime is reached."
	| execTime i |
	"make sure no timers are recorded for this"
	timers := nil.
	
	i := 1.
	execTime := Time millisecondsToRun: [ suite perform: aSelector. ].
	
	[ execTime > targetTime ] whileFalse: [
		i := i * 2. "Was thinking of doing something fancy here, but just go with this simple staight-forward solution"
		execTime := Time millisecondsToRun: [ 1 to: i do: [:ignored| suite perform: aSelector]].
	].

	innerLoopIterations := i.
	
	"Then start executing the benchmark"
	^ super performBenchmark: aSelector.! !

!SMarkAutosizeRunner methodsFor: 'benchmarking' stamp: 'DamienPollet 1/15/2015 11:46'!
runBaseBenchmark
	"baseBenchmark is not supported with autosizing. I do not see how that can be made possible since all different benchmarks will have different number of iterations, and the only way how a consistent baseline could be found would be to normalize the results, but well, incorporating the baseline measurement with the statistical evaluation is harder than just substracting a one time value..., I am not going to do that here for the moment. Stefan 2011-03-20"
	
	(suite respondsTo: #baseBenchmark) 
		ifFalse: [ ^ nil ].
	
	"I decided to go here with a silent solution to avoid thinking about logging frameworks and Transcript to console convertion..."
	self recordResults: (self class defaultTimer new: 'total') for: #baseBenchmark  ! !

!SMarkAutosizeRunner methodsFor: 'accessing' stamp: 'StefanMarr 3/19/2011 19:18'!
targetTime
	"Target time in milliseconds"
	^ targetTime! !

!SMarkAutosizeRunner methodsFor: 'accessing' stamp: 'StefanMarr 3/19/2011 19:18'!
targetTime: anIntInMilliseconds
	"Target time in milliseconds"
	targetTime := anIntInMilliseconds! !

!SMarkAutosizeRunner methodsFor: 'benchmarking' stamp: 'StefanMarr 3/20/2011 10:31'!
timedBenchmarkExecution: aSelector
	"Will do a timed execution of the benchmark and return the result timer"
	| timer |
	timers := Dictionary new.

	timer := self createTimer: 'total'.
	
	timer start.
	1 to: innerLoopIterations do: [:ignored|
		suite perform: aSelector.
	].
	timer stop.
	
	self recordResults: timer for: aSelector.
	
	^ timer! !

SMarkAutosizeRunner subclass: #SMarkTestAutosizeRunner
	instanceVariableNames: 'performedBenchmarks notifyRunnerWasExecuted setUpCount tearDownCount didTearDownBenchC didSetUpBenchB baseBenchmarkExecuted'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkTestAutosizeRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 11:01'!
baseBenchmarkExecuted
	^ baseBenchmarkExecuted! !

!SMarkTestAutosizeRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 11:01'!
countSetUpInvoke
	setUpCount := setUpCount + 1! !

!SMarkTestAutosizeRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 11:01'!
countTearDownInvoke
	tearDownCount := tearDownCount + 1! !

!SMarkTestAutosizeRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 11:01'!
countWasExecuted
	^ notifyRunnerWasExecuted ! !

!SMarkTestAutosizeRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 11:01'!
didPerform: aSymbol
	performedBenchmarks add: aSymbol.! !

!SMarkTestAutosizeRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 11:01'!
didRunBaseBenchmark
	baseBenchmarkExecuted := true.! !

!SMarkTestAutosizeRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 11:01'!
didSetUpBenchB
	didSetUpBenchB := true.! !

!SMarkTestAutosizeRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 11:01'!
didTearDownBenchC
	didTearDownBenchC := true.! !

!SMarkTestAutosizeRunner methodsFor: 'initialization' stamp: 'StefanMarr 3/19/2011 23:24'!
initialize
	super initialize.

	setUpCount := 0.
	tearDownCount := 0.
	notifyRunnerWasExecuted := 0.
	
	baseBenchmarkExecuted := false.

	performedBenchmarks := Set new.
	
	"set the targetTime to something which is more reasonable for testing"
	targetTime := 50.! !

!SMarkTestAutosizeRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 11:01'!
isNotifyRunnerWasExecutedSet
	^ notifyRunnerWasExecuted! !

!SMarkTestAutosizeRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 11:01'!
notifyRunnerWasExecuted
	notifyRunnerWasExecuted := notifyRunnerWasExecuted + 1! !

!SMarkTestAutosizeRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 11:01'!
performedBenchmarks
	^ performedBenchmarks! !

!SMarkTestAutosizeRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 11:01'!
setUp
	notifyRunnerWasExecuted := false! !

SMarkRunner subclass: #SMarkCogRunner
	instanceVariableNames: 'warmingUp'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

!SMarkCogRunner commentStamp: 'StefanMarr 12/30/2011 23:24' prior: 0!
This runner is doing warmup on for Cog VMs with just-in-time compilation.
The goal is to bring the JIT compiler into a steady state where no jitting is performed anymore during benchmarking.!

!SMarkCogRunner methodsFor: 'initialization' stamp: 'StefanMarr 12/30/2011 22:51'!
initialize
	super initialize.
	warmingUp := false.! !

!SMarkCogRunner methodsFor: 'initialization' stamp: 'StefanMarr 12/30/2011 23:26'!
performBenchmark: aSelector
	"Based on an email by Eliot from May 16th, 2011.
	 The first time a method is executed it will get into the inline cache.
	 The second time, it is found in the inline cache, which triggers the JIT compiler to produce code.
	 Thus, the third time it should be executed in the steady state."
	warmingUp := true.
		suite runBenchmark: aSelector.
		Smalltalk garbageCollect.
		suite runBenchmark: aSelector.
		Smalltalk garbageCollect.
	warmingUp := false.
	
	^ super performBenchmark: aSelector.! !

!SMarkCogRunner methodsFor: 'initialization' stamp: 'StefanMarr 12/30/2011 22:57'!
recordResults: timer for: aSelector
	"Only record the results when we are not in warmup mode."
	warmingUp ifFalse: [
		super recordResults: timer for: aSelector.
	].! !

SMarkRunner subclass: #SMarkProfileRunner
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

!SMarkProfileRunner commentStamp: '<historical>' prior: 0!
This runner profiles the benchmarks for better analysis. Unlike the classical benchmark runner this one will not collect the results. Instead it will execute the benchmarks in the system profiler.!

!SMarkProfileRunner class methodsFor: 'profiling' stamp: 'CamilloBruni 1/28/2013 18:19'!
execute: aSuite selector: aBenchmarkSelector
	| runner |
	runner := self new.
	aSuite runner: runner.
	runner 
		suite: aSuite;
		execute: aBenchmarkSelector.
	^ runner! !

!SMarkProfileRunner class methodsFor: 'profiling' stamp: 'CamilloBruni 1/28/2013 18:31'!
execute: aSuite selector: aBenchmarkSelector iterations: nIterations
	| runner |
	runner := self new.
	aSuite runner: runner.
	runner 
		suite: aSuite;
		iterations: nIterations;
		execute: aBenchmarkSelector.
	^ runner! !

!SMarkProfileRunner methodsFor: 'profiling' stamp: 'CamilloBruni 1/28/2013 18:24'!
execute
	"run all benchmnarks in a benchmark suite "
	[ suite run ] timeProfile! !

!SMarkProfileRunner methodsFor: 'profiling' stamp: 'CamilloBruni 1/28/2013 18:28'!
execute: aSelector
	
	[ self performBenchmark: aSelector ] timeProfile! !

!SMarkProfileRunner methodsFor: 'initialization' stamp: 'CamilloBruni 1/28/2013 18:28'!
initialize
	super initialize.
	numIterations := 1.! !

!SMarkProfileRunner methodsFor: 'benchmarks' stamp: 'CamilloBruni 1/28/2013 18:34'!
performBenchmark: aSelector
	currentBenchmark := aSelector.
	
	1 to: numIterations do: [:i|
		suite runBenchmark: aSelector ].	
	
	currentBenchmark := nil.! !

!SMarkProfileRunner methodsFor: 'benchmarks' stamp: 'CamilloBruni 1/28/2013 18:34'!
timedBenchmarkExecution: aSelector
	suite perform: aSelector! !

!SMarkRunner class methodsFor: 'defaults' stamp: 'StefanMarr 3/21/2011 11:43'!
defaultNumberOfIterations
	^ 1! !

!SMarkRunner class methodsFor: 'defaults' stamp: 'StefanMarr 5/14/2011 11:09'!
defaultNumberOfProcesses
	^ 1! !

!SMarkRunner class methodsFor: 'defaults' stamp: 'StefanMarr 3/20/2011 20:56'!
defaultTimer
	^ SMarkTimer! !

!SMarkRunner class methodsFor: 'benchmarking' stamp: 'StefanMarr 3/20/2011 19:51'!
execute: aSuite
	^ self execute: aSuite with: 1.! !

!SMarkRunner class methodsFor: 'benchmarking' stamp: 'StefanMarr 3/20/2011 19:51'!
execute: aSuite with: nIterations
	| runner |
	runner := self new.
	aSuite runner: runner.
	runner suite: aSuite.
	runner iterations: nIterations.  
	runner execute.
	^ runner! !

!SMarkRunner methodsFor: 'helper' stamp: 'StefanMarr 3/20/2011 10:28'!
createTimer: name
	"Create and register a new timer for the current benchmark"
	| timer |
	timer := self class defaultTimer new: name.
	
	timers ifNotNil: [
		timers at: name put: timer.
	].
	
	^ timer.! !

!SMarkRunner methodsFor: 'execution' stamp: 'tfel 5/11/2015 19:58'!
execute
	selector ifNotNil: [suite runOnly: selector].
	suite run.
	self runBaseBenchmark.
	^ results
	! !

!SMarkRunner methodsFor: 'initialization' stamp: 'StefanMarr 5/14/2011 11:10'!
initialize
	super initialize.
	numIterations := self class defaultNumberOfIterations.
	numProcesses  := self class defaultNumberOfProcesses.
	results := Dictionary new.! !

!SMarkRunner methodsFor: 'accessing' stamp: 'StefanMarr 3/21/2011 12:20'!
iterations
	^ numIterations! !

!SMarkRunner methodsFor: 'accessing' stamp: 'StefanMarr 3/21/2011 12:20'!
iterations: anInteger
	numIterations := anInteger! !

!SMarkRunner methodsFor: 'benchmarking' stamp: 'StefanMarr 4/15/2011 11:50'!
performBenchmark: aSelector
	currentBenchmark := aSelector.
	
	1 to: numIterations do: [:i|
		"self timedBenchmarkExecution: aSelector."
		suite runBenchmark: aSelector.  
	].	
	
	currentBenchmark := nil.
	
	^ results at: (suite benchmarkNameForSelector: aSelector)! !

!SMarkRunner methodsFor: 'printing' stamp: 'CamilloBruni 9/5/2011 13:13'!
printOn: aStream
	^ self reportOn: aStream.! !

!SMarkRunner methodsFor: 'accessing' stamp: 'StefanMarr 5/8/2013 13:55'!
problemSize
	<omniUnenforced> "Hint for the OMOP that it is part of the meta infrastructure"
	^ problemSize! !

!SMarkRunner methodsFor: 'accessing' stamp: 'StefanMarr 8/21/2012 09:44'!
problemSize: aValue
	"Do some conversion to make it easier for the benchmarks"
	(aValue isString and: [aValue isAllDigits]) ifTrue: [
		problemSize := Number readFrom: aValue.
		^ self.
	].

	problemSize := aValue! !

!SMarkRunner methodsFor: 'accessing' stamp: 'StefanMarr 5/14/2011 11:08'!
processes
	"The standard runner does use only a single process, but in case a benchmark supports parallelism it can query for the intended degree of parallelism"
	^ numProcesses! !

!SMarkRunner methodsFor: 'accessing' stamp: 'StefanMarr 5/14/2011 11:08'!
processes: anInt
	"The standard runner does use only a single process, but a benchmark can use that to do its own parallelism"
	numProcesses := anInt! !

!SMarkRunner methodsFor: 'helper' stamp: 'DamienPollet 1/15/2015 18:44'!
recordResults: timer for: aSelector
	| result name |
	name := suite benchmarkNameForSelector: aSelector.
	
	result := SMarkResult new
		total: timer totalTime;
		benchmarkName: name;
		suite: suite;
		criteria: timers;
		yourself.
	
	(results at: name ifAbsentPut: OrderedCollection new) add: result.! !

!SMarkRunner methodsFor: 'accessing' stamp: 'StefanMarr 4/15/2011 16:46'!
report
	SMarkReporter defaultReporter reportFor: self.  
	! !

!SMarkRunner methodsFor: 'reporting' stamp: 'sm 5/25/2011 17:50'!
reportConfiguration: aStream
	aStream << 'Runner Configuration:';cr.
	aStream << ('  iterations: ', numIterations asString); cr.
	aStream << ('  processes: ', numProcesses asString); cr.
	aStream << ('  problem size: ', problemSize asString); cr.
! !

!SMarkRunner methodsFor: 'reporting' stamp: 'StefanMarr 4/15/2011 16:47'!
reportOn: aStream
	SMarkReporter defaultReporter reportFor: self on: aStream  
	! !

!SMarkRunner methodsFor: 'accessing' stamp: 'StefanMarr 3/18/2011 19:41'!
results
	^ results! !

!SMarkRunner methodsFor: 'benchmarking' stamp: 'StefanMarr 3/18/2011 18:42'!
runBaseBenchmark
	"In certain sitatuations it is one wants a baseline that is incooprated in all 
	 benchmark results to be substracted from the final values.
	
	#baseBenchmark can be used to charaterize such a baseline"
	
	(suite respondsTo: #baseBenchmark) 
		ifFalse: [ ^ nil ].
	
	^ self performBenchmark: #baseBenchmark.! !

!SMarkRunner methodsFor: 'accessing' stamp: 'tfel 5/11/2015 19:57'!
selector

	^ selector! !

!SMarkRunner methodsFor: 'accessing' stamp: 'tfel 5/11/2015 19:57'!
selector: aSymbol

	selector := aSymbol.! !

!SMarkRunner methodsFor: 'accessing' stamp: 'StefanMarr 3/20/2011 18:45'!
suite
	^ suite! !

!SMarkRunner methodsFor: 'accessing' stamp: 'StefanMarr 4/15/2011 16:14'!
suite: aBenchmarkSuite
	suite := aBenchmarkSuite.
	suite runner: self.! !

!SMarkRunner methodsFor: 'benchmarking' stamp: 'ReinoutStevens 12/6/2011 14:30'!
timedBenchmarkExecution: aSelector
	"Will do a timed execution of the benchmark and return the result timer"
	| timer result |
	timers := Dictionary new.

	timer := self createTimer: 'total'.
	
	timer start.
	result := suite perform: aSelector.
	timer stop.
	suite processResult: result withTimer: timer.
	
	self recordResults: timer for: aSelector.
	
	^ timer! !

SMarkRunner subclass: #SMarkTestRunner
	instanceVariableNames: 'performedBenchmarks notifyRunnerWasExecuted setUpCount tearDownCount didTearDownBenchC didSetUpBenchB baseBenchmarkExecuted'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/18/2011 18:35'!
baseBenchmarkExecuted
	^ baseBenchmarkExecuted! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/18/2011 17:36'!
countSetUpInvoke
	setUpCount := setUpCount + 1! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/18/2011 17:39'!
countTearDownInvoke
	tearDownCount := tearDownCount + 1! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/18/2011 19:44'!
countWasExecuted
	^ notifyRunnerWasExecuted ! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/18/2011 19:00'!
didPerform: aSymbol
	performedBenchmarks add: aSymbol.! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/18/2011 17:56'!
didRunBaseBenchmark
	baseBenchmarkExecuted := true.! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/18/2011 17:43'!
didSetUpBenchB
	didSetUpBenchB := true.! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/18/2011 17:42'!
didTearDownBenchC
	didTearDownBenchC := true.! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 12:14'!
hasSetUpBenchB
	^ didSetUpBenchB.! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 12:21'!
hasTearDownBenchC
	^ didTearDownBenchC! !

!SMarkTestRunner methodsFor: 'initialization' stamp: 'StefanMarr 3/18/2011 19:49'!
initialize
	super initialize.

	setUpCount := 0.
	tearDownCount := 0.
	notifyRunnerWasExecuted := 0.
	
	baseBenchmarkExecuted := false.

	performedBenchmarks := OrderedCollection new.! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/15/2011 22:16'!
isNotifyRunnerWasExecutedSet
	^ notifyRunnerWasExecuted! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/18/2011 19:44'!
notifyRunnerWasExecuted
	notifyRunnerWasExecuted := notifyRunnerWasExecuted + 1! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/18/2011 19:00'!
performedBenchmarks
	^ performedBenchmarks! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/15/2011 22:15'!
setUp
	notifyRunnerWasExecuted := false! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 12:27'!
setUpCount
	^ setUpCount! !

!SMarkTestRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 12:39'!
tearDownCount
	^ tearDownCount! !

SMarkRunner subclass: #SMarkWeakScalingRunner
	instanceVariableNames: 'numInnerIterations runningProcesses completionSignal runningProcessesMtx'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

!SMarkWeakScalingRunner commentStamp: 'DamienPollet 1/16/2015 17:24' prior: 0!
This runner is specific to platforms with support for real parallelism.

Weak scaling is defined as how the solution time varies with the number of processors for a fixed problem size per processor.!

SMarkWeakScalingRunner subclass: #SMarkTestWeakScalingRunner
	instanceVariableNames: 'setUpCount tearDownCount notifyRunnerWasExecuted performedBenchmarks baseBenchmarkExecuted didSetUpBenchB didTearDownBenchC'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkTestWeakScalingRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 10:44'!
baseBenchmarkExecuted
	^ baseBenchmarkExecuted! !

!SMarkTestWeakScalingRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 10:41'!
countSetUpInvoke
	setUpCount := setUpCount + 1! !

!SMarkTestWeakScalingRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 10:41'!
countTearDownInvoke
	tearDownCount := tearDownCount + 1! !

!SMarkTestWeakScalingRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 10:41'!
countWasExecuted
	^ notifyRunnerWasExecuted ! !

!SMarkTestWeakScalingRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 10:41'!
didPerform: aSymbol
	performedBenchmarks add: aSymbol.! !

!SMarkTestWeakScalingRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 10:42'!
didRunBaseBenchmark
	baseBenchmarkExecuted := true.! !

!SMarkTestWeakScalingRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 10:42'!
didSetUpBenchB
	didSetUpBenchB := true.! !

!SMarkTestWeakScalingRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 10:42'!
didTearDownBenchC
	didTearDownBenchC := true.! !

!SMarkTestWeakScalingRunner methodsFor: 'initialization' stamp: 'StefanMarr 4/15/2011 10:42'!
initialize
	super initialize.

	setUpCount := 0.
	tearDownCount := 0.
	notifyRunnerWasExecuted := 0.
	
	baseBenchmarkExecuted := false.

	performedBenchmarks := OrderedCollection new.! !

!SMarkTestWeakScalingRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 13:20'!
isNotifyRunnerWasExecutedSet
	^ notifyRunnerWasExecuted > 0! !

!SMarkTestWeakScalingRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 10:42'!
notifyRunnerWasExecuted
	notifyRunnerWasExecuted := notifyRunnerWasExecuted + 1! !

!SMarkTestWeakScalingRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 10:42'!
performedBenchmarks
	^ performedBenchmarks! !

!SMarkTestWeakScalingRunner methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/15/2011 10:43'!
setUp
	notifyRunnerWasExecuted := false! !

!SMarkWeakScalingRunner class methodsFor: 'defaults' stamp: 'StefanMarr 4/15/2011 15:42'!
defaultNumberOfInnerIterations
	"The number of iterations of the inner loop
	 in which the benchmark is executed."
	^ 1! !

!SMarkWeakScalingRunner methodsFor: 'benchmarking' stamp: 'sm 5/30/2011 17:13'!
execute: aSelector withProcesses: numberOfProcesses withTimer: timer
	"This case is meant for all cases. REM: this is also used for numProc==1 to be able to measure the process start overhead in all cases.
	 It will start the processes and wait for their completion."
	
	| processes |
	processes			:= Array new: numberOfProcesses.
	runningProcessesMtx := Semaphore forMutualExclusion.
	completionSignal		:= Semaphore new.
	runningProcesses := numberOfProcesses.
	
	"First initialize the processes"
	1 to: numberOfProcesses do: [ :procNum |
		| proc |
		proc := SMarkWeakScalingRunnerExecutor createFor: aSelector for: numInnerIterations with: self and: suite.
		proc priority: Processor highestPriority.
		proc name: (self class name, '-',  procNum asString).
		processes at: procNum put: proc.
		"On: procNum"
	].
	
	"Now, execute the benchmark and do the timing now"
	timer start.
	1 to: numberOfProcesses do: [ :procNum |
		(processes at: procNum) resume.
	].
	completionSignal wait.
	timer stop.
	! !

!SMarkWeakScalingRunner methodsFor: 'benchmarking' stamp: 'sm 5/30/2011 16:17'!
executorCompleted
	runningProcessesMtx critical: [
		runningProcesses := runningProcesses - 1.
		(runningProcesses == 0) ifTrue: [
			completionSignal signal.
		]
	]! !

!SMarkWeakScalingRunner methodsFor: 'initialization' stamp: 'StefanMarr 4/15/2011 15:43'!
initialize
	super initialize.
	numProcesses			:= self class defaultNumberOfProcesses.
	numInnerIterations	:= self class defaultNumberOfInnerIterations.
! !

!SMarkWeakScalingRunner methodsFor: 'benchmarking' stamp: 'StefanMarr 4/15/2011 15:43'!
innerIterations
	"The number of inner iterations the benchmark is executed inside a processes"
	^ numInnerIterations! !

!SMarkWeakScalingRunner methodsFor: 'benchmarking' stamp: 'StefanMarr 4/15/2011 15:43'!
innerIterations: anInteger
	"The number of inner iterations the benchmark is executed inside a processes"
	numInnerIterations := anInteger! !

!SMarkWeakScalingRunner methodsFor: 'benchmarking' stamp: 'StefanMarr 4/15/2011 10:54'!
processes
	^ numProcesses! !

!SMarkWeakScalingRunner methodsFor: 'benchmarking' stamp: 'StefanMarr 4/15/2011 10:53'!
processes: anInteger
	numProcesses := anInteger! !

!SMarkWeakScalingRunner methodsFor: 'reporting' stamp: 'sm 5/25/2011 17:50'!
reportConfiguration: aStream
	super reportConfiguration: aStream.
	aStream << ('inner iterations: ', numInnerIterations asString); cr.! !

!SMarkWeakScalingRunner methodsFor: 'benchmarking' stamp: 'sm 5/25/2011 18:31'!
timedBenchmarkExecution: aSelector
	"Will do a timed execution of the benchmark and return the result timer"
	| timer |
	timers := Dictionary new.

	timer := self createTimer: 'total'.
	
	self execute: aSelector withProcesses: numProcesses withTimer: timer.
	
	self recordResults: timer for: aSelector.
	
	^ timer! !

Object subclass: #SMarkSuite
	instanceVariableNames: 'runner selectedBenchmarks'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

SMarkSuite class
	instanceVariableNames: 'history'!

!SMarkSuite commentStamp: '<historical>' prior: 0!
A Benchmark Suite is a set of benchmarks and it knows what exactly needs to be executed.
However, it does not really know how to execute it.
It knows all the magic, that is, how to set up and tear down the environment for the benchmarks, but does not have the knowledge of how many iterations need to be done and how to evaluate any results that might be produced.

Please see also SMarkHarness, which is the main class relevant for users to execute benchmarks with SMark.

Usage:

Choose a suite (i.e. one of my subclasses) and use the class-side #run or run: messages.

To get an example print the result of the following expression:
	
	SMarkCompiler run: 10
	
	SMarkLoops runOnly: #benchArrayAccess
!

SMarkSuite class
	instanceVariableNames: 'history'!

SMarkSuite subclass: #BenchRSqueak
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-RSqueak'!

!BenchRSqueak methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 09:57'!
benchBitBlt

	#( paint over ) do: [:rule|
		| destRect depths |
		destRect := 0@0 extent: 600@600.
		depths := #( 1 2 4 8 16 32 ).
		depths do:[:destDepth| | dest |
			depths do:[:sourceDepth| | bb source |
				dest := Form extent: destRect extent depth: destDepth.
				source := Form extent: destRect extent depth: sourceDepth.
				(source getCanvas) fillOval: dest boundingBox color: Color yellow borderWidth: 30 borderColor: Color black.
				bb := (WarpBlt toForm: dest)
					sourceForm: source;
					sourceRect: source boundingBox;
					destRect: dest boundingBox;
					colorMap: (source colormapIfNeededFor: dest);
					combinationRule: (Form perform: rule).
				self problemSize timesRepeat: [bb copyBits]]]].! !

!BenchRSqueak methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 09:59'!
benchDSAGen

	| randomString |
	randomString := '4647912 April 2015''unix'''.
	self problemSize timesRepeat: [
		[DigitalSignatureAlgorithm new
			initRandomFromString: randomString;
			generateKeySet] timeToRun].! !

!BenchRSqueak methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 09:59'!
benchRenderFont
	| height ttc glyphs |
	height := (TextStyle pointsToPixels: (12 * 1.2)) rounded.
	ttc := (TextStyle named: 'BitstreamVeraSerif') fontArray last ttcDescription.
	glyphs := (((1 to: ttc size - 1) collect: [:i | ttc at: i]) reject: [:g | g = ttc fallbackGlyph or: [g contours size = 0]]) first: 52.
	self problemSize timesRepeat: [
		glyphs do: [:g| 
			g asFormWithScale: (height asFloat / ttc fontHeight)
					ascender: ttc ascender 
					descender: ttc descender 
					fgColor: Color black bgColor: Color transparent depth: 32]].! !

!BenchRSqueak methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 09:59'!
benchShaLongString

	| longString veryLongString |
	longString := (PackageInfo named: 'Help-Squeak-Project') postscript contents asString.
	veryLongString := String streamContents: [:s| 5 timesRepeat: [s nextPutAll: longString]].
	self problemSize timesRepeat: [SecureHashAlgorithm new hashMessage: veryLongString].! !

SMarkSuite subclass: #BenchSqueakJS
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-SqueakJS'!

!BenchSqueakJS methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 09:36'!
benchmarkBitBltExampleOne

	self bltExampleOneTimes: self problemSize extent: 32@32.
	self bltExampleOneTimes: self problemSize extent: 64@64.
	self bltExampleOneTimes: self problemSize extent: 256@256.! !

!BenchSqueakJS methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 09:33'!
benchmarkFib
	
	self problemSize benchFib.! !

!BenchSqueakJS methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 09:52'!
benchmarkFillArray
	| b o |
	b := Array new: self problemSize.
	o := Object new.
	self problemSize timesRepeat: [b atAllPut: o].! !

!BenchSqueakJS methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 09:51'!
benchmarkFillByteArray
	| b |
	b := ByteArray new: self problemSize.
	self problemSize timesRepeat: [b atAllPut: 11].! !

!BenchSqueakJS methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 09:51'!
benchmarkFillString
	| b |
	b := String new: self problemSize.
	self problemSize timesRepeat: [b atAllPut: $a].! !

!BenchSqueakJS methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 09:42'!
benchmarkIntegerBench

	self problemSize benchmark.! !

!BenchSqueakJS methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 09:31'!
benchmarkMandala
	
	| f npoints d l pen points |
	npoints := self problemSize.
	f := Form extent: 1000@1000 depth: 32.
	pen := Pen newOnForm: f.
	d := f height - 50.
	l := 3.14 * d / npoints.
	pen home; up; turn: -90; go: d // 2; turn: 90; go: 0 - l / 2; down.
	points := Array new: npoints.
	1 to: npoints do: 
		[:i | 
		points at: i put: pen location rounded.
		pen go: l; turn: 360.0 / npoints].
	npoints // 2
		to: 1
		by: -1
		do: 
			[:i | 
			pen color: i.
			1 to: npoints do: 
				[:j | 
				pen place: (points at: j).
				pen goto: (points at: j + i - 1 \\ npoints + 1)]]
! !

!BenchSqueakJS methodsFor: 'as yet unclassified' stamp: 'tfel 5/15/2015 09:41'!
bltExampleOneTimes: n extent: aPoint

	| s d aBitBlt |
	s := Form extent: aPoint.
	s fillWhite.
	'Starting and Quitting
Like most Smalltalk''s, the machine-executable portion is a relatively small program known as the "virtual machine" (VM).  The VM''s job is to provide services from the physical machine to real Smalltalk objects.  Services like input and output.  The Smalltalk system, including all data and code, is a system of objects, built from the ground up, and interpreted by this virtual computer.  This affords Smalltalk system platform portability.' asParagraph
		wrappingBox: (10@10 corner: 600@600)
		clippingBox: (0@0 extent: aPoint);
		displayOn: s at: 0@0.
	s fillBlack: (7@7 corner: 25@25).
	s fillBlack: (39@39 corner: 47@47).
	d := Form extent: aPoint.
	aBitBlt := BitBlt
				destForm: d
				sourceForm: s
				fillColor: nil
				combinationRule: 1
				destOrigin: 0@0
				sourceOrigin: 0@0
				extent: s extent
				clipRect: (0@0 corner: aPoint).
	1 to: n do: [:i |
		aBitBlt combinationRule: i \\ 16.
		d fillWhite.
		d fillBlack: (0@0 corner: 64@16).
		aBitBlt copyBits].
	! !

SMarkSuite subclass: #GSGraphSearch
	instanceVariableNames: 'graphNodes graphMask updatingGraphMask graphVisited cost graphEdges k firstCost'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-GraphSearch'!

!GSGraphSearch class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 15:37'!
ExpectedNoOfNodes
	^ 3000000! !

!GSGraphSearch class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 15:37'!
ExpectedTotalCost
	^ 26321966! !

!GSGraphSearch class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 15:36'!
MaxInitEdges
	^ 4! !

!GSGraphSearch class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 15:36'!
MaxNodes 
	^ 1 << 31! !

!GSGraphSearch class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 15:37'!
MaxWeight
	^ 1! !

!GSGraphSearch class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 15:36'!
MinEdges
	^ 2
! !

!GSGraphSearch class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 15:35'!
MinNodes
	^ 20! !

!GSGraphSearch class methodsFor: 'constants' stamp: 'StefanMarr 4/3/2015 15:36'!
MinWeight
	^ 1! !

!GSGraphSearch methodsFor: 'benchmarking' stamp: 'StefanMarr 4/3/2015 15:33'!
benchGraphSearch
    | noOfNodes |
    SJenkinsRandom seed: 49734321.
    noOfNodes := GSGraphSearch ExpectedNoOfNodes / 1000 * self problemSize.

    self initializeGraph: noOfNodes.
    self breadthFirstSearch: noOfNodes.
    ^ cost! !

!GSGraphSearch methodsFor: 'helper' stamp: 'StefanMarr 4/3/2015 15:34'!
breadthFirstSearch: noOfNodes
    | stop |
    stop := true.
    [stop] whileTrue: [
      stop := false.
      
      1 to: noOfNodes do: [:tid |
        (graphMask at: tid) ifTrue: [
          graphMask at: tid put: false.
          (graphNodes at: tid) starting 
              to: ((graphNodes at: tid) noOfEdges + (graphNodes at: tid) starting) - 1
              do: [:i |
            | id |
            id := graphEdges at: i.
            (graphVisited at: id) ifFalse: [
              cost at: id put: (cost at: tid) + 1.
              updatingGraphMask at: id put: true.
            ]
          ]
        ]
      ].
      
      1 to: noOfNodes do: [:tid |
        (updatingGraphMask at: tid) ifTrue: [
          graphMask    at: tid put: true.
          graphVisited at: tid put: true.
          stop := true.
          updatingGraphMask at: tid put: false
        ]
      ]
    ]! !

!GSGraphSearch methodsFor: 'initialization' stamp: 'StefanMarr 4/3/2015 15:29'!
initializeGraph: noOfNodes
    | source graph totalEdges |
    graphNodes        := Array new: noOfNodes.
    graphMask         := Array new: noOfNodes withAll: false.
    updatingGraphMask := Array new: noOfNodes withAll: false.
    graphVisited      := Array new: noOfNodes withAll: false.
    cost              := Array new: noOfNodes withAll: -1.

    source := 1.

    graph := Array new: noOfNodes.
    1 to: noOfNodes do: [:i | graph at: i put: OrderedCollection new ].

    1 to: noOfNodes do: [:i |
      | noOfEdges |
      noOfEdges := (SJenkinsRandom random
                       rem: GSGraphSearch MaxInitEdges - GSGraphSearch MinEdges + 1) abs
                   + GSGraphSearch MinEdges.
      1 to: noOfEdges do: [:j |
        | nodeId weight |
        nodeId := (SJenkinsRandom random rem: noOfNodes) abs + 1.
        weight := (SJenkinsRandom random rem: (
                    GSGraphSearch MaxWeight - GSGraphSearch MinWeight + 1)) abs
                  + GSGraphSearch MinWeight.
        (graph at: i)      append: (GSEdge newWith: nodeId and: weight).
        (graph at: nodeId) append: (GSEdge newWith:      i and: weight).
      ]
    ].

    totalEdges := 0.
    1 to: noOfNodes do: [:i |
      | noOfEdges |
      noOfEdges := (graph at: i) size.
      graphNodes at: i put: (GSNode newWith: totalEdges + 1 and: noOfEdges).
      totalEdges := totalEdges + noOfEdges
    ].

    graphMask    at: source put: true.
    graphVisited at: source put: true.

    graphEdges := Array new: totalEdges withAll: 0.
    
    k := 1.
    graph do: [:i |
      i do: [:j |
        graphEdges at: k put: j dest.
        k := k + 1.
      ]
    ].
    cost at: source put: 0.! !

!GSGraphSearch methodsFor: 'benchmarking' stamp: 'StefanMarr 4/3/2015 15:35'!
verifyGraphSearch: result
    | totalCost |
    cost length = (GSGraphSearch ExpectedNoOfNodes / 1000 * self problemSize)
      ifFalse: [ ^ false ].

    totalCost := 0.    
    cost do: [:c | totalCost := totalCost + c].

    cost length = GSGraphSearch ExpectedNoOfNodes
      ifTrue:  [
        totalCost = GSGraphSearch ExpectedTotalCost ifFalse: [
          self error: 'ERROR: the total cost obtained for ', cost length asString,
              ' nodes is ', totalCost asString, ' while the expected cost is ',
              GSGraphSearch ExpectedTotalCost 
        ]]
      ifFalse: [
        firstCost == nil
          ifTrue: [
            firstCost := totalCost.
            ^ true ]
          ifFalse: [
            ^ firstCost = totalCost
          ]
      ].

    ^ true! !

SMarkSuite subclass: #JSJson
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Json'!

!JSJson class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/2/2015 22:45'!
RapBenchmarkMinified
	^ '{"head":{"requestCounter":4},"operations":[["destroy","w54"],["set","w2",{"activeControl":"w99"}],["set","w21",{"customVariant":"variant_navigation"}],["set","w28",{"customVariant":"variant_selected"}],["set","w53",{"children":["w95"]}],["create","w95","rwt.widgets.Composite",{"parent":"w53","style":["NONE"],"bounds":[0,0,1008,586],"children":["w96","w97"],"tabIndex":-1,"clientArea":[0,0,1008,586]}],["create","w96","rwt.widgets.Label",{"parent":"w95","style":["NONE"],"bounds":[10,30,112,26],"tabIndex":-1,"customVariant":"variant_pageHeadline","text":"TableViewer"}],["create","w97","rwt.widgets.Composite",{"parent":"w95","style":["NONE"],"bounds":[0,61,1008,525],"children":["w98","w99","w226","w228"],"tabIndex":-1,"clientArea":[0,0,1008,525]}],["create","w98","rwt.widgets.Text",{"parent":"w97","style":["LEFT","SINGLE","BORDER"],"bounds":[10,10,988,32],"tabIndex":22,"activeKeys":["#13","#27","#40"]}],["listen","w98",{"KeyDown":true,"Modify":true}],["create","w99","rwt.widgets.Grid",{"parent":"w97","style":["SINGLE","BORDER"],"appearance":"table","indentionWidth":0,"treeColumn":-1,"markupEnabled":false}],["create","w100","rwt.widgets.ScrollBar",{"parent":"w99","style":["HORIZONTAL"]}],["create","w101","rwt.widgets.ScrollBar",{"parent":"w99","style":["VERTICAL"]}],["set","w99",{"bounds":[10,52,988,402],"children":[],"tabIndex":23,"activeKeys":["CTRL+#70","CTRL+#78","CTRL+#82","CTRL+#89","CTRL+#83","CTRL+#71","CTRL+#69"],"cancelKeys":["CTRL+#70","CTRL+#78","CTRL+#82","CTRL+#89","CTRL+#83","CTRL+#71","CTRL+#69"]}],["listen","w99",{"MouseDown":true,"MouseUp":true,"MouseDoubleClick":true,"KeyDown":true}],["set","w99",{"itemCount":118,"itemHeight":28,"itemMetrics":[[0,0,50,3,0,3,44],[1,50,50,53,0,53,44],[2,100,140,103,0,103,134],[3,240,180,243,0,243,174],[4,420,50,423,0,423,44],[5,470,50,473,0,473,44]],"columnCount":6,"headerHeight":35,"headerVisible":true,"linesVisible":true,"focusItem":"w108","selection":["w108"]}],["listen","w99",{"Selection":true,"DefaultSelection":true}],["set","w99",{"enableCellToolTip":true}],["listen","w100",{"Selection":true}],["set","w101",{"visibility":true}],["listen","w101",{"Selection":true}],["create","w102","rwt.widgets.GridColumn",{"parent":"w99","text":"Nr.","width":50,"moveable":true}],["listen","w102",{"Selection":true}],["create","w103","rwt.widgets.GridColumn",{"parent":"w99","text":"Sym.","index":1,"left":50,"width":50,"moveable":true}],["listen","w103",{"Selection":true}],["create","w104","rwt.widgets.GridColumn",{"parent":"w99","text":"Name","index":2,"left":100,"width":140,"moveable":true}],["listen","w104",{"Selection":true}],["create","w105","rwt.widgets.GridColumn",{"parent":"w99","text":"Series","index":3,"left":240,"width":180,"moveable":true}],["listen","w105",{"Selection":true}],["create","w106","rwt.widgets.GridColumn",{"parent":"w99","text":"Group","index":4,"left":420,"width":50,"moveable":true}],["listen","w106",{"Selection":true}],["create","w107","rwt.widgets.GridColumn",{"parent":"w99","text":"Period","index":5,"left":470,"width":50,"moveable":true}],["listen","w107",{"Selection":true}],["create","w108","rwt.widgets.GridItem",{"parent":"w99","index":0,"texts":["1","H","Hydrogen","Nonmetal","1","1"],"cellBackgrounds":[null,null,null,[138,226,52,255],null,null]}],["create","w109","rwt.widgets.GridItem",{"parent":"w99","index":1,"texts":["2","He","Helium","Noble gas","18","1"],"cellBackgrounds":[null,null,null,[114,159,207,255],null,null]}],["create","w110","rwt.widgets.GridItem",{"parent":"w99","index":2,"texts":["3","Li","Lithium","Alkali metal","1","2"],"cellBackgrounds":[null,null,null,[239,41,41,255],null,null]}],["create","w111","rwt.widgets.GridItem",{"parent":"w99","index":3,"texts":["4","Be","Beryllium","Alkaline earth metal","2","2"],"cellBackgrounds":[null,null,null,[233,185,110,255],null,null]}],["create","w112","rwt.widgets.GridItem",{"parent":"w99","index":4,"texts":["5","B","Boron","Metalloid","13","2"],"cellBackgrounds":[null,null,null,[156,159,153,255],null,null]}],["create","w113","rwt.widgets.GridItem",{"parent":"w99","index":5,"texts":["6","C","Carbon","Nonmetal","14","2"],"cellBackgrounds":[null,null,null,[138,226,52,255],null,null]}],["create","w114","rwt.widgets.GridItem",{"parent":"w99","index":6,"texts":["7","N","Nitrogen","Nonmetal","15","2"],"cellBackgrounds":[null,null,null,[138,226,52,255],null,null]}],["create","w115","rwt.widgets.GridItem",{"parent":"w99","index":7,"texts":["8","O","Oxygen","Nonmetal","16","2"],"cellBackgrounds":[null,null,null,[138,226,52,255],null,null]}],["create","w116","rwt.widgets.GridItem",{"parent":"w99","index":8,"texts":["9","F","Fluorine","Halogen","17","2"],"cellBackgrounds":[null,null,null,[252,233,79,255],null,null]}],["create","w117","rwt.widgets.GridItem",{"parent":"w99","index":9,"texts":["10","Ne","Neon","Noble gas","18","2"],"cellBackgrounds":[null,null,null,[114,159,207,255],null,null]}],["create","w118","rwt.widgets.GridItem",{"parent":"w99","index":10,"texts":["11","Na","Sodium","Alkali metal","1","3"],"cellBackgrounds":[null,null,null,[239,41,41,255],null,null]}],["create","w119","rwt.widgets.GridItem",{"parent":"w99","index":11,"texts":["12","Mg","Magnesium","Alkaline earth metal","2","3"],"cellBackgrounds":[null,null,null,[233,185,110,255],null,null]}],["create","w120","rwt.widgets.GridItem",{"parent":"w99","index":12,"texts":["13","Al","Aluminium","Poor metal","13","3"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w121","rwt.widgets.GridItem",{"parent":"w99","index":13,"texts":["14","Si","Silicon","Metalloid","14","3"],"cellBackgrounds":[null,null,null,[156,159,153,255],null,null]}],["create","w122","rwt.widgets.GridItem",{"parent":"w99","index":14,"texts":["15","P","Phosphorus","Nonmetal","15","3"],"cellBackgrounds":[null,null,null,[138,226,52,255],null,null]}],["create","w123","rwt.widgets.GridItem",{"parent":"w99","index":15,"texts":["16","S","Sulfur","Nonmetal","16","3"],"cellBackgrounds":[null,null,null,[138,226,52,255],null,null]}],["create","w124","rwt.widgets.GridItem",{"parent":"w99","index":16,"texts":["17","Cl","Chlorine","Halogen","17","3"],"cellBackgrounds":[null,null,null,[252,233,79,255],null,null]}],["create","w125","rwt.widgets.GridItem",{"parent":"w99","index":17,"texts":["18","Ar","Argon","Noble gas","18","3"],"cellBackgrounds":[null,null,null,[114,159,207,255],null,null]}],["create","w126","rwt.widgets.GridItem",{"parent":"w99","index":18,"texts":["19","K","Potassium","Alkali metal","1","4"],"cellBackgrounds":[null,null,null,[239,41,41,255],null,null]}],["create","w127","rwt.widgets.GridItem",{"parent":"w99","index":19,"texts":["20","Ca","Calcium","Alkaline earth metal","2","4"],"cellBackgrounds":[null,null,null,[233,185,110,255],null,null]}],["create","w128","rwt.widgets.GridItem",{"parent":"w99","index":20,"texts":["21","Sc","Scandium","Transition metal","3","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w129","rwt.widgets.GridItem",{"parent":"w99","index":21,"texts":["22","Ti","Titanium","Transition metal","4","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w130","rwt.widgets.GridItem",{"parent":"w99","index":22,"texts":["23","V","Vanadium","Transition metal","5","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w131","rwt.widgets.GridItem",{"parent":"w99","index":23,"texts":["24","Cr","Chromium","Transition metal","6","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w132","rwt.widgets.GridItem",{"parent":"w99","index":24,"texts":["25","Mn","Manganese","Transition metal","7","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w133","rwt.widgets.GridItem",{"parent":"w99","index":25,"texts":["26","Fe","Iron","Transition metal","8","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w134","rwt.widgets.GridItem",{"parent":"w99","index":26,"texts":["27","Co","Cobalt","Transition metal","9","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w135","rwt.widgets.GridItem",{"parent":"w99","index":27,"texts":["28","Ni","Nickel","Transition metal","10","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w136","rwt.widgets.GridItem",{"parent":"w99","index":28,"texts":["29","Cu","Copper","Transition metal","11","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w137","rwt.widgets.GridItem",{"parent":"w99","index":29,"texts":["30","Zn","Zinc","Transition metal","12","4"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w138","rwt.widgets.GridItem",{"parent":"w99","index":30,"texts":["31","Ga","Gallium","Poor metal","13","4"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w139","rwt.widgets.GridItem",{"parent":"w99","index":31,"texts":["32","Ge","Germanium","Metalloid","14","4"],"cellBackgrounds":[null,null,null,[156,159,153,255],null,null]}],["create","w140","rwt.widgets.GridItem",{"parent":"w99","index":32,"texts":["33","As","Arsenic","Metalloid","15","4"],"cellBackgrounds":[null,null,null,[156,159,153,255],null,null]}],["create","w141","rwt.widgets.GridItem",{"parent":"w99","index":33,"texts":["34","Se","Selenium","Nonmetal","16","4"],"cellBackgrounds":[null,null,null,[138,226,52,255],null,null]}],["create","w142","rwt.widgets.GridItem",{"parent":"w99","index":34,"texts":["35","Br","Bromine","Halogen","17","4"],"cellBackgrounds":[null,null,null,[252,233,79,255],null,null]}],["create","w143","rwt.widgets.GridItem",{"parent":"w99","index":35,"texts":["36","Kr","Krypton","Noble gas","18","4"],"cellBackgrounds":[null,null,null,[114,159,207,255],null,null]}],["create","w144","rwt.widgets.GridItem",{"parent":"w99","index":36,"texts":["37","Rb","Rubidium","Alkali metal","1","5"],"cellBackgrounds":[null,null,null,[239,41,41,255],null,null]}],["create","w145","rwt.widgets.GridItem",{"parent":"w99","index":37,"texts":["38","Sr","Strontium","Alkaline earth metal","2","5"],"cellBackgrounds":[null,null,null,[233,185,110,255],null,null]}],["create","w146","rwt.widgets.GridItem",{"parent":"w99","index":38,"texts":["39","Y","Yttrium","Transition metal","3","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w147","rwt.widgets.GridItem",{"parent":"w99","index":39,"texts":["40","Zr","Zirconium","Transition metal","4","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w148","rwt.widgets.GridItem",{"parent":"w99","index":40,"texts":["41","Nb","Niobium","Transition metal","5","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w149","rwt.widgets.GridItem",{"parent":"w99","index":41,"texts":["42","Mo","Molybdenum","Transition metal","6","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w150","rwt.widgets.GridItem",{"parent":"w99","index":42,"texts":["43","Tc","Technetium","Transition metal","7","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w151","rwt.widgets.GridItem",{"parent":"w99","index":43,"texts":["44","Ru","Ruthenium","Transition metal","8","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w152","rwt.widgets.GridItem",{"parent":"w99","index":44,"texts":["45","Rh","Rhodium","Transition metal","9","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w153","rwt.widgets.GridItem",{"parent":"w99","index":45,"texts":["46","Pd","Palladium","Transition metal","10","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w154","rwt.widgets.GridItem",{"parent":"w99","index":46,"texts":["47","Ag","Silver","Transition metal","11","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w155","rwt.widgets.GridItem",{"parent":"w99","index":47,"texts":["48","Cd","Cadmium","Transition metal","12","5"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w156","rwt.widgets.GridItem",{"parent":"w99","index":48,"texts":["49","In","Indium","Poor metal","13","5"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w157","rwt.widgets.GridItem",{"parent":"w99","index":49,"texts":["50","Sn","Tin","Poor metal","14","5"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w158","rwt.widgets.GridItem",{"parent":"w99","index":50,"texts":["51","Sb","Antimony","Metalloid","15","5"],"cellBackgrounds":[null,null,null,[156,159,153,255],null,null]}],["create","w159","rwt.widgets.GridItem",{"parent":"w99","index":51,"texts":["52","Te","Tellurium","Metalloid","16","5"],"cellBackgrounds":[null,null,null,[156,159,153,255],null,null]}],["create","w160","rwt.widgets.GridItem",{"parent":"w99","index":52,"texts":["53","I","Iodine","Halogen","17","5"],"cellBackgrounds":[null,null,null,[252,233,79,255],null,null]}],["create","w161","rwt.widgets.GridItem",{"parent":"w99","index":53,"texts":["54","Xe","Xenon","Noble gas","18","5"],"cellBackgrounds":[null,null,null,[114,159,207,255],null,null]}],["create","w162","rwt.widgets.GridItem",{"parent":"w99","index":54,"texts":["55","Cs","Caesium","Alkali metal","1","6"],"cellBackgrounds":[null,null,null,[239,41,41,255],null,null]}],["create","w163","rwt.widgets.GridItem",{"parent":"w99","index":55,"texts":["56","Ba","Barium","Alkaline earth metal","2","6"],"cellBackgrounds":[null,null,null,[233,185,110,255],null,null]}],["create","w164","rwt.widgets.GridItem",{"parent":"w99","index":56,"texts":["57","La","Lanthanum","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w165","rwt.widgets.GridItem",{"parent":"w99","index":57,"texts":["58","Ce","Cerium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w166","rwt.widgets.GridItem",{"parent":"w99","index":58,"texts":["59","Pr","Praseodymium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w167","rwt.widgets.GridItem",{"parent":"w99","index":59,"texts":["60","Nd","Neodymium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w168","rwt.widgets.GridItem",{"parent":"w99","index":60,"texts":["61","Pm","Promethium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w169","rwt.widgets.GridItem",{"parent":"w99","index":61,"texts":["62","Sm","Samarium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w170","rwt.widgets.GridItem",{"parent":"w99","index":62,"texts":["63","Eu","Europium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w171","rwt.widgets.GridItem",{"parent":"w99","index":63,"texts":["64","Gd","Gadolinium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w172","rwt.widgets.GridItem",{"parent":"w99","index":64,"texts":["65","Tb","Terbium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w173","rwt.widgets.GridItem",{"parent":"w99","index":65,"texts":["66","Dy","Dysprosium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w174","rwt.widgets.GridItem",{"parent":"w99","index":66,"texts":["67","Ho","Holmium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w175","rwt.widgets.GridItem",{"parent":"w99","index":67,"texts":["68","Er","Erbium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w176","rwt.widgets.GridItem",{"parent":"w99","index":68,"texts":["69","Tm","Thulium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w177","rwt.widgets.GridItem",{"parent":"w99","index":69,"texts":["70","Yb","Ytterbium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w178","rwt.widgets.GridItem",{"parent":"w99","index":70,"texts":["71","Lu","Lutetium","Lanthanide","3","6"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w179","rwt.widgets.GridItem",{"parent":"w99","index":71,"texts":["72","Hf","Hafnium","Transition metal","4","6"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w180","rwt.widgets.GridItem",{"parent":"w99","index":72,"texts":["73","Ta","Tantalum","Transition metal","5","6"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w181","rwt.widgets.GridItem",{"parent":"w99","index":73,"texts":["74","W","Tungsten","Transition metal","6","6"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w182","rwt.widgets.GridItem",{"parent":"w99","index":74,"texts":["75","Re","Rhenium","Transition metal","7","6"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w183","rwt.widgets.GridItem",{"parent":"w99","index":75,"texts":["76","Os","Osmium","Transition metal","8","6"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w184","rwt.widgets.GridItem",{"parent":"w99","index":76,"texts":["77","Ir","Iridium","Transition metal","9","6"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w185","rwt.widgets.GridItem",{"parent":"w99","index":77,"texts":["78","Pt","Platinum","Transition metal","10","6"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w186","rwt.widgets.GridItem",{"parent":"w99","index":78,"texts":["79","Au","Gold","Transition metal","11","6"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w187","rwt.widgets.GridItem",{"parent":"w99","index":79,"texts":["80","Hg","Mercury","Transition metal","12","6"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w188","rwt.widgets.GridItem",{"parent":"w99","index":80,"texts":["81","Tl","Thallium","Poor metal","13","6"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w189","rwt.widgets.GridItem",{"parent":"w99","index":81,"texts":["82","Pb","Lead","Poor metal","14","6"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w190","rwt.widgets.GridItem",{"parent":"w99","index":82,"texts":["83","Bi","Bismuth","Poor metal","15","6"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w191","rwt.widgets.GridItem",{"parent":"w99","index":83,"texts":["84","Po","Polonium","Metalloid","16","6"],"cellBackgrounds":[null,null,null,[156,159,153,255],null,null]}],["create","w192","rwt.widgets.GridItem",{"parent":"w99","index":84,"texts":["85","At","Astatine","Halogen","17","6"],"cellBackgrounds":[null,null,null,[252,233,79,255],null,null]}],["create","w193","rwt.widgets.GridItem",{"parent":"w99","index":85,"texts":["86","Rn","Radon","Noble gas","18","6"],"cellBackgrounds":[null,null,null,[114,159,207,255],null,null]}],["create","w194","rwt.widgets.GridItem",{"parent":"w99","index":86,"texts":["87","Fr","Francium","Alkali metal","1","7"],"cellBackgrounds":[null,null,null,[239,41,41,255],null,null]}],["create","w195","rwt.widgets.GridItem",{"parent":"w99","index":87,"texts":["88","Ra","Radium","Alkaline earth metal","2","7"],"cellBackgrounds":[null,null,null,[233,185,110,255],null,null]}],["create","w196","rwt.widgets.GridItem",{"parent":"w99","index":88,"texts":["89","Ac","Actinium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w197","rwt.widgets.GridItem",{"parent":"w99","index":89,"texts":["90","Th","Thorium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w198","rwt.widgets.GridItem",{"parent":"w99","index":90,"texts":["91","Pa","Protactinium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w199","rwt.widgets.GridItem",{"parent":"w99","index":91,"texts":["92","U","Uranium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w200","rwt.widgets.GridItem",{"parent":"w99","index":92,"texts":["93","Np","Neptunium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w201","rwt.widgets.GridItem",{"parent":"w99","index":93,"texts":["94","Pu","Plutonium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w202","rwt.widgets.GridItem",{"parent":"w99","index":94,"texts":["95","Am","Americium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w203","rwt.widgets.GridItem",{"parent":"w99","index":95,"texts":["96","Cm","Curium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w204","rwt.widgets.GridItem",{"parent":"w99","index":96,"texts":["97","Bk","Berkelium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w205","rwt.widgets.GridItem",{"parent":"w99","index":97,"texts":["98","Cf","Californium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w206","rwt.widgets.GridItem",{"parent":"w99","index":98,"texts":["99","Es","Einsteinium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w207","rwt.widgets.GridItem",{"parent":"w99","index":99,"texts":["100","Fm","Fermium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w208","rwt.widgets.GridItem",{"parent":"w99","index":100,"texts":["101","Md","Mendelevium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w209","rwt.widgets.GridItem",{"parent":"w99","index":101,"texts":["102","No","Nobelium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w210","rwt.widgets.GridItem",{"parent":"w99","index":102,"texts":["103","Lr","Lawrencium","Actinide","3","7"],"cellBackgrounds":[null,null,null,[173,127,168,255],null,null]}],["create","w211","rwt.widgets.GridItem",{"parent":"w99","index":103,"texts":["104","Rf","Rutherfordium","Transition metal","4","7"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w212","rwt.widgets.GridItem",{"parent":"w99","index":104,"texts":["105","Db","Dubnium","Transition metal","5","7"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w213","rwt.widgets.GridItem",{"parent":"w99","index":105,"texts":["106","Sg","Seaborgium","Transition metal","6","7"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w214","rwt.widgets.GridItem",{"parent":"w99","index":106,"texts":["107","Bh","Bohrium","Transition metal","7","7"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w215","rwt.widgets.GridItem",{"parent":"w99","index":107,"texts":["108","Hs","Hassium","Transition metal","8","7"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w216","rwt.widgets.GridItem",{"parent":"w99","index":108,"texts":["109","Mt","Meitnerium","Transition metal","9","7"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w217","rwt.widgets.GridItem",{"parent":"w99","index":109,"texts":["110","Ds","Darmstadtium","Transition metal","10","7"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w218","rwt.widgets.GridItem",{"parent":"w99","index":110,"texts":["111","Rg","Roentgenium","Transition metal","11","7"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w219","rwt.widgets.GridItem",{"parent":"w99","index":111,"texts":["112","Uub","Ununbium","Transition metal","12","7"],"cellBackgrounds":[null,null,null,[252,175,62,255],null,null]}],["create","w220","rwt.widgets.GridItem",{"parent":"w99","index":112,"texts":["113","Uut","Ununtrium","Poor metal","13","7"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w221","rwt.widgets.GridItem",{"parent":"w99","index":113,"texts":["114","Uuq","Ununquadium","Poor metal","14","7"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w222","rwt.widgets.GridItem",{"parent":"w99","index":114,"texts":["115","Uup","Ununpentium","Poor metal","15","7"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w223","rwt.widgets.GridItem",{"parent":"w99","index":115,"texts":["116","Uuh","Ununhexium","Poor metal","16","7"],"cellBackgrounds":[null,null,null,[238,238,236,255],null,null]}],["create","w224","rwt.widgets.GridItem",{"parent":"w99","index":116,"texts":["117","Uus","Ununseptium","Halogen","17","7"],"cellBackgrounds":[null,null,null,[252,233,79,255],null,null]}],["create","w225","rwt.widgets.GridItem",{"parent":"w99","index":117,"texts":["118","Uuo","Ununoctium","Noble gas","18","7"],"cellBackgrounds":[null,null,null,[114,159,207,255],null,null]}],["create","w226","rwt.widgets.Composite",{"parent":"w97","style":["BORDER"],"bounds":[10,464,988,25],"children":["w227"],"tabIndex":-1,"clientArea":[0,0,986,23]}],["create","w227","rwt.widgets.Label",{"parent":"w226","style":["NONE"],"bounds":[10,10,966,3],"tabIndex":-1,"text":"Hydrogen (H)"}],["create","w228","rwt.widgets.Label",{"parent":"w97","style":["WRAP"],"bounds":[10,499,988,16],"tabIndex":-1,"foreground":[150,150,150,255],"font":[["Verdana","Lucida Sans","Arial","Helvetica","sans-serif"],10,false,false],"text":"Shortcuts: [CTRL+F] - Filter | Sort by: [CTRL+R] - Number, [CTRL+Y] - Symbol, [CTRL+N] - Name, [CTRL+S] - Series, [CTRL+G] - Group, [CTRL+E] - Period"}],["set","w1",{"focusControl":"w99"}],["call","rwt.client.BrowserNavigation","addToHistory",{"entries":[["tableviewer","TableViewer"]]}]]}'
    ! !

!JSJson methodsFor: 'benchmarking' stamp: 'StefanMarr 4/2/2015 22:43'!
benchJson
	^ (JSJsonParser with: JSJson RapBenchmarkMinified) parse! !

!JSJson methodsFor: 'benchmarking' stamp: 'StefanMarr 4/2/2015 22:44'!
verifyJson: result
	result class = JSParseException ifTrue: [ result println ].
	result isObject ifFalse: [^ false].
	(result asObject at: 'head') isObject ifFalse: [^ false].
	(result asObject at: 'operations') isArray ifFalse: [^ false].
	^ (result asObject at: 'operations') asArray size = 156! !

SMarkSuite subclass: #SMarkCompiler
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Classic'!

!SMarkCompiler class methodsFor: 'script entry' stamp: ''!
defaultNumberOfIterations
	^ 50! !

!SMarkCompiler class methodsFor: 'script entry' stamp: ''!
defaultNumberOfProcesses
	^ 8! !

!SMarkCompiler methodsFor: 'benchmarking' stamp: ''!
benchCompiler
	"was Benchmark>>testCompiler"
	SMarkCompilerTargetClass new compile:
		'somethingDummy: aBlock repeated: nTimes | i emptyBlock emptyTime blockTime |
		nTimes > 1000 ifTrue: [^self time: aBlock repeatedK: nTimes // 1000].
		emptyBlock := [].
		emptyTime := Time millisecondsToRun:
						[i := 0.
						 [(i := i + 1) <= nTimes] whileTrue: [emptyBlock value]].
		blockTime := Time millisecondsToRun:
						[i := 0.
						 [(i := i + 1) <= nTimes] whileTrue: [aBlock value]].
		^blockTime - emptyTime'.! !

SMarkSuite subclass: #SMarkDeltaBlue
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-DeltaBlue'!

!SMarkDeltaBlue commentStamp: '<historical>' prior: 0!
One-way constraint solver Benchmark. The main focus in DeltaBlue is on polymorphism and object-oriented programming

To run the benchmark, execute the expression 'SMarkDeltaBlue run'.!

!SMarkDeltaBlue methodsFor: 'benchmarking' stamp: 'StefanMarr 4/2/2015 17:29'!
benchDeltaBlue
	"This the combined benchmark."
	
	| n |
	
	n := self problemSize.
	
	DBStrength initialize.
	DBPlanner chainTest: n.
	DBPlanner projectionTest: n.
	^ true! !

!SMarkDeltaBlue methodsFor: 'defaults' stamp: 'ClementBera 9/23/2013 11:18'!
defaultProblemSize
	^ 5000! !

!SMarkDeltaBlue methodsFor: 'accessing' stamp: 'ClementBera 9/23/2013 11:05'!
problemSize
	<omniUnenforced> "Hint for the OMOP that it is part of the meta infrastructure"
	| ps |
	ps := super problemSize.
	
	ps isInteger ifFalse: [	^ self defaultProblemSize].
		
	^ ps! !

SMarkSuite subclass: #SMarkLoops
	instanceVariableNames: 'someInstVar'
	classVariableNames: 'ClassVarValue'
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Classic'!

!SMarkLoops commentStamp: '<historical>' prior: 0!
SMarkLoops implements a set of microbenchmarks measuring a number of basic aspects such as message send, instance field access, and array access cost.

Example use:
  (SMarkLoops new runOnly: #benchFloatLoop) run: 10 !

!SMarkLoops class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 1/10/2012 16:56'!
defaultProblemSize
	^ 50000! !

!SMarkLoops methodsFor: 'benchmarking' stamp: 'StefanMarr 9/12/2012 13:23'!
benchArrayAccess
	| i obj array |
	
	array := Array new: 1.
	obj   := Object new.
	
	i := self problemSize.
	[i > 0] whileTrue: [
		array at: 1.
		array at: 1 put: obj.
		
		i := i - 1.
	].! !

!SMarkLoops methodsFor: 'benchmarking' stamp: 'StefanMarr 1/10/2012 16:55'!
benchClassVarBinding
	| i r |
	i := self problemSize.
	[i > 0] whileTrue: [
		r := ClassVarValue.
		ClassVarValue := nil.

		i := i - 1.
	].! !

!SMarkLoops methodsFor: 'benchmarking' stamp: 'StefanMarr 1/10/2012 16:56'!
benchFloatLoop
	| a limit |
	a := 1.23.
	limit := self problemSize.
	[a < limit] whileTrue: [
		a := a + 0.6.
		a := a + 0.4.
	].! !

!SMarkLoops methodsFor: 'benchmarking' stamp: 'StefanMarr 1/10/2012 16:56'!
benchInstVarAccess
	| i r |
	i := self problemSize.
	[i > 0] whileTrue: [
		r := someInstVar.
		someInstVar := nil.

		i := i - 1.
	].! !

!SMarkLoops methodsFor: 'benchmarking' stamp: 'StefanMarr 1/10/2012 16:56'!
benchIntLoop
	| i |
	i := self problemSize.
	[i > 0] whileTrue: [
		i := i - 1.
	].! !

!SMarkLoops methodsFor: 'benchmarking' stamp: 'StefanMarr 1/10/2012 16:56'!
benchSend
	| i r |
	i := self problemSize.
	[i > 0] whileTrue: [
		self doNothing.
		i := i - 1.
	].! !

!SMarkLoops methodsFor: 'benchmarking' stamp: 'StefanMarr 1/10/2012 16:56'!
benchSendWithManyArguments
	| i r |
	i := self problemSize.
	[i > 0] whileTrue: [
		self doNothingA: i b: i c: i d: i e: i f: i g: i h: i i: i j: i.
		i := i - 1.
	].! !

!SMarkLoops methodsFor: 'helper' stamp: 'StefanMarr 1/10/2012 10:32'!
doNothing! !

!SMarkLoops methodsFor: 'helper' stamp: 'StefanMarr 1/10/2012 10:35'!
doNothingA: p1 b: p2 c: p3 d: p4 e: p5 f: p6 g: p7 h: p8 i: p9 j: p10! !

SMarkSuite subclass: #SMarkRichards
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Richards'!

!SMarkRichards commentStamp: '<historical>' prior: 0!
Richards is an OS kernel simulation benchmark, originally written in BCPL by Martin Richards. The main focus in Richards is on property access and calling functions and methods.!

!SMarkRichards methodsFor: 'benchs' stamp: 'StefanMarr 4/3/2015 14:12'!
benchRichards

	RichObject initialize.
	^ RichRunner start! !

!SMarkRichards methodsFor: 'benchs' stamp: 'ClementBera 9/27/2013 15:56'!
defaultProblemSize
	^ 50! !

!SMarkRichards methodsFor: 'benchs' stamp: 'ClementBera 9/27/2013 15:56'!
problemSize
	<omniUnenforced> "Hint for the OMOP that it is part of the meta infrastructure"
	| ps |
	ps := super problemSize.
	
	ps isInteger ifFalse: [	^ self defaultProblemSize].
		
	^ ps! !

!SMarkRichards methodsFor: 'benchs' stamp: 'StefanMarr 4/3/2015 14:12'!
verifyRichards: result
	^ result! !

SMarkSuite subclass: #SMarkSOM
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-SOM'!

SMarkSuite subclass: #SMarkSlopstone
	instanceVariableNames: 'obj o'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Classic'!

!SMarkSlopstone commentStamp: 'nishis 2/21/98 06:26' prior: 0!
modified for Squeak by nishis@urban.ne.jp
I added one class methods.
and modified execute method.  block variables are renamed for Squeak.
Thanx to Mr. Tim Olson, Mr. Kohler Markus, Mr. Tim Rowledge, Mr. John Maloney, Mr. Ian Piumarta.

original comments
	NAME			STones80
	AUTHOR			bruce@utafll.uta.edu (Bruce Samuelson)
	FUNCTION		low and medium level benchmarks for ST80 and ST/V
	ST-VERSIONS		pre R4.0, R4.0, R4.1, ST/V
	PREREQUISITES	need floating point hardware or emulation
	CONFLICTS		none
	DISTRIBUTION	world
	VERSION			1.0
	DATE			April 16, 1993

SUMMARY The filein includes two classes: Slopstones (Smalltalk Low level
OPeration Stones) and Smopstones (Smalltalk Medium level OPeration Stones).
Each includes seven cpu intensive benchmarks. They work equally well with
16-bit and 32-bit implementations and are designed to be portable to all
Smalltalk versions from ParcPlace and Digitalk. They are normalized to the
author's 486/33 Windows 3.1 machine running ParcPlace VisualWorks 1.0.
Results have been posted to the Usenet group comp.lang.smalltalk and form the
basis of an article that is scheduled to be published in the June issue of The
Smalltalk Report.

The only difference between the ST80 (STones80) and ST/V (STonesV) filein is
in the messages that define the classes in the first few lines of code. The
ST80 messages specify the class category and message protocol, which are not
used in ST/V.

Bruce Samuelson

!

!SMarkSlopstone methodsFor: 'benchmarking' stamp: 'StefanMarr 5/7/2013 17:16'!
benchStone
	"Using the SlopStone benchmarks, but not doing the
	 old style Stone performance number reporting"

	| n |
	n := self problemSize.
	n > SmallInteger maxVal
		ifTrue: [ self error: 'Count exceeded max small int.' ].
	
	o := obj new.
	
	1 to: n do: [:i | self doIntAdd         ].
	1 to: n do: [:i | self doFloatAdd       ].
	1 to: n do: [:i | self doStringAccess   ].
	1 to: n do: [:i | self doObjectCreation ].
	1 to: n do: [:i | self doObjectCopy     ].
	1 to: n do: [:i | self doPerform        ].
	1 to: n do: [:i | self doBlockValue     ].
! !

!SMarkSlopstone methodsFor: 'defaults' stamp: ''!
defaultProblemSize
	^ 1! !

!SMarkSlopstone methodsFor: 'stone benchmarks' stamp: 'StefanMarr 9/9/2012 01:52'!
doBlockValue
	[] value. [] value. [] value. [] value. [] value. [] value. [] value.
	[] value. [] value. [] value. [] value. [] value. [] value. [] value.
	[] value. [] value. [] value. [] value. [] value. [] value. [] value.
	[] value. [] value. [] value. [] value. [] value. [] value. [] value.
	[] value. [] value. [] value. [] value. [] value. [] value. [] value.
	[] value. [] value. [] value. [] value. [] value. [] value. [] value.
	[] value. [] value. [] value. [] value. [] value. [] value. [] value.
	[] value. [] value. [] value. [] value. [] value. [] value. [] value! !

!SMarkSlopstone methodsFor: 'stone benchmarks' stamp: 'StefanMarr 9/9/2012 01:48'!
doFloatAdd
	1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+
	1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0+1.0! !

!SMarkSlopstone methodsFor: 'stone benchmarks' stamp: 'StefanMarr 9/9/2012 01:47'!
doIntAdd
	1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+
	1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+
	1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+
	1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+
	1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+
	1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+
	1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1+1! !

!SMarkSlopstone methodsFor: 'stone benchmarks' stamp: 'StefanMarr 5/7/2013 17:16'!
doObjectCopy
	o copy copy copy copy copy copy copy copy copy copy! !

!SMarkSlopstone methodsFor: 'stone benchmarks' stamp: 'StefanMarr 5/7/2013 17:11'!
doObjectCreation
	obj new. obj new. obj new. obj new. obj new.
	obj new. obj new. obj new. obj new. obj new.
	obj new. obj new. obj new. obj new. obj new.
	obj new. obj new. obj new. obj new. obj new! !

!SMarkSlopstone methodsFor: 'stone benchmarks' stamp: 'StefanMarr 5/7/2013 17:17'!
doPerform
	o perform: #yourself. o perform: #yourself. o perform: #yourself.
	o perform: #yourself. o perform: #yourself. o perform: #yourself.
	o perform: #yourself. o perform: #yourself. o perform: #yourself.
	o perform: #yourself. o perform: #yourself. o perform: #yourself.
	o perform: #yourself. o perform: #yourself. o perform: #yourself.
	o perform: #yourself. o perform: #yourself. o perform: #yourself.
	o perform: #yourself. o perform: #yourself. o perform: #yourself.
	o perform: #yourself. o perform: #yourself. o perform: #yourself.
	o perform: #yourself. o perform: #yourself. o perform: #yourself.
	o perform: #yourself. o perform: #yourself. o perform: #yourself! !

!SMarkSlopstone methodsFor: 'stone benchmarks' stamp: 'StefanMarr 9/9/2012 01:50'!
doStringAccess
	'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1.
	'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1.
	'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1.
	'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1.
	'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1.
	'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1.
	'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1.
	'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1.
	'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1.
	'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1. 'a' at: 1! !

!SMarkSlopstone methodsFor: 'benchmarking' stamp: 'StefanMarr 5/8/2013 13:55'!
problemSize
	<omniUnenforced> "Hint for the OMOP that it is part of the meta infrastructure"
	| ps |
	ps := super problemSize.
	
	ps isInteger ifFalse: [	^ self defaultProblemSize].
		
	^ ps! !

!SMarkSlopstone methodsFor: 'benchmarking' stamp: 'StefanMarr 9/9/2012 02:16'!
readme
   "STEFAN: use the original benchmarks, but do not use original way
            of reporting results. Rely on the SMark reporting instead."

	"INTRODUCTION 
	 
	Slopstone: Smalltalk Low level OPeration Stones 
	Portable Low Level Benchmarks for ST80 and ST/V (using 16-bit 
	SmallIntegers) Placed in public domain January 1993  (c) Bruce 
	Samuelson Permission is given to place this in public Smalltalk archives 
	 
	Use monospaced fonts if possible to view the methods in this class. 
	 
	(1) Collect garbage if supported (2) do 'SlopstoneBenchmark new 
	runBenchmark'. Results are printed in the Transcript window. 
	Post results for your machines to comp.lang.smalltalk or 
	mail them to bruce@ling.uta.edu or bruce@utafll.uta.edu. 
	 
	DISCUSSION 
	    
	This readme method would normally be in the class comment for ST80. 
	ST/V-DOS doesn't support class comments. 
	  
	The benchmarks test strictly low level operations. They do not test 
	higher level operations such as forming sets, sorting, or streaming, nor 
	do they test 
	applications. They also do not test user interface operations because of 
	the non-portability of this area of Smalltalk and its sensitivity to the 
	performance of the video subsystem. The tests are cpu bound. They do 
	not access files and should not cause disk paging. 
	 
	The benchmarks use loop counts of 16000 because SmallIntegers cannot 
	exceed 16383 for ST/V-DOS. 16-bit implementions would perform worse 
	with large loop 
	counts. The benchmarks are also suitable for testing 32-bit versions of 
	Smalltalk. 
	 
	DEFINITION OF REFERENCE MACHINE (ONE SLOPSTONE) 
	 
	The following machine is the one on which I developed these 
	benchmarks. By 
	convention it is defined to operate at one slopstone. It's a mid range 
	performer for current ParcPlace versions of Smalltalk. 
	 
	Hardware: Amax 486DX/33 (includes internal floating point processor 
	and internal 8K cache), 256K external cache, 16MB RAM. 
	 
	Software: ParcPlace VisualWorks 1.0, Windows 3.1, DOS 5.0 (plain vanilla 
	setup). 
	 
	COMPARISON TO XEROX DORADO 
	 
	For reference, the machine runs at 649% of a Dorado on ParcPlace 
	benchmarks for ST80 4.1. Its fast video card helps on these PPS 
	benchmarks. I didn't run 
	them for VisualWorks 1.0. It would be somewhat slower because there 
	are vastly 
	more classes. 
	 
	EXAMPLE RESULTS FOR REFERENCE MACHINE 
	 
	1000s    time    1000s of 
	itera-   sec-    iterations   slop- 
	tions    onds    per sec      stones   explanation 
	 
	3808     0.577   6600         1.0      add integers 
	 544     2.262    240         1.0      add floats 
	 960     1.088    882         1.0      access strings 
	 320     0.908    352         1.0      create objects 
	 160     1.49     107         1.0      copy objects 
	 480     1.129    425         1.0      perform selectors 
	 896     1.237    724         1.0      evaluate blocks 
	 
	 640     1.151    555         1.0      geometric mean"
	^ self! !

!SMarkSlopstone methodsFor: 'initialize-release' stamp: 'StefanMarr 5/16/2013 12:26'!
setUp
	obj := Object.! !

SMarkSuite subclass: #SMarkSmopstone
	instanceVariableNames: 'ordCollectionFact intervalFact rectFactory sortedColFact arrFact setFact pointFact'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-Classic'!

!SMarkSmopstone commentStamp: 'nishis 2/21/98 06:26' prior: 0!
modified for Squeak by nishis@urban.ne.jp
see the comment of SlopstoneBenchmark class.
I added all class methods and initialization>initilize method,
modified:
	streamTestsOn: method.  In some case use testPrintOn: s base: instead of printOn:,
	primesUpTo: method. From recursively calling a block to whileTrue: block.
	sorcerersApprentice method. From recursively calling a block to  recursive method.
	execute and setup method.
Thanx to Mr. Tim Olson, Mr. Kohler Markus, Mr. Tim Rowledge, Mr. John Maloney, Mr. Ian Piumarta.

originak code by
	NAME			STones80
	AUTHOR			bruce@utafll.uta.edu (Bruce Samuelson)
	FUNCTION		low and medium level benchmarks for ST80 and ST/V
	ST-VERSIONS		pre R4.0, R4.0, R4.1, ST/V
	PREREQUISITES	need floating point hardware or emulation
	CONFLICTS		none
	DISTRIBUTION	world
	VERSION			1.0
	DATE			April 16, 1993

!

!SMarkSmopstone methodsFor: 'benchmarking' stamp: 'StefanMarr 5/16/2013 11:43'!
benchStone
	| n primes strings set |
	
	n := self problemSize.
	
	self fractonacci: n / 13. "13/2"
	
	primes := self primesUpTo: n * 100. "7351 - 1" "9000"
	self streamTestsOn: primes.
	
	strings := self stringsUpTo: n * 100. "8000"
	set := self setFrom: strings.
	self sort: set.
	
	self sorcerersApprentice: n.! !

!SMarkSmopstone methodsFor: 'defaults' stamp: 'StefanMarr 5/16/2013 11:44'!
defaultProblemSize
	^ 80! !

!SMarkSmopstone methodsFor: 'stone benchmarks' stamp: 'nishis 2/14/98 02:47'!
fractonacci: n 
	"Return something like the fibonacci function of n but
	using fractional numbers rather than whole ones. The
	reason for this variation is to run long enough to get
	a decent time measurement without exceeding 16383, the
	limit of small integers for ST/V-DOS. Choosing n = 13/2
	takes enough time and computes to 13581.

	Fibonacci uses n-1 and n-2 instead of n-(1/2) and n-(1/3).
	However, I couldn't get it to run in the above constraints.

	This benchmark tests the efficiency of recursively calling
	a method that does a little fractional arithmetic internally."

	^n > 1
		ifTrue: [(self fractonacci: n - (1/2)) + (self fractonacci: n - (1/3))]
		ifFalse: [1]! !

!SMarkSmopstone methodsFor: 'stone benchmarks' stamp: 'StefanMarr 5/7/2013 18:10'!
primesUpTo: n
	"modified by nishis"

	"Return the prime numbers between 2 and n.

	This method tests the efficiency of recursively calling a block
	that does some collection enumeration based on integer arithmetic."

	| nSqrt lowPrimes highPrimes genNext first |
	n < 5 | (n > 16363) ifTrue: [self error: 'Prime limit(s) out of range.'].
	nSqrt := n sqrt rounded.
	lowPrimes := ordCollectionFact with: 2.
	highPrimes := intervalFact from: 5 to: n by: 2.
	
	genNext := [:nextPrime |
		lowPrimes add: nextPrime.
		highPrimes := highPrimes select: [:k | k \\ nextPrime ~= 0].
		(first := highPrimes first) <= nSqrt ifTrue: [genNext value: first]].
	
	genNext value: 3.

	^ lowPrimes , highPrimes! !

!SMarkSmopstone methodsFor: 'benchmarking' stamp: 'StefanMarr 5/8/2013 13:55'!
problemSize
	<omniUnenforced> "Hint for the OMOP that it is part of the meta infrastructure"
	| ps |
	ps := super problemSize.
	
	ps isInteger ifFalse: [	^ self defaultProblemSize].
		
	^ ps! !

!SMarkSmopstone methodsFor: 'benchmarking' stamp: 'StefanMarr 9/9/2012 02:50'!
readme
   "STEFAN: use the original benchmarks, but do not use original way
            of reporting results. Rely on the SMark reporting instead."

	"INTRODUCTION 
	 
	Smopstone: Smalltalk Medium level OPeration Stones 
	Portable Medium level Benchmarks for ST80 and ST/V (using 16-bit 
	SmallInts) Placed in public domain January 1993  (c) Bruce Samuelson 
	Permission is given to place this in public Smalltalk archives 
	 
	Use monospaced fonts if possible to view the methods in this class. 
	 
	(1) Collect garbage if supported (2) do 'SmopstoneBenchmark new 
	runBenchmark'. Results are printed in the Transcript window. 
	Post results for your machines to comp.lang.smalltalk or 
	mail them to bruce@ling.uta.edu or bruce@utafll.uta.edu. 
	 
	DISCUSSION 
	    
	This readme method would normally be in the class comment for ST80. 
	ST/V-DOS doesn't support class comments. 
	 
	These benchmarks are a companion to the SlopstoneBenchmark class 
	posted to 
	comp.lang.smalltalk this month. Slopstones tested low level operations. 
	  
	Smopstones test medium level operations that exercise recursive block 
	and  
	method calls, collection building and enumeration, streaming, and 
	sorting. The 
	lower level operations contained in them exercise arithmetic (mostly 
	integer, with some fractions and floats) string manipulation, and low 
	level streaming. 
	 
	The benchmarks do not test applications. They also do not test user 
	interface performance because of the non-portability of this area of 
	Smalltalk and its  
	sensitivity to the speed of the video subsystem. The tests are cpu bound. 
	They do not access files and should not cause disk paging. 
	 
	The main weaknesses of the benchmarks are (1) they are not high 
	enough level 
	to test actual applications, and (2) they concentrate in too few areas of 
	Smalltalk, omitting many of the diverse capabilities of its class library. 
	My excuse is that one can only devote limited time writing public 
	domain benchmarks. 
	 
	The tests avoid generating integers larger than 16383, the maximum 
	SmallInteger in ST/V-DOS. 16-bit implementions would perform worse 
	with larger 
	integers. The benchmarks are also suitable for testing 32-bit versions of 
	Smalltalk. They try to avoid other pitfalls that would skew the results 
	such as the lack of an adequate hash function for a class. Someone 
	warned of this 
	in comp.lang.smalltalk (I forget who). 
	 
	DEFINITION OF REFERENCE MACHINE (ONE SMOPSTONE) 
	 
	The following machine is the one on which I developed these 
	benchmarks. By 
	convention it is defined to operate at one smopstone. It's a mid range 
	performer for current ParcPlace versions of Smalltalk. 
	 
	Hardware: Amax 486DX/33 (includes internal floating point processor 
	and internal 8K cache), 256K external cache, 16MB RAM. 
	 
	Software: ParcPlace VisualWorks 1.0, Windows 3.1, DOS 5.0 (plain vanilla 
	setup). 
	 
	COMPARISON TO XEROX DORADO 
	 
	For reference, the machine runs at 649% of a Dorado on ParcPlace 
	benchmarks for ST80 4.1. Its fast video card helps on these PPS 
	benchmarks. I didn't run 
	them for VisualWorks 1.0. It would be somewhat slower because there 
	are vastly 
	more classes. 
	 
	EXAMPLE RESULTS FOR REFERENCE MACHINE 
	 
	time in    smop- 
	seconds    stones    explanation 
	 
	 
	3.157      1.0       generating fractonaccis 
	1.123      1.0       generating primes 
	1.091      1.0       generating and parsing streams 
	3.091      1.0       generating strings 
	1.167      1.0       forming sets 
	5.139      1.0       sorting strings 
	5.601      1.0       sorcerer's apprentice 
	 
	2.355      1.0       geometric mean"
	^ self! !

!SMarkSmopstone methodsFor: 'stone benchmarks' stamp: 'StefanMarr 5/7/2013 19:05'!
setFrom: collection
	"Form a set from collection and return it.

	This method tests the efficiency of building a fairly large set
	from strings. It indirectly tests the effectiveness of the string
	hash function. Strings are used often enough as dictionary keys
	that this may be worth including in the benchmark suite.  ST/V-DOS
	has a primitive hash for strings, and ST80 has an elaborate one
	written in Smalltalk."

	^ setFact withAll: collection! !

!SMarkSmopstone methodsFor: 'running' stamp: 'StefanMarr 5/7/2013 19:27'!
setUp
	ordCollectionFact := OrderedCollection.
	intervalFact      := Interval.
	rectFactory       := Rectangle.
	sortedColFact     := SortedCollection.
	arrFact           := Array.
	setFact           := Set.
	pointFact         := Point.! !

!SMarkSmopstone methodsFor: 'stone benchmarks' stamp: 'StefanMarr 5/16/2013 11:43'!
sorcerersApprentice: m
	"modified by nishis"

	" FORMATTED FOR MONOSPACED FONT

	Perform various operations on rectangles.

	This method tests the efficiency of recursively calling a block that
	includes lots of integer arithmetic, collection building, and collection
	enumeration. The method:

	(1) Creates a collection of pseudo random rectangles 
	(2) Forms a new collection of all their intersections 
	(3) Recursively continues until there are no more intersections 
	(4) Returns a collection with the counts of rectangles in each generation.

	Because the intersections are forming progressively smaller rectangles
	(we exclude intersections of a rectangle with itself), the algorithm will
	eventually converge. Depending on the choice of numeric parameters, it may
	converge very quickly or very slowly. The parameters used below make it
	converge in a reasonable amount of time (a few seconds on a one-smopstone
	machine). It took some experimentation with different combinations to
	achieve this.

	The pseudo random number generator isn't very good, but it's adequate
	for this benchmark. I had intended the number '87' it uses to be a prime,
	but 87 = 29 * 3. The numbers may have been a bit more random otherwise.

	One could write an algorithm that would converge much more quickly and in
	a more predictable amount of time by sorting the intermediate rectangles
	in two dimensions and not bothering to test for intersections those
	rectangles that are contained in mutually exclusive regions. We have
	chosen algorithmic simplicity over performance optimization. We simply
	perform intersections of each rectangle with every possible partner in
	each generation. The time consumed is quadratic in the number of rectangles.

	The algorithm originally stored rectangles in sets to eliminate duplicates.
	Unfortunately, ST/V-DOS uses the hash function inherited from Object for
	Rectangle, which will allow duplicates to be stored. So we were forced to
	store rectangles in ordered collections and eliminate duplicates by brute
	force. The brutality was heightened because we could not use the test
	collection>>includes: to decide whether to add a rectangle to the ordered
	collections, since ST/V-DOS does not define equality (=) for rectangles
	either. The remaining warts in the code are not worth explaining.

	In an actual application, these shortcomings of ST/V-DOS would have been
	overcome by adding subclasses and methods rather than writing kludgy code."

	| n firstGen intersection isIncluded counts r random
	a b c d e f g h generate nextGen |
	
	n := 20 * m.
	firstGen := ordCollectionFact new.
	counts   := ordCollectionFact new.
	r := 50.
	random := [r := r + 1 * 87 \\ n].
	m timesRepeat: [
		a := random value.
		b := random value.
		c := random value.
		d := random value.
		e := a min: b.
		f := c min: d.
		g := a max: b.
		h := c max: d.
		firstGen add: (rectFactory origin: (pointFact x: e y: f) corner: (pointFact x: g y: h))].
	generate := 
		[:lastGen |
		counts add: lastGen size.
		nextGen := ordCollectionFact new.
		lastGen do:
			[:r1 |
			lastGen do: 
				[:r2 | 
				(r1 origin ~= r2 origin or: [r1 corner ~= r2 corner])
				"In ST80 this test would have simply been r1 ~= r2"
					ifTrue:
						[(r1 intersects: r2)
							ifTrue:
								[intersection := r1 intersect: r2.
								isIncluded := false.                     "All these lines"
								nextGen do:                              "would have been"
									[:rec |                                "avoided if we"
										(rec origin = intersection origin and: "could have used"
										[rec corner = intersection corner])    "a set for"
											ifTrue: [isIncluded := true]].       "nextGen. See"
								isIncluded                               "explanation"
									ifFalse:                               "above."
										[nextGen size > 500
											ifTrue: [self error: 'Converges too slowly.']
											ifFalse: [nextGen add: intersection]]]]]].
			nextGen size > 0 ifTrue: [generate value: nextGen]].
	
	generate value: firstGen.
	^counts! !

!SMarkSmopstone methodsFor: 'stone benchmarks' stamp: 'StefanMarr 5/7/2013 18:18'!
sort: collection
	"Form a sorted collection from collection and return it.

	This method tests the efficiency of sorting a fairly large
	collection of strings. It indirectly measures the efficiency
	of the sorting algorithm and of string comparison operations."

	^collection as: sortedColFact! !

!SMarkSmopstone methodsFor: 'stone benchmarks' stamp: 'StefanMarr 5/7/2013 18:19'!
streamTestsOn: integers 
	"modified by nishis"
	"Test steaming operations on the collection of integers. 
	 
	This method measures the efficiency of integer-to-float conversion, of 
	printing numbers to a write stream, of parsing tokens in a read 
	stream,   and of converting the tokens from strings to numbers. The 
	technique for 
	converting tokens into floats is constrained by portability between 
	ST80 and ST/V. 
	 
	To validate the logic, the original integers are compared with the 
	final   floats. There should be no roundoff errors."
	"The following line accounts for the different implementations of 
	Float>>printString for some versions of Smalltalk. USA versions use 
	the decimal character, while some European versions use the comma 
	char.   Thanks to Marten Feldtmann for pointing this out."

	| delim space s floats float string |
	delim := 1.0 printString at: 2.	"$. for USA, $, for some Europe."

	space := Character value: 32.
	"Can't use Character space in ST/V-DOS"
	s := ReadWriteStream on: String new.
	integers do: 
		[:i |
			i asFloat printOn: s.
			s space].
	"Now make sure the underlying string size is < 16383, a 16-bit small int."
	s contents size > (SmallInteger maxVal) ifTrue: [self error: 'String too big.'].
	s reset.
	floats := ordCollectionFact new: integers size.
	[s atEnd]
		whileFalse: 
			[float := 0.0.
			string := s upTo: delim.
			s upTo: space.
			"In the following, digitValue is portable between ST80 and ST/V-DOS. "
			string do: [:char | float := float * 10.0 + char digitValue].
			floats add: float].
	integers = floats ifFalse: [self error: 'Numbers do not compare.']! !

!SMarkSmopstone methodsFor: 'stone benchmarks' stamp: 'StefanMarr 5/7/2013 20:18'!
stringsUpTo: n 
	"Return a collection of strings representing the integers from 1 
	to n with their digits reversed. 
	 
	This method tests the efficiency of creating small streams, performing 
	string operations, and building collections. It includes a gross kludge 
	to coerce portability between ST80 and ST/V. They vary slightly in the 
	selector used to reverse collections. Subsequent to freezing this code, 
	someone pointed out that it could have been written more simply 
	without   using perform:. It could also have been simplified using 
	select:. It 
	was too late to change. The techniques are left as an exercise for the 
	reader."

	| selector |
	(arrFact with: #reverse with: #reversed)
		do: [:symbol | (String canUnderstand: symbol)
			ifTrue: [selector := symbol]].
	^ (intervalFact from: 1 to: n)
		collect: [:m | m printString perform: selector]! !

!SMarkSuite class methodsFor: 'benchmarking' stamp: 'tfel 5/11/2015 16:20'!
benchmarkSelectors

	^(self selectors asArray select: [ :each | 
		(each beginsWith: 'bench') and: [ each numArgs isZero ] ]) sort! !

!SMarkSuite class methodsFor: 'defaults' stamp: ''!
defaultProblemSize
	^ nil! !

!SMarkSuite class methodsFor: 'defaults' stamp: 'StefanMarr 12/30/2011 22:43'!
defaultRunner
	^ self onCog: [SMarkCogRunner]
	       else:  [SMarkRunner]! !

!SMarkSuite class methodsFor: 'history' stamp: 'tfel 5/11/2015 19:42'!
historyAdd: aResult

	history ifNil: [history := IdentityDictionary new].
	history add: aResult.
	history size > self maxHistory ifTrue: [history := history copyWithoutFirst].! !

!SMarkSuite class methodsFor: 'history' stamp: 'tfel 5/11/2015 19:46'!
historyAdd: aResult at: selector
	| list |
	history ifNil: [history := IdentityDictionary new].
	list := history at: selector ifAbsentPut: OrderedCollection new.
	list add: aResult.
	list size > self maxHistory ifTrue: [
		history at: selector put: list copyWithoutFirst].! !

!SMarkSuite class methodsFor: 'history' stamp: 'tfel 5/11/2015 20:12'!
historyAt: aSelector

	history ifNil: [^ nil].
	^ history at: aSelector ifAbsent: [nil]! !

!SMarkSuite class methodsFor: 'testing' stamp: 'tfel 5/11/2015 16:06'!
isAbstract

	^ self name = #SMarkSuite! !

!SMarkSuite class methodsFor: 'benchmarking' stamp: ''!
isAbstractClass
	"This is a hack that is necessary in Squeak since it does not provide #isAbstractClass.
	 Actually this class is supposed to be abstract, but well, inheritance..."
	
	^ false! !

!SMarkSuite class methodsFor: 'history' stamp: 'tfel 5/11/2015 19:42'!
maxHistory
	"The maximum amount of history to keep"
	^ 10! !

!SMarkSuite class methodsFor: 'platform support' stamp: 'StefanMarr 4/5/2012 22:20'!
onCog: cogSpecificBlock else: general
	^ (Smalltalk vm isRunningCogit)
		ifTrue:  [cogSpecificBlock value]
		ifFalse: [general value]! !

!SMarkSuite class methodsFor: 'profiling' stamp: 'CamilloBruni 1/28/2013 18:22'!
profile: aSelector
	^ self profileRunner 
		execute: self new selector: aSelector.! !

!SMarkSuite class methodsFor: 'profiling' stamp: 'CamilloBruni 1/28/2013 18:30'!
profile: aSelector iterations: nIterations
	^ self profileRunner 
		execute: self new selector: aSelector iterations: nIterations.! !

!SMarkSuite class methodsFor: 'profiling' stamp: 'CamilloBruni 1/28/2013 18:21'!
profileAll
	^ self profileRunner 
		execute: self new.! !

!SMarkSuite class methodsFor: 'profiling' stamp: 'CamilloBruni 1/28/2013 18:17'!
profileRunner
	^ SMarkProfileRunner! !

!SMarkSuite class methodsFor: 'benchmarking' stamp: 'StephaneDucasse 6/16/2013 12:11'!
run
	"Execute the suite one time."
	^ self defaultRunner execute: self new.! !

!SMarkSuite class methodsFor: 'benchmarking' stamp: 'StephaneDucasse 6/16/2013 12:11'!
run: nIterations
	"Execute the suite a given number of iterations."
	
	^ self defaultRunner execute: self new with: nIterations.! !

!SMarkSuite class methodsFor: 'benchmarking' stamp: 'StefanMarr 6/30/2013 00:57'!
runOnly: aSelector
  "aSelector should refer to a benchmark method.
   Example:
     SMarkLoops runOnly: #benchFloatLoop
  "
  ^ self defaultRunner execute: (self new runOnly: aSelector)! !

!SMarkSuite class methodsFor: 'benchmarking' stamp: 'StephaneDucasse 6/16/2013 21:03'!
runOnly: aSelector iterations: anInteger
	"Execute only the bench name aSelector from the suite."
	
	^ self defaultRunner execute: (self new runOnly: aSelector) with: anInteger! !

!SMarkSuite methodsFor: 'helper' stamp: ''!
benchmarkNameForSelector: selector
	"Extracts the actual name of the benchmark from the selector"
	(selector beginsWith: #bench) ifTrue: [ ^ selector copyFrom: 6 to: selector size].
	^ selector asSymbol
	! !

!SMarkSuite methodsFor: 'running' stamp: 'MartinDias 3/12/2014 15:52'!
cleanUpInstanceVariables
	"Make sure all variables that are 'user variables' get cleaned"
	
	(self class allInstVarNames copyWithoutAll: self instanceVariablesToPreserve) do: [ :name |
		self instVarNamed: name put: nil ]! !

!SMarkSuite methodsFor: 'running' stamp: 'MartinDias 3/12/2014 15:48'!
instanceVariablesToPreserve
	^ #(runner)! !

!SMarkSuite methodsFor: 'benchmarking' stamp: ''!
performCustomSelector: aSelector with: aPrefix
	| customSelector |
	customSelector := (aPrefix, aSelector capitalized) asSymbol.
	(self respondsTo: customSelector) ifTrue: [ 
		self perform: customSelector].! !

!SMarkSuite methodsFor: 'benchmarking' stamp: 'StefanMarr 5/8/2013 13:55'!
problemSize
	<omniUnenforced> "Hint for the OMOP that it is part of the meta infrastructure"
	runner             ifNil: [^ self class defaultProblemSize].
	runner problemSize ifNil: [^ self class defaultProblemSize].
	^ runner problemSize! !

!SMarkSuite methodsFor: 'benchmarking' stamp: 'ReinoutStevens 12/6/2011 14:31'!
processResult: anObject withTimer: aSMarkTimer
	"subclass responsability. You can verify your results here, or do things with the timer."
	^self.! !

!SMarkSuite methodsFor: 'benchmarking' stamp: 'tfel 5/11/2015 19:52'!
run
	"Executes all the benchmarks in the suite, 
	 coordinating with the runner when necessary"
	
	| potentialBenchmarkSelectors |
	selectedBenchmarks
		ifNotNil: [ potentialBenchmarkSelectors := selectedBenchmarks copy ]
		ifNil:    [ potentialBenchmarkSelectors := self class allSelectors ].				
	
	potentialBenchmarkSelectors
		do: [ :selector |
			(self shouldRunSelector: selector)
				ifTrue: [
					runner performBenchmark: selector ]
		].
	! !

!SMarkSuite methodsFor: 'benchmarking' stamp: ''!
runBenchmark: aSelector
	
	[self setUp.
	 self performCustomSelector: aSelector with: #setUp.
	 runner timedBenchmarkExecution: aSelector] ensure: [
		self performCustomSelector: aSelector with: #tearDown.
		self tearDown.
		self cleanUpInstanceVariables]! !

!SMarkSuite methodsFor: 'benchmarking' stamp: ''!
runOnly: aSymbol
	selectedBenchmarks := IdentitySet newFrom: { aSymbol }.! !

!SMarkSuite methodsFor: 'accessing' stamp: ''!
runner
	^ runner! !

!SMarkSuite methodsFor: 'accessing' stamp: ''!
runner: aRunner
	runner := aRunner.! !

!SMarkSuite methodsFor: 'benchmarking' stamp: ''!
selectedBenchmarks
	^ selectedBenchmarks! !

!SMarkSuite methodsFor: 'running' stamp: 'StefanMarr 6/30/2013 00:59'!
setUp
	"It is the subclass' responsibility to set up the necessary environment for a benchmark"
	^ self! !

!SMarkSuite methodsFor: 'testing' stamp: 'StefanMarr 6/30/2013 00:59'!
shouldRunSelector: selector
	"Tells whether the given selector is in the form bench*, and thus is a benchmark that should be executed."
	
	(selector includes: $:) ifTrue: [ ^ false ].
	^ selector beginsWith: #bench! !

!SMarkSuite methodsFor: 'running' stamp: 'StefanMarr 6/30/2013 00:58'!
tearDown
	"It is the subclass' responsibility to clean up the environment after a benchmark"
	^ self! !

SMarkSuite subclass: #SMarkTestRunnerSuite
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkTestRunnerSuite commentStamp: 'StefanMarr 3/13/2011 22:58' prior: 0!
This is a benchmark suite used to test the benchmark runner!

!SMarkTestRunnerSuite class methodsFor: 'as yet unclassified' stamp: ''!
isAbstractClass
	"This is more like a hack, but we do not want to execute this as a benchmark"
	^ true! !

!SMarkTestRunnerSuite methodsFor: 'as yet unclassified' stamp: ''!
benchDoNothing
	"Just a dummy that should be executed and result in some measured value"
	^ self! !

!SMarkTestRunnerSuite methodsFor: 'as yet unclassified' stamp: ''!
benchNotifyRunner
	runner notifyRunnerWasExecuted.
	^ self! !

SMarkSuite subclass: #SMarkTestRunnerSuiteForAutosizing
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkTestRunnerSuiteForAutosizing class methodsFor: 'as yet unclassified' stamp: ''!
defaultRunner
	^ SMarkTestRunner! !

!SMarkTestRunnerSuiteForAutosizing methodsFor: 'as yet unclassified' stamp: ''!
baseBenchmark
	(Delay forMilliseconds: 5) wait.! !

!SMarkTestRunnerSuiteForAutosizing methodsFor: 'as yet unclassified' stamp: ''!
benchBig
	(Delay forMilliseconds: 100) wait.! !

!SMarkTestRunnerSuiteForAutosizing methodsFor: 'as yet unclassified' stamp: ''!
benchSmall
	(Delay forMilliseconds: 5) wait.! !

SMarkSuite subclass: #SMarkTestRunnerSuiteForPerfromBenchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkTestRunnerSuiteForPerfromBenchmark class methodsFor: 'as yet unclassified' stamp: ''!
defaultRunner
	^ SMarkTestRunner! !

!SMarkTestRunnerSuiteForPerfromBenchmark methodsFor: 'as yet unclassified' stamp: ''!
BenchShouldNotBeExecuted
	"This one should not be executed"
	(Delay forMilliseconds: 5) wait.
	runner didPerform: #BenchShouldNotBeExecuted.! !

!SMarkTestRunnerSuiteForPerfromBenchmark methodsFor: 'as yet unclassified' stamp: ''!
baseBenchmark
	(Delay forMilliseconds: 5) wait.
	runner didPerform: #baseBenchmark.! !

!SMarkTestRunnerSuiteForPerfromBenchmark methodsFor: 'as yet unclassified' stamp: ''!
benchAfsdfsdfsdf
	(Delay forMilliseconds: 5) wait.
	runner didPerform: #benchAfsdfsdfsdf.! !

!SMarkTestRunnerSuiteForPerfromBenchmark methodsFor: 'as yet unclassified' stamp: ''!
benchBsdfasereSDSfsdfsdfs
	(Delay forMilliseconds: 5) wait.
	runner didPerform: #benchBsdfasereSDSfsdfsdfs.! !

!SMarkTestRunnerSuiteForPerfromBenchmark methodsFor: 'as yet unclassified' stamp: ''!
benchC
	(Delay forMilliseconds: 5) wait.
	runner didPerform: #benchC.! !

!SMarkTestRunnerSuiteForPerfromBenchmark methodsFor: 'as yet unclassified' stamp: ''!
benchZZ
	(Delay forMilliseconds: 5) wait.
	runner didPerform: #benchZZ.! !

SMarkSuite subclass: #SMarkTestSuite
	instanceVariableNames: 'a b c d'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkTestSuite commentStamp: 'StefanMarr 3/12/2011 22:13' prior: 0!
This class is used for unit testing only.!

!SMarkTestSuite class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/20/2011 21:09'!
defaultRunner
	^ SMarkTestRunner! !

!SMarkTestSuite class methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/13/2011 20:57'!
isAbstractClass
	"This is more like a hack, but we do not want to execute this as a benchmark"
	^ true! !

!SMarkTestSuite methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/12/2011 22:15'!
a
	^a! !

!SMarkTestSuite methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/12/2011 22:15'!
b
	^b! !

!SMarkTestSuite methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 00:35'!
benchA
	a := #executed.
	(Delay forMilliseconds: 1) wait.! !

!SMarkTestSuite methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 00:38'!
benchB
	b := #executed.
	(Delay forMilliseconds: 1) wait.! !

!SMarkTestSuite methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/12/2011 22:37'!
benchC
	c := #executed.! !

!SMarkTestSuite methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/19/2011 01:04'!
benchWithMultipleCriteria
	| t1 t2 t3 |
	t3 := runner createTimer: 't3'.
	t3 start.
	
	(Delay forMilliseconds: 25) wait.
	
	t2:= runner createTimer: 't2'.
	t2 start.
	(Delay forMilliseconds: 15) wait.	
	
	t1 := runner createTimer: 't1'.  
	t1 start.
	(Delay forMilliseconds: 10) wait.
	
	
	t3 stop.
	t2 stop.
	t1 stop.    ! !

!SMarkTestSuite methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/12/2011 22:15'!
c
	^c! !

!SMarkTestSuite methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/12/2011 22:15'!
d
	^d! !

!SMarkTestSuite methodsFor: 'initialize-release' stamp: 'StefanMarr 3/18/2011 19:50'!
initialize
	super initialize.
	
	"Filling in some test data"
	a := 1.
	b := 2.
	c := #three.
	d := '42'.
	! !

!SMarkTestSuite methodsFor: 'running' stamp: 'StefanMarr 3/12/2011 22:39'!
setUp
	runner countSetUpInvoke! !

!SMarkTestSuite methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/13/2011 20:29'!
setUpBenchB
	runner didSetUpBenchB! !

!SMarkTestSuite methodsFor: 'running' stamp: 'StefanMarr 3/12/2011 22:39'!
tearDown
	runner countTearDownInvoke! !

!SMarkTestSuite methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/13/2011 20:30'!
tearDownBenchC
	runner didTearDownBenchC! !

SMarkTestSuite subclass: #SMarkTestSuiteWithBaseBenchmark
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Tests'!

!SMarkTestSuiteWithBaseBenchmark methodsFor: 'as yet unclassified' stamp: 'StefanMarr 3/18/2011 18:32'!
baseBenchmark
	runner didRunBaseBenchmark.! !

SMarkSuite subclass: #SomFannkuch
	instanceVariableNames: 'timesRotated perm atEnd'
	classVariableNames: 'Results'
	poolDictionaries: ''
	category: 'SMark-Benchmarks-SOM'!

!SomFannkuch class methodsFor: 'as yet unclassified' stamp: 'tfel 5/11/2015 15:46'!
expectedResult: problemSize
      Results isNil ifTrue: [
        Results := Array new: 12.
        Results at:  1 put:  0.
        Results at:  2 put:  1.
        Results at:  3 put:  2.
        Results at:  4 put:  4.
        Results at:  5 put:  7.
        Results at:  6 put: 10.
        Results at:  7 put: 16.
        Results at:  8 put: 22.
        Results at:  9 put: 30.
        Results at: 10 put: 38.
        Results at: 11 put: 51.
        Results at: 12 put: 65.
      ].
      ^ Results at: problemSize! !

!SomFannkuch methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 16:23'!
atEnd
        ^atEnd! !

!SomFannkuch methodsFor: 'benchmarking' stamp: 'StefanMarr 4/3/2015 16:24'!
benchFannkuch
		| result |
      self initialize: self problemSize.
      result := self maxPfannkuchen.
      ^ result = (SomFannkuch expectedResult: self problemSize)! !

!SomFannkuch methodsFor: 'benchmarking' stamp: 'StefanMarr 4/3/2015 16:26'!
benchmark
		| result |
      self initialize: self problemSize.
      result := self maxPfannkuchen.
      ^ result = (SomFannkuch expectedResult: self problemSize)! !

!SomFannkuch methodsFor: 'initialization' stamp: 'StefanMarr 4/3/2015 16:31'!
initialize: size
	perm := Array new: size.
	1 to: size do: [:i | perm at: i put: i ].
	timesRotated := Array new: size withAll: 0.
	atEnd := false! !

!SomFannkuch methodsFor: 'benchmarking' stamp: 'tfel 4/10/2015 11:17'!
innerBenchmarkLoop
		| result |
      self initialize: innerIterations.
      result := self maxPfannkuchen.
      ^ result = (SomFannkuch expectedResult: innerIterations).! !

!SomFannkuch methodsFor: 'initialize-release' stamp: 'StefanMarr 4/3/2015 16:27'!
makeNext
	| temp remainder |
	"* Generate the next permutation. *"
	2 to: perm size do: [ :r |
		"* Rotate the first r items to the left. *"
		temp := perm at: 1.
		1 to: r - 1 do: [ :i | perm at: i put: (perm at: i + 1) ].
		perm at: r put: temp.

		timesRotated at: r put: ((timesRotated at: r) + 1) % r.
		remainder := timesRotated at: r.
		remainder = 0 ifFalse: [ ^self ].

		"* After r rotations, the first r items are in their original positions.
		 Go on rotating the first r+1 items. *"
	].

	"* We are past the final permutation. *"
	atEnd := true.! !

!SomFannkuch methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 16:23'!
maxPfannkuchen
        | max permutation |
        max := 0.
        [self atEnd] whileFalse: [
            permutation := self next.
            max := max max: (self pfannkuchen: permutation)].
        ^max! !

!SomFannkuch methodsFor: 'accessing' stamp: 'StefanMarr 4/3/2015 16:24'!
next
        | result |
        result := perm copy.
        self makeNext.
        ^result! !

!SomFannkuch methodsFor: 'benchmarking' stamp: 'StefanMarr 4/3/2015 16:21'!
pfannkuchen: anArray
	| first complement a b k |
	k := 0.
	[ (first := anArray at: 1) = 1 ] whileFalse: [
		k := k + 1.
		complement := first + 1.

		1 to: first / 2 do: [ :i |
			a := anArray at: i.
			b := anArray at: complement - i.
			anArray at: i put: b.
			anArray at: complement - i put: a.
	] ].
	^ k! !

SMarkSuite subclass: #SomMandelbrot
	instanceVariableNames: 'firstResult'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-SOM'!

!SomMandelbrot methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 16:47'!
benchmark
	^ self mandelbrot: self problemSize! !

!SomMandelbrot methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 16:46'!
mandelbrot: size
        | sum byteAcc bitNum y |
        sum     := 0.
        byteAcc := 0.
        bitNum  := 0.
  
        y := 0.
        
        [y < size] whileTrue: [
            | ci x |
            ci := (2.0 * y / size) - 1.0.
            x  := 0.
            
            [x < size] whileTrue: [
                | zr zrzr zi zizi cr escape z notDone |
                zrzr := zr := 0.0.
                zizi := zi := 0.0.
                cr   := (2.0 * x / size) - 1.5.
                
                z := 0.
                notDone := true.
                escape := 0.
                [notDone and: [z < 50]] whileTrue: [
                    zr := zrzr - zizi + cr.
                    zi := 2.0 * zr * zi + ci.
            
                    "preserve recalculation"
                    zrzr := zr * zr.
                    zizi := zi * zi.
            
                    (zrzr + zizi > 4.0) ifTrue: [
                        notDone := false.
                        escape  := 1.
                    ].
                    z := z + 1.
                ].
  
                byteAcc := (byteAcc << 1) + escape.
                bitNum  := bitNum + 1.
  
                " Code is very similar for these cases, but using separate blocks
                  ensures we skip the shifting when it's unnecessary,
                  which is most cases. "
                bitNum = 8 
                    ifTrue: [
                      sum := sum bitXor: byteAcc.
                      byteAcc := 0.
                      bitNum  := 0. ]
                    ifFalse: [
                      (x = (size - 1)) ifTrue: [
                          byteAcc := byteAcc << (8 - bitNum).
                          sum := sum bitXor: byteAcc.
                          byteAcc := 0.
                          bitNum  := 0. ]].
                x := x + 1.
            ].
            y := y + 1.
        ].
        
        ^ sum! !

!SomMandelbrot methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 16:44'!
verifymark: result
	  self problemSize = 750 ifTrue: [
        ^ result = 50
      ].
      firstResult isNil ifTrue: [
        firstResult := result.
      ].
      ^ result = firstResult! !

SMarkSuite subclass: #SomNBody
	instanceVariableNames: 'expectedEnergy'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark-Benchmarks-NBody'!

!SomNBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:08'!
benchmark
	     | bodies |
        bodies := NBNBodySystem new.
        
        1 to: self problemSize do: [:i |
            bodies advance: 0.01.
        ].
        
        self problemSize = 250000 ifTrue: [
          ^ bodies energy = -0.1690859889909308
        ].
        expectedEnergy isNil ifTrue: [
          expectedEnergy := bodies energy.
          ^ true
        ].
        
        ^ expectedEnergy = bodies energy.! !

!SomNBody methodsFor: 'as yet unclassified' stamp: 'StefanMarr 4/3/2015 17:07'!
initialize
	NBBody initialize! !

Object subclass: #SMarkTimer
	instanceVariableNames: 'startTime elapsedTime name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

!SMarkTimer commentStamp: '<historical>' prior: 0!
A SMarkTimer is a simple timer.
A subclass can measure alternative metrics, or for instance use different time sources.

A subclass of SMarkRunner can then use the custom timer class by overriding SMarkRunner class >> #defaultTimer.!

!SMarkTimer class methodsFor: 'instance creation' stamp: 'StefanMarr 3/14/2011 08:54'!
new: aName
	| timer |
	
	timer := super new.
	timer name: aName.
	
	^timer! !

!SMarkTimer methodsFor: 'timing' stamp: 'StephaneDucasse 6/16/2013 21:15'!
currentMillis

	^ Time millisecondClockValue! !

!SMarkTimer methodsFor: 'initialization' stamp: 'ReinoutStevens 12/6/2011 14:32'!
initialize
	super initialize.
	elapsedTime := 0! !

!SMarkTimer methodsFor: 'accessing' stamp: 'StefanMarr 3/14/2011 08:54'!
name
	^name! !

!SMarkTimer methodsFor: 'accessing' stamp: 'StefanMarr 3/14/2011 08:54'!
name: aString
	name := aString ! !

!SMarkTimer methodsFor: 'timing' stamp: 'StefanMarr 3/14/2011 08:54'!
reset
	startTime := 0.
	elapsedTime := 0.! !

!SMarkTimer methodsFor: 'timing' stamp: 'StefanMarr 3/20/2011 12:24'!
start
	startTime := self currentMillis.! !

!SMarkTimer methodsFor: 'timing' stamp: 'StefanMarr 3/20/2011 12:25'!
stop
	| elapsedInThisPeriod current |
	current := self currentMillis.
	
	elapsedInThisPeriod := Time milliseconds: current since: startTime.
	
	elapsedTime := elapsedTime + elapsedInThisPeriod.! !

!SMarkTimer methodsFor: 'accessing' stamp: 'StefanMarr 3/14/2011 08:54'!
totalTime
	^elapsedTime! !

Object subclass: #SMarkTransporter
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

!SMarkTransporter commentStamp: 'StefanMarr 4/5/2012 21:43' prior: 0!
SMarkTransporter is a used to interact with a git-fileout system used in the RoarVM project to manage Smalltalk source code.

SMarkTransporter is not actually a Transporter class, since there are currently no needs for customization.
Thus, it is just a dummy class for future use, and to hold #transportersForFileOutMenu.
!

!SMarkTransporter class methodsFor: 'transporter' stamp: 'StefanMarr 4/5/2012 21:46'!
transportersForFileOutMenu
	^ { (Smalltalk at: #Transporter ifAbsent: [^#()])
			forPackage: (PackageInfo named: 'SMark') }! !

Object subclass: #SMarkWeakScalingRunnerExecutor
	instanceVariableNames: 'numInnerIterations benchmarkSelector suite runner'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'SMark'!

!SMarkWeakScalingRunnerExecutor class methodsFor: 'as yet unclassified' stamp: 'sm 5/30/2011 16:33'!
createFor: aSelector for: numIterations with: aRunner and: aSuite
	| o |
	o := self new.
	o runner: aRunner.
	o suite: aSuite.
	o innerIterations: numIterations.
	o benchmarkSelector: aSelector.
	^ ([ o run ] newProcess)! !

!SMarkWeakScalingRunnerExecutor methodsFor: 'accessing' stamp: 'sm 5/30/2011 16:20'!
benchmarkSelector: aSelector
	benchmarkSelector := aSelector! !

!SMarkWeakScalingRunnerExecutor methodsFor: 'accessing' stamp: 'sm 5/30/2011 16:20'!
innerIterations: anInt
	numInnerIterations := anInt! !

!SMarkWeakScalingRunnerExecutor methodsFor: 'benchmarking' stamp: 'sm 5/30/2011 16:22'!
run
	1 to: numInnerIterations do: [:i |
		suite perform: benchmarkSelector.].
	
	runner executorCompleted.! !

!SMarkWeakScalingRunnerExecutor methodsFor: 'accessing' stamp: 'sm 5/30/2011 16:21'!
runner: aRunner
	runner := aRunner! !

!SMarkWeakScalingRunnerExecutor methodsFor: 'accessing' stamp: 'sm 5/30/2011 16:21'!
suite: aSuite
	suite := aSuite! !
BenchmarkRunner initialize!
DBStrength initialize!
JSJsonLiteral initialize!
NBBody initialize!
RichObject initialize!
SMarkConsole initialize!
